/*****************************************************************************
 *
 * File: Macros.h
 * Version: 4.0.0
 * Date: 09/11/08
 *
 *****************************************************************************
 * Copyright (c) 2008, HART Communication Foundation, All Rights Reserved
 *****************************************************************************
 *
 * Description: HART Device Description Macro Definitions
 *
 *****************************************************************************
 */

#define USER_INTERFACE LOCAL
#define LOCAL_INTERFACE LOCAL_DISPLAY
#define ANALOG_CHANNEL ANALOG_OUTPUT
#define RANGE INPUT

#define I_NO_COMMAND_SPECIFIC_ERRORS     0
#define I_INVALID_SELECTION              2
#define I_PARAMETER_TOO_LARGE            3
#define I_PARAMETER_TOO_SMALL            4
#define I_TOO_FEW_DATA_BYTES             5
#define I_TRANSMITTER_SPECIFIC_ERROR     6
#define I_IN_WRITE_PROTECT_MODE          7
#define I_ACCESS_RESTRICTED             16


/*
 * Strings for the single definition response codes.
 */
#define S_NO_COMMAND_SPECIFIC_ERRORS [no_command_specific_errors]
#define S_INVALID_SELECTION [invalid_selection]
#define S_PARAMETER_TOO_LARGE [passed_parameter_too_large]
#define S_PARAMETER_TOO_SMALL [passed_parameter_too_small]
#define S_TOO_FEW_DATA_BYTES [too_few_data_bytes_recieved]
#define S_TRANSMITTER_SPECIFIC_ERROR [xmtr_specific_command_error]
#define S_IN_WRITE_PROTECT_MODE [in_write_protect_mode]
#define S_ACCESS_RESTRICTED [access_restricted]

#define PRIMARY         0
#define SECONDARY       1
#define TERTIARY        2
#define QUATERNARY      3

#define PV  dynamic_variables[PRIMARY]
#define SV  dynamic_variables[SECONDARY]
#define TV  dynamic_variables[TERTIARY]
#define QV  dynamic_variables[QUATERNARY]


/*
 * Common table references.
 */
#define ROSEMOUNT_MODEL_CODE(N) { N, rosemount_model_code(N) }
#define ROSEMOUNT_ANALYTICAL_MODEL_CODE(N) { N, rosemount_analytical_model_code(N) }
#define KAYRAY_MODEL_CODE(N) { N, kayray_model_code(N) }
#define MICRO_MOTION_MODEL_CODE(N) { N, micro_motion_model_code(N) }
#define UNITS_CODE(N) { N, units_code(N) }
#define TRANSFER_FUNCTION_CODE(N) { N, transfer_function_code(N) }
#define MATERIAL_CODE(N) { N, material_code(N) }
#define ALARM_SELECTION_CODE(N) { N, alarm_selection_code(N) }
#define WRITE_PROTECT_CODE(N) { N, write_protect_code(N) }
#define COMPANY_IDENTIFICATION_CODE(N) { N, company_identification_code(N) }
#define DEVICE_TYPE_CODE(N) { N, device_type_code(N) }
#define BURST_MODE_CONTROL_CODE(N) { N, burst_mode_control_code(N) }
#define PHYSICAL_SIGNALING_CODE(N) { N, physical_signaling_codes(N) }
#define FLAG_ASSIGNMENT(N) { N, flag_assignment(N) }
#define OPERATING_MODE_CODE(N) { N, operating_mode_code(N) }
#define ANALOG_OUTPUT_NUMBERS_CODE(N) { N, analog_output_numbers_code(N) }
#define LOOP_CURRENT_MODE_CODE(N)  { N, loop_current_mode_codes(N) }
#define EXTENDED_DEVICE_STATUS_CODE(N)  { N, extended_device_status_codes(N)}
#define TRIM_POINT_CODE(N)   { N, trim_point_codes(N) }
#define DEVICE_VARIABLE_FAMILY_CODE(N)  { N, device_variable_family_codes(N) }
#define DEVICE_VARIABLE_CLASSIFICATION_CODE(N)  { N, device_variable_classification_codes(N) }
#define WRITE_DEVICE_VARIABLE_CODE(N) { N, write_device_variable_codes(N) }
#define CAPTURE_MODE_CODE(N) { N, capture_mode_codes(N) }
#define LOCK_DEVICE_CODE(N) { N, lock_device_codes(N) }
#define ANALOG_CHANNEL_FLAG(N) { N, analog_channel_flags(N) }

#define PROCESS_DATA_STATUS_CODES \
        {0 , process_data_status(0)}, \
        {1 , process_data_status(1)}, \
        {2 , process_data_status(2)}, \
        {3 , process_data_status(3)}

#define LIMIT_STATUS_CODES \
        {0 , limit_status(0)}, \
        {1 , limit_status(1)}, \
        {2 , limit_status(2)}, \
        {3 , limit_status(3)}

/*
 * The reply message of an identify request is defined here as a macro because
 * the same reply is returned by command 0,11,21,73 and 75 (the identify commands).
 */

#define IDENTIFY_REPLY_7 \
    response_code, device_status, 254, device_type, \
    request_preambles, universal_revision, transmitter_revision, \
    software_revision, hardware_revision <0xf8>, physical_signaling_code <0x07>, \
    device_flags, device_id, response_preambles, max_num_device_variables, \
    config_change_counter, extended_fld_device_status, manufacturer_id, \
    private_label_distributor, device_profile

#define IDENTIFY_REPLY_6 \
    response_code, device_status, 254, manufacturer_id, device_type, \
    request_preambles, universal_revision, transmitter_revision, \
    software_revision, hardware_revision <0xf8>, physical_signaling_code <0x07>, \
    device_flags, device_id, response_preambles, max_num_device_variables, \
    config_change_counter, extended_fld_device_status

#define IDENTIFY_REPLY_5 \
    response_code, device_status, 254, manufacturer_id, device_type, \
    request_preambles, universal_revision, transmitter_revision, \
    software_revision, hardware_revision <0xf8>, physical_signaling_code <0x07>, \
    device_flags, device_id

#define ALL_TABLES \
    VARIABLE units_code; \
    VARIABLE transfer_function_code; \
    VARIABLE material_code; \
    VARIABLE alarm_selection_code; \
    VARIABLE write_protect_code; \
    VARIABLE company_identification_code; \
    VARIABLE burst_mode_control_code; \
    VARIABLE physical_signaling_codes; \
    VARIABLE flag_assignment; \
    VARIABLE operating_mode_code; \
    VARIABLE analog_output_numbers_code; \
    VARIABLE loop_current_mode_codes; \
    VARIABLE extended_device_status_codes; \
    VARIABLE lock_device_codes; \
    VARIABLE write_device_variable_codes; \
    VARIABLE device_variable_family_codes; \
    VARIABLE device_variable_classification_codes; \
    VARIABLE trim_point_codes; \
    VARIABLE capture_mode_codes; \
    VARIABLE physical_layer_type_codes; \
    VARIABLE lock_device_status_codes; \
    VARIABLE analog_channel_flags; \
    VARIABLE process_data_status; \
    VARIABLE limit_status; \
    VARIABLE device_family_status; \
    VARIABLE manufacturer_id; \
    VARIABLE private_label_distributor; \
    VARIABLE device_type; \
    VARIABLE write_protect; \
    VARIABLE physical_signaling_code; \
    VARIABLE operatingMode; \
    VARIABLE extended_fld_device_status; \
    VARIABLE loop_current_mode; \
    VARIABLE device_flags

/* 
 * Define device variable classifications 
 */
#define ALL_CLASSIFICATIONS \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(0),  \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(64), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(65), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(66), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(67), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(68), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(69), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(70), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(71), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(72), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(73), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(74), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(75), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(76), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(77), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(78), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(79), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(80), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(81), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(82), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(83), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(84), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(85), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(86), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(87), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(88), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(89), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(90), \
        DEVICE_VARIABLE_CLASSIFICATION_CODE(91)

#define ALL_FAMILIES \
        DEVICE_VARIABLE_FAMILY_CODE(4), \
        DEVICE_VARIABLE_FAMILY_CODE(5), \
        DEVICE_VARIABLE_FAMILY_CODE(6), \
        DEVICE_VARIABLE_FAMILY_CODE(7), \
        DEVICE_VARIABLE_FAMILY_CODE(8), \
        DEVICE_VARIABLE_FAMILY_CODE(9), \
        DEVICE_VARIABLE_FAMILY_CODE(10), \
        DEVICE_VARIABLE_FAMILY_CODE(250)


/*
 * Defined Colors.
 */
#define BLACK   0x000000                        
#define SILVER  0xC0C0C0                        
#define GRAY    0x808080                        
#define WHITE   0xFFFFFF                        
#define MAROON  0x800000                        
#define RED     0xFF0000                        
#define ORANGE  0xFFA500                        
#define PURPLE  0x800080                        
#define FUCHSIA 0xFF00FF                                
#define GREEN   0x008000
#define LIME    0x00FF00
#define OLIVE   0x808000
#define YELLOW  0xFFFF00
#define NAVY    0x000080
#define BLUE    0x0000FF
#define TEAL    0x008080
#define AQUA    0x00FFFF


#ifdef __GNUC__
#define LOCALVAR( x) #x     /*this is used by the get_local_var commands*/
#else
#define LOCALVAR( x) "x"
#endif

#define TRIGGER_CONTINOUS 0
#define TRIGGER_ONCHANGE 4
#define SUBDEVICE_UNINITIALIZED 0x7FFF

