/*****************************************************************************
 *
 * File: WHA_UT.ddl
 * Version: 0.1
 * Date: 07/20/10
 * Copyright (c) 2008; CodeWrights GmbH Germany
 *
 * Description:
 * Wireless HART UT
 *
 *****************************************************************************
 */
/*
 * @BugId: 21778
 */

/*
 * The manufacturer line
 */
#ifdef _PDM_
MANUFACTURER PEPPERL_FUCHS, DEVICE_TYPE 0xe2, DEVICE_REVISION 1, DD_REVISION 1
#else
MANUFACTURER PEPPERL_FUCHS, DEVICE_TYPE _WHA_UT, DEVICE_REVISION 1, DD_REVISION 1
#endif



/*
 * Include Files
 */
#include "Macros.h"
#include "Definitions_WHA.h"
#include "WHA_UT.h"
#include "Variables_UT.dd"
#include "Burst.dd"
#include "Methods.h"
#include "Menus.dd"
#include "MenusAMS.dd"
#include "commands.dd"
#include "Structures.dd"



/*****************************************************************************
 * PDM IDENTCHECK
 *****************************************************************************
 */
#ifdef _PDM_
#include "identcheck.inc"
#endif




/*****************************************************************************
 * Common Tables
 */
#ifndef _PDM_
IMPORT STANDARD _TABLES, DEVICE_REVISION 20, DD_REVISION 4
#else
IMPORT "Tabl20_4.ddl"
#endif
{
    //    ALL_TABLES;
    EVERYTHING;

    REDEFINITIONS
    {
        REDEFINE VARIABLE device_status
        {
            LABEL DCT_CONFIGURATION_CHANGED_FLAG_LABEL;
            CLASS HART & DYNAMIC & DIAGNOSTIC;
            HANDLING READ;
            TYPE BIT_ENUMERATED
            {
                { 0x01, "Primary Variable Out of Limits",     MISC},
                { 0x02, "Non-Primary Variable Out of Limits", MISC},
                { 0x04, "Loop Current Saturated",             MISC},
                { 0x08, "Loop Current Fixed",                 MISC},
                { 0x10, "More Status Available",              MORE },
                { 0x20, "Cold Start",                         MISC },
                { 0x40, "Configuration Changed",              MISC },
                { 0x80, "Device Malfunction",                 MORE }
            }
        }


        VARIABLE burst_trigger_classification
        {
            REDEFINE LABEL DCT_LABEL_BURST_TRIGGER_CLASSIFICATION_MESSAGE_N;
						REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
            REDEFINE ENUM_TABLE_21
        }

        VARIABLE burst_trigger_units
        {
            REDEFINE LABEL DCT_LABEL_BURST_TRIGGER_UNITS_MESSAGE_N;
						REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
            REDEFINE TYPE ENUMERATED
            {
                ALL_UNITS_SORTED_EXTENDED
            }
        }

        VARIABLE burst_message_trigger_mode
        {
            REDEFINE LABEL DCT_LABEL_BURST_MESSAGE_TRIGGER_MODE_MESSASGE_N;
						REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE burst_mode_select
        {
            REDEFINE LABEL DCT_LABEL_BURST_MODE_SELECT_N;
						REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
            REDEFINE TYPE ENUMERATED
            {
                BURST_MODE_CONTROL_CODE(0),  /* Off */
		        		{2, DCT_WIRELESS}
            }
        }


        VARIABLE device_type
        {
            REDEFINE TYPE ENUMERATED(2)
            {
              { 0x5de2,  DCT_WHAUT }
            }
        }


        VARIABLE manufacturer_id
        {
            REDEFINE TYPE ENUMERATED(2)
            {
               { 0x005d,  [PEPPERL_FUCHS] }
            }
        }


    }
}


#ifndef _PDM_
IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 7, DD_REVISION 2
#else
IMPORT "Univ7.ddl"
#endif
{
    // All HART devices must support all Universal Commands
    EVERYTHING;
	REDEFINITIONS{

        VARIABLE config_change_counter
        {
          REDEFINE LABEL  "Configuration Change Counter";
        }


        VARIABLE longTag
        {
            REDEFINE LABEL DCT_LONG_TAG_WHAUT;
        }

        VARIABLE tag
        {
            REDEFINE LABEL "Device Tag";
        }

        VARIABLE descriptor
        {
            REDEFINE LABEL "Descriptor";
        }

        VARIABLE date
        {
            REDEFINE LABEL DCT_DATE_WHAUT;
        }

        VARIABLE message
        {
            REDEFINE LABEL "Message";
        }

        VARIABLE polling_address
        {
            REDEFINE LABEL "Polling Address";
        }

        VARIABLE response_preambles
        {
            REDEFINE LABEL DCT_PREAMBLES_LABEL;

            REDEFINE TYPE UNSIGNED_INTEGER  //21.09.2010 CW_JO TEMP device response is 4
            {
                MIN_VALUE  4;
                MAX_VALUE  20;
            }
        }

COMMAND read_pv_sensor_info
{
    REDEFINE TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            PV.DEVICE_VARIABLE.SENSOR_SERIAL_NUMBER,
            PV.DEVICE_VARIABLE.DIGITAL_UNITS,
            dummy_Float,
            dummy_Float,
            PV.DEVICE_VARIABLE.MINIMUM_SPAN
        }
    }

}

//21.09.2010 CW_JO  TEMP->

#ifdef _PDM_
#define IDENTIFY_REPLY_7 \
    response_code, device_status, 254, device_type, \
    request_preambles, universal_revision, transmitter_revision, \
    software_revision, hardware_revision <0xf8>, physical_signaling_code <0x07>, \
    device_flags, device_id, response_preambles, max_num_device_variables, \
    config_change_counter, extended_fld_device_status, manufacturer_id, \
    private_label_distributor, device_profile


    REDEFINE COMMAND read_unique_identifier
    {
        NUMBER 0;
        OPERATION READ;
        TRANSACTION
        {
            REQUEST {}
            REPLY
            {
                IDENTIFY_REPLY_7
            }
        }
        RESPONSE_CODES
        {
            0,  SUCCESS,    [no_command_specific_errors];
            6,  MISC_ERROR, [xmtr_specific_command_error];
            16, MODE_ERROR, [access_restricted];
        }
    }
    DELETE COMMAND read_dynamic_variables_and_pv_current;

#endif

//21.09.2010 CW_JO <-





      //WHA UT specific

      REDEFINE COMMAND read_pv_output_info
      {
          NUMBER 15;
          OPERATION READ;
          TRANSACTION 0
          {
              REQUEST {}
              REPLY
              {
                  response_code, device_status,
                  250,
                  0,
                  deviceVariables[0].DIGITAL_UNITS,
                  deviceVariables[0].UPPER_SENSOR_LIMIT,
                  deviceVariables[0].LOWER_SENSOR_LIMIT,
                  deviceVariables[0].DAMPING_VALUE,
                  write_protect,
                  250,
                  0x01
              }
          }
          TRANSACTION 1
          {
              REQUEST {}
              REPLY
              {
                  response_code, device_status,
                  250,
                  0,
                  deviceVariables[0].DIGITAL_UNITS (INFO),
                  dev_var_input_1_upper_sensor_limit_SHADOW,
                  dev_var_input_1_lower_sensor_limit_SHADOW,
                  deviceVariables[0].DAMPING_VALUE (INFO),
                  write_protect (INFO),
                  250,
                  0x01
              }
          }
          RESPONSE_CODES
          {
              0,  SUCCESS,    [no_command_specific_errors];
              16, MODE_ERROR, [access_restricted];
              32, MODE_ERROR, [busy];
          }
      }

	  //WHA-UT specific END







         VARIABLE device_specific_status_1
         {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_1;
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "Transceiver: Non-Volatile Error."},
                { 0x02, "Transceiver: Supply Voltage is below Operating minimum."},
                { 0x04, "Transceiver: Error in the factory programmed settings" },
                { 0x08, "Transceiver: Join Operation Failed"},
                { 0x10, "The Transceiver has disconnected from the Network" },
                { 0x20, "The Transceiver has a connection to the gateway to send data." },
                { 0x40, "The Transceiver Booted Up." }
            }
         }


         VARIABLE device_specific_status_2
         {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_2;
            REDEFINE TYPE BIT_ENUMERATED
            {
               { 0x01, "Burst Message 0 is being published."},
               { 0x02, "Burst Message 1 is being published."},
               { 0x04, "Burst Message 2 is being published."},
               { 0x08, "Request to Network Manager for Burst Msg. 0 pending"},
               { 0x10, "Request to Network Manager for Burst Msg. 1 pending." },
               { 0x20, "Request to Network Manager for Burst Msg. 2 pending" }
            }

         }


         VARIABLE device_specific_status_3
         {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_3;
            REDEFINE TYPE BIT_ENUMERATED
            {
               { 0x01, "The Transceiver didn't sent the boot event to the Device."},
               { 0x02, "The ADC used for acquisition has problems."},
               { 0x04, "The internal Cold Junction Temperature sensor has problems." },
               { 0x08, "Microcontroller access to an invalid memory address."},
               { 0x10, "Configuration/calibration parameters corrupted" },
               { 0x20, "The checksum of the application code is wrong." },
               { 0x40, "The checksum of the bootloader code is wrong." },
               { 0x80, "Transceiver to Microcontroller Communication Timeout." }
            }
         }

         VARIABLE device_specific_status_4
         {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_4;
            REDEFINE TYPE BIT_ENUMERATED
            {
               { 0x01, "Joining Wireless Network unsuccessfull after 30 minutes."}
               { 0x02, "Transceiver connected but hasn't got alternative path."}
               { 0x04, "A low battery level has been detected ."}
               { 0x08, "Request to Network Manager for Burst Msg. 0 rejected."}
               { 0x10, "Request to Network Manager for Burst Msg. 1 rejected." }
               { 0x20, "Request to Network Manager for Burst Msg. 2 rejected." }
            }
         }

         VARIABLE device_specific_status_5
         {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_5;
            REDEFINE TYPE BIT_ENUMERATED
            {
               { 0x01, "The Device Variable 0 is in fault condition"}
               { 0x02, "The Device Variable 1 is in fault condition"}
               { 0x04, "The Device Variable 2 is in fault condition"}
            }
         }



        VARIABLE extended_fld_device_status
        {
            REDEFINE LABEL DCT_DEVICE_EXTENDED_STATUS;
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01,  "Maintenance Required"},
                { 0x02,  "Device Variable Alert"},
                { 0x04,  "Critical Power Failure"}
            }
        }

        VARIABLE standardized_status_0
        {
            REDEFINE LABEL DCT_DEVICE_STANDARDIZED_STATUS_0;
            REDEFINE TYPE BIT_ENUMERATED
            {
            { 0x01, "Simulation active", "The device has been put to simulation mode. The transmitted measurement value is a simulated one" },
            { 0x02, "Non-Volatile Memory Defect" },
            { 0x04, "Volatile Memory Defect" },
            { 0x08, "Watchdog Reset Executed" },
            { 0x10, "Voltage Conditions Out of Range" },
            { 0x20, "Environmental Conditions Out of Range" },
            { 0x40, "Electronic Defect" }
            }
        }

        VARIABLE standardized_status_3
        {
            REDEFINE LABEL DCT_DEVICE_STANDARDIZED_STATUS_3;
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "Capacity Denied"},
                { 0x04, "Bandwidth Allocation Pending"}
            }
        }



        // redefine this to get the unit codes.
        COMMAND read_dynamic_variables_and_pv_current
        {
            REDEFINE TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status,
                    PV.DAQ.ANALOG_VALUE,
                    PV.DEVICE_VARIABLE.DIGITAL_UNITS <0xFF>, PV.DEVICE_VARIABLE.DIGITAL_VALUE,
                    SV.DEVICE_VARIABLE.DIGITAL_UNITS <0xFF>, SV.DEVICE_VARIABLE.DIGITAL_VALUE,
                    TV.DEVICE_VARIABLE.DIGITAL_UNITS <0xFF>, TV.DEVICE_VARIABLE.DIGITAL_VALUE,
                    QV.DEVICE_VARIABLE.DIGITAL_UNITS <0xFF>, QV.DEVICE_VARIABLE.DIGITAL_VALUE
                 }
            }
        }

        /*****************************************************************************
        * Command 48
        */
        REDEFINE COMMAND read_additional_device_status
        {
            NUMBER 48;
            OPERATION READ;
            TRANSACTION 0
            {
                REQUEST
                { }
                REPLY
                {
                    response_code, device_status,
                    dummy_variable_1           <0xF0>,
                    device_specific_status_0_mote   <0x0F>,

                    dummy_variable_1            <0x80>,
                    device_specific_status_1_40 <0x40>,
                    device_specific_status_1_20 <0x20>,
                    device_specific_status_1_10 <0x10>,
                    device_specific_status_1_8  <0x08>,
                    device_specific_status_1_4  <0x04>,
                    device_specific_status_1_2  <0x02>,
                    device_specific_status_1_1  <0x01>,

                    dummy_variable_1            <0xC0>,
                    device_specific_status_2_20 <0x20>,
                    device_specific_status_2_10 <0x10>,
                    device_specific_status_2_8  <0x08>,
                    device_specific_status_2_4  <0x04>,
                    device_specific_status_2_2  <0x02>,
                    device_specific_status_2_1  <0x01>,

                    device_specific_status_3_80 <0x80>,
                    device_specific_status_3_40 <0x40>,
                    device_specific_status_3_20 <0x20>,
                    device_specific_status_3_10 <0x10>,
                    device_specific_status_3_8  <0x08>,
                    device_specific_status_3_4  <0x04>,
                    device_specific_status_3_2  <0x02>,
                    device_specific_status_3_1  <0x01>,

                    dummy_variable_1            <0xC0>,
                    device_specific_status_4_20 <0x20>,
                    device_specific_status_4_10 <0x10>,
                    device_specific_status_4_8  <0x08>,
                    device_specific_status_4_4  <0x04>,
                    device_specific_status_4_2  <0x02>,
                    device_specific_status_4_1  <0x01>,

                    dummy_variable_1            <0xF8>,
                    device_specific_status_5_4  <0x04>,
                    device_specific_status_5_2  <0x02>,
                    device_specific_status_5_1  <0x01>,

                    dummy_variable_1             <0xF8>,
                    extended_fld_device_status_4 <0x04>,
                    extended_fld_device_status_2 <0x02>,
                    extended_fld_device_status_1 <0x01>,

                    0x00,

                    dummy_variable_1 <0x80>,
                    standardized_status_0_40 <0x40>,
                    standardized_status_0_20 <0x20>,
                    standardized_status_0_10 <0x10>,
                    standardized_status_0_8  <0x08>,
                    standardized_status_0_4  <0x04>,
                    standardized_status_0_2  <0x02>,
                    standardized_status_0_1  <0x01>,

                    standardized_status_1,
                    analog_channel_saturated1,
                    standardized_status_2,

                    dummy_variable_1              <0xF8>,
                    standardized_status_3_4       <0x04>,
                    dummy_variable_1              <0x02>,
                    standardized_status_3_1       <0x01>
                }
            }

            TRANSACTION 1
            {
                REQUEST
                {
                }
                REPLY
                {
                    response_code, device_status,
                    device_specific_status_0,
                    device_specific_status_1,
                    device_specific_status_2,
                    device_specific_status_3,
                    device_specific_status_4,
                    device_specific_status_5,
                    extended_fld_device_status, 0x00,
                    standardized_status_0, standardized_status_1, analog_channel_saturated1,
                    standardized_status_2, standardized_status_3
                }
            }

            TRANSACTION 2
            {
                REQUEST
                {
                }
                REPLY
                {
                    response_code, device_status,
                    dummy_variable_1        <0x80>,
                    config_changed_flag      <0x40>,
                    cold_start_flag          <0x20>,
                    dummy_variable_1         <0x1F>,
                    device_specific_status_1,
                    device_specific_status_2,
                    device_specific_status_3,
                    device_specific_status_4,
                    device_specific_status_5,
                    extended_fld_device_status, 0x00,
                    standardized_status_0, standardized_status_1, analog_channel_saturated1,
                    standardized_status_2, standardized_status_3
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,            [no_command_specific_errors];
                5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
                6,  MISC_ERROR,         [xmtr_specific_command_error];
                8,  MISC_WARNING,       [update_in_progress];
                14, MISC_WARNING,       "Status bytes mismatch";
                16, MODE_ERROR,         [access_restricted];
            }
        }



#define CMD_9_DEVICE_VAR_RESPONSE_BLOCK(num) \
                    num, \
                    deviceVariables[num].CLASSIFICATION, \
                    deviceVariables[num].DIGITAL_UNITS <0xFF>, \
                    deviceVariables[num].DIGITAL_VALUE, \
                    deviceVariables[num].DATA_QUALITY <0xC0>, \
                    deviceVariables[num].LIMIT_STATUS <0x30>, \
                    deviceVariables[num].DEVICE_FAMILY_STATUS <0x0F>



        REDEFINE COMMAND read_device_variables_and_status
        {
            NUMBER 9;
            OPERATION READ;

            // To minimize read cycles by AMS Device Diagnostics you have to pack all vars shown
            // in Diagnostics in one transaction and be shure these are not included in another transaction.
            // If all device vars are in a single transaction only you can prevent flickerering in AMS and
            // you prevent useless multi cmd 9 transactions.

            // TRANSACTION 0 includes all transactions shown in Device Diagnostics.
            TRANSACTION 0
            {
                REQUEST
                {
                    0,
                    1,
                    2,
                    3,
                    4,
                    243
                }
                REPLY
                {
                    response_code, device_status,
                    extended_fld_device_status,

                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(0),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(1),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(2),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(3),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(4),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(243),

                    time_stamp
                }
            }


            TRANSACTION 1
            {
                REQUEST
                {
                    243,
                    244,
                    245,
                    0,
                    1,
                    2
                }
                REPLY
                {
                    response_code, device_status,
                    extended_fld_device_status,

                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(243),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(244),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(245),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(0),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(1),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(2),

                    time_stamp
                }
            }

            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                8,  MISC_WARNING,     [update_failure];
                14, MISC_WARNING,     "Dynamic Variables Returned for Device Variables"; // CPK - add this to standard dictionary
                16, MODE_ERROR,       [access_restricted];
                30, MISC_WARNING,     "Command Response Truncated"; // CPK  - add this to standard dictionary
            }
        }
    }
}


/*****************************************************************************
 * PV; when the primary variable is not mappable we can import the PV
 * standard DD to take care of the dynamic variable array; loop current
 * and ranging functions
 */
#ifndef _PDM_
IMPORT STANDARD _PV, DEVICE_REVISION 1, DD_REVISION 1
#else
IMPORT "PV1_1.ddl"
#endif
{
    EVERYTHING;


    REDEFINITIONS
    {

        COLLECTION OF COLLECTION primary_variable
        {
            MEMBERS
            {
            	REDEFINE DEVICE_VARIABLE, PVVariables[primary_variable_code];
            }
        }

        COLLECTION OF COLLECTION secondary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE, PVVariables[secondary_variable_code];
            }
        }

        COLLECTION OF COLLECTION tertiary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE, deviceVariables[tertiary_variable_code];
            }
        }

        COLLECTION OF COLLECTION quaternary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE, deviceVariables[quaternary_variable_code];
            }
        }

        ARRAY OF COLLECTION dynamic_variables
        {
            REDEFINE ELEMENTS
            {
                PRIMARY,        primary_variable;
                SECONDARY,      secondary_variable;
                TERTIARY,       tertiary_variable;
                QUATERNARY,     quaternary_variable;
            }
        }


        VARIABLE loopCurrent
        {
            REDEFINE LABEL "Loop Current";
        }

         REDEFINE COLLECTION OF VARIABLE analog_io
         {
             LABEL [analog_input];
             HELP [analog_input_block_help];
             MEMBERS
             {
                 /* Analog Information */
                 ANALOG_VALUE,           loopCurrent,        "Loop Current",         [analog_value_help];
                 ALARM_CODE,             loop_alarm_code,    [alarm_type],           [alarm_code_help];
                 ANALOG_CHANNEL_FLAGS,   loop_flags,         [analog_channel_flags], [analog_channel_flags_help];
             }
         }

    }
}


/*****************************************************************************
 * Common Practice Commands
 */
#ifndef _PDM_
IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 9, DD_REVISION 2
#else
IMPORT "Common9.ddl"
#endif
{
    COMMAND read_device_variable_information; //27.07.2010 CW_JO
    COMMAND write_device_variable;                  // Command  9

    COMMAND write_pv_damping_value;                 // Command 34
    COMMAND write_pv_range_values;                  // Command 35
    COMMAND set_pv_upper_range_value;               // Command 36
    COMMAND set_pv_lower_range_value;               // Command 37

    COMMAND enter_exit_fixed_pv_current_mode;       // Command 40
    COMMAND perform_self_test;                      // Command 41
    COMMAND perform_device_reset;                   // Command 42
    COMMAND set_pv_zero;                            // Command 43
    COMMAND write_pv_units;                         // Command 44
    COMMAND trim_pv_current_dac_zero;               // Command 45
    COMMAND trim_pv_current_dac_gain;               // Command 46

    COMMAND read_device_variable_assignments;       // Command 50
    COMMAND write_device_variable_assignments;      // Command 51

    COMMAND write_device_variable_units;            // COMMAND 53
    COMMAND write_number_of_response_preambles;     // Command 59

    COMMAND read_real_time_clock;                   // Command 90

    COMMAND write_burst_period;						// Command 103
    COMMAND write_burst_trigger;					// Command 104
    COMMAND read_burst_mode_configuration;			// Command 105
    COMMAND write_burst_device_variables;			// Command 107
    COMMAND write_burst_command_number;				// Command 108
    COMMAND burst_mode_control;						// Command 109




    METHOD applied_rerange;
    METHOD zero_trim;
    METHOD return_to_normal;
    METHOD warning_message;
    METHOD device_self_test;
    METHOD device_master_reset;
    METHOD transmitter_loop_test;
    METHOD leave_fixed_current_mode;
    METHOD transmitter_dac_trim;
    METHOD lock_unlock_device;

	VARIABLE total_number_burst_messages;

    VARIABLE device_variable_code;

    VARIABLE max_io_cards;
    VARIABLE max_channels_per_io_card;
    VARIABLE max_sub_dev_per_channel;

    VARIABLE max_dr;

    VARIABLE primary_variable_code;
    VARIABLE secondary_variable_code;
    VARIABLE tertiary_variable_code;
    VARIABLE quaternary_variable_code;

    VARIABLE current_date;
    VARIABLE current_time;
    VARIABLE last_clock_date;
    VARIABLE last_clock_time;

    VARIABLE burst_message_number;

#ifndef _PDM_
    ARRAY burst_variable_codes;
    VARIABLE burst_variable_code;
#endif

    VARIABLE subdev_IO_card;
    VARIABLE subdev_channel;
    VARIABLE subdev_device_id;
    VARIABLE subdev_universal_command_revision;
    VARIABLE subdev_long_tag;
    VARIABLE stx_sent_to_subdev;
    VARIABLE ack_received_from_subdev;
    VARIABLE back_received_from_subdev;

    COLLECTION subdev_identity;
    COLLECTION subdev_stats;
    COLLECTION sub_device;





    VARIABLE constZero;
    VARIABLE constOne;
    VARIABLE number_events_supported;

    VARIABLE time_of_first_unack_event;
    VARIABLE event_notification_retry_time_event;
    VARIABLE max_update_time_event;
    VARIABLE event_debounce_interval_event;
    VARIABLE device_specific_status_0_mask;
    VARIABLE device_specific_status_1_mask;
    VARIABLE device_specific_status_2_mask;
    VARIABLE device_specific_status_3_mask;
    VARIABLE device_specific_status_4_mask;
    VARIABLE device_specific_status_5_mask;
    VARIABLE device_specific_status_14_mask;
    VARIABLE device_specific_status_15_mask;
    VARIABLE device_specific_status_16_mask;
    VARIABLE device_specific_status_17_mask;
    VARIABLE device_specific_status_18_mask;
    VARIABLE device_specific_status_19_mask;
    VARIABLE device_specific_status_20_mask;
    VARIABLE device_specific_status_21_mask;
    VARIABLE device_specific_status_22_mask;
    VARIABLE device_specific_status_23_mask;
    VARIABLE device_specific_status_24_mask;
    VARIABLE device_specific_status_0_latched_value;
    VARIABLE device_specific_status_1_latched_value;
    VARIABLE device_specific_status_2_latched_value;
    VARIABLE device_specific_status_3_latched_value;
    VARIABLE device_specific_status_4_latched_value;
    VARIABLE device_specific_status_5_latched_value;
    VARIABLE device_specific_status_14_latched_value;
    VARIABLE device_specific_status_15_latched_value;
    VARIABLE device_specific_status_16_latched_value;
    VARIABLE device_specific_status_17_latched_value;
    VARIABLE device_specific_status_18_latched_value;
    VARIABLE device_specific_status_19_latched_value;
    VARIABLE device_specific_status_20_latched_value;
    VARIABLE device_specific_status_21_latched_value;
    VARIABLE device_specific_status_22_latched_value;
    VARIABLE device_specific_status_23_latched_value;
    VARIABLE device_specific_status_24_latched_value;
    VARIABLE config_change_counter_latched_value;


    VARIABLE burst_command_number;
    VARIABLE update_period;
    VARIABLE max_update_period;
    VARIABLE trigger_level;
    VARIABLE subdevice_missing;


    REDEFINITIONS
    {
        VARIABLE burst_message_number
        {
            REDEFINE TYPE INDEX compat_burst_messages;
        }

      //CMD 103
        COMMAND write_burst_period
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    burst_message_number (INFO, INDEX),
                    compat_burst_messages[burst_message_number].UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].MAX_UPDATE_PERIOD
                }
                REPLY
                {
                    response_code, device_status,
                    burst_message_number (INFO, INDEX),
                    compat_burst_messages[burst_message_number].UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].MAX_UPDATE_PERIOD
                }
            }

        }
        COMMAND write_pv_range_values
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    deviceVariables[0].DIGITAL_UNITS, deviceVariables[0].UPPER_SENSOR_LIMIT,deviceVariables[0].LOWER_SENSOR_LIMIT
                }
                REPLY
                {
                    response_code, device_status,
                    deviceVariables[0].DIGITAL_UNITS, deviceVariables[0].UPPER_SENSOR_LIMIT,deviceVariables[0].LOWER_SENSOR_LIMIT
                }
            }

        }


      //CMD 104
        COMMAND write_burst_trigger
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    burst_message_number (INFO, INDEX),
                    compat_burst_messages[burst_message_number].BURST_TRIGGER_MODE,
                    compat_burst_messages[burst_message_number].BURST_CLASSIFICATION,
                    compat_burst_messages[burst_message_number].BURST_UNIT,
                    compat_burst_messages[burst_message_number].TRIGGER_LEVEL
                }
                REPLY
                {
                    response_code, device_status,
                    burst_message_number (INFO, INDEX),
                    compat_burst_messages[burst_message_number].BURST_TRIGGER_MODE,
                    compat_burst_messages[burst_message_number].BURST_CLASSIFICATION,
                    compat_burst_messages[burst_message_number].BURST_UNIT,
                    compat_burst_messages[burst_message_number].TRIGGER_LEVEL
                }
            }
        }


        REDEFINE COMMAND write_burst_device_variables
        {
            NUMBER 107;
            OPERATION WRITE;
            TRANSACTION 1
            {
                REQUEST
                {
                     compat_burst_vars_0_0,
                     compat_burst_vars_0_1,
                     compat_burst_vars_0_2,
                     compat_burst_vars_0_3,
                     compat_burst_vars_0_4,
                     compat_burst_vars_0_5,
                     compat_burst_vars_0_6,
                     compat_burst_vars_0_7,
                     0
                }

                REPLY
                {
                     response_code, device_status,
                     compat_burst_vars_0_0,
                     compat_burst_vars_0_1,
                     compat_burst_vars_0_2,
                     compat_burst_vars_0_3,
                     compat_burst_vars_0_4,
                     compat_burst_vars_0_5,
                     compat_burst_vars_0_6,
                     compat_burst_vars_0_7,
                     0
                }
            }
            TRANSACTION 2
            {
                REQUEST
                {
                     compat_burst_vars_1_0,
                     compat_burst_vars_1_1,
                     compat_burst_vars_1_2,
                     compat_burst_vars_1_3,
                     compat_burst_vars_1_4,
                     compat_burst_vars_1_5,
                     compat_burst_vars_1_6,
                     compat_burst_vars_1_7,
                     1
                }

                REPLY
                {
                     response_code, device_status,
                     compat_burst_vars_1_0,
                     compat_burst_vars_1_1,
                     compat_burst_vars_1_2,
                     compat_burst_vars_1_3,
                     compat_burst_vars_1_4,
                     compat_burst_vars_1_5,
                     compat_burst_vars_1_6,
                     compat_burst_vars_1_7,
                     1
                }
            }
            TRANSACTION 3
            {
                REQUEST
                {
                     compat_burst_vars_2_0,
                     compat_burst_vars_2_1,
                     compat_burst_vars_2_2,
                     compat_burst_vars_2_3,
                     compat_burst_vars_2_4,
                     compat_burst_vars_2_5,
                     compat_burst_vars_2_6,
                     compat_burst_vars_2_7,
                     2
                }

                REPLY
                {
                     response_code, device_status,
                     compat_burst_vars_2_0,
                     compat_burst_vars_2_1,
                     compat_burst_vars_2_2,
                     compat_burst_vars_2_3,
                     compat_burst_vars_2_4,
                     compat_burst_vars_2_5,
                     compat_burst_vars_2_6,
                     compat_burst_vars_2_7,
                     2
                }
            }

            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, DCT_BurstVarsWriteErrorSWA;
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                8,  MISC_WARNING,     "Burst condition conflict";
                9,  DATA_ENTRY_ERROR, "Invalid burst message";
            }
        }


        REDEFINE COMMAND write_burst_command_number
        {
            NUMBER 108;
            OPERATION WRITE;

            TRANSACTION 1
            {
               REQUEST
                {
                    compat_burst_messages[burst_message_number].COMMAND_NUMBER_ENUM, //09.11.2010 B23023
                    burst_message_number (INFO, INDEX)
                }

                REPLY
                {
                    response_code, device_status,
					compat_burst_messages[burst_message_number].COMMAND_NUMBER_ENUM, //09.11.2010 B23023
                    burst_message_number (INFO, INDEX)
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, DCT_BurstCmdWriteErrorSWA;
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                8,  MISC_WARNING,     "Burst condition conflict";
                9,  DATA_ENTRY_ERROR, "Invalid burst message";
            }
        }

        //CMD 109
        COMMAND burst_mode_control
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    compat_burst_messages[burst_message_number].BURST_MODE_SELECT,
                    burst_message_number (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    compat_burst_messages[burst_message_number].BURST_MODE_SELECT,
                    burst_message_number (INFO, INDEX)
                }
            }

		    REDEFINE RESPONSE_CODES
		    {
		        0,  SUCCESS,          [no_command_specific_errors];
		        2,  DATA_ENTRY_ERROR, [invalid_selection];
		        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
		        6,  MISC_ERROR,       [xmtr_specific_command_error];
		        7,  MODE_ERROR,       [in_write_protect_mode];
		        8,  MISC_WARNING,     "Update period increased";
		        9,  MODE_ERROR,       "Insufficient bandwidth";
		        16, MODE_ERROR,       [access_restricted];
		        32, MODE_ERROR,         "Burst mode will be enabled, this process can take a minute.  Continue trying? ";
		        33, MODE_ERROR,       [delayed_response_initiated];
		        34, MODE_ERROR,       [delayed_response_running];
		        35, MODE_ERROR,       [delayed_response_dead];
		        36, MODE_ERROR,       [delayed_response_conflict];
		    }


        }


         REDEFINE COMMAND read_device_variable_information
         {
             NUMBER 54;
             OPERATION READ;
             TRANSACTION
             {
                 REQUEST
                 {
                     device_variable_code (INFO,INDEX)
                 }
                 REPLY
                 {
                     response_code, device_status,
                     device_variable_code (INFO,INDEX),
                     deviceVariables[device_variable_code].SENSOR_SERIAL_NUMBER,
                     dummy_variable_1,
                     /*deviceVariables[device_variable_code].DIGITAL_UNITS <0xFF>,*/
                     //deviceVariables[device_variable_code].UPPER_SENSOR_LIMIT,
                     //deviceVariables[device_variable_code].LOWER_SENSOR_LIMIT,
                     dummy_Float,
                     dummy_Float,
                     deviceVariables[device_variable_code].DAMPING_VALUE,
                     deviceVariables[device_variable_code].MINIMUM_SPAN,
                     deviceVariables[device_variable_code].CLASSIFICATION,
                     deviceVariables[device_variable_code].DEVICE_FAMILY,
                     deviceVariables[device_variable_code].UPDATE_TIME_PERIOD
                 }
             }
             RESPONSE_CODES
             {
                 0,  SUCCESS,          [no_command_specific_errors];
                 2,  DATA_ENTRY_ERROR, [invalid_selection];
                 5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                 6,  MISC_ERROR,       [xmtr_specific_command_error];
                 16, MODE_ERROR,       [access_restricted];
                 32, MODE_ERROR,       [busy];
             }
         }


			REDEFINE COMMAND write_device_variable_assignments
			{
			    NUMBER 51;
			    OPERATION WRITE;
			    TRANSACTION
			    {
			        REQUEST
			        {
			            primary_variable_code,
                     secondary_variable_code,
			            02,
			            250
			        }
			        REPLY
			        {
			            response_code, device_status,
			            primary_variable_code,
                     secondary_variable_code,
			            tertiary_variable_code,
                     quaternary_variable_code
			        }
			    }
			    RESPONSE_CODES
			    {
			        0,  SUCCESS,          [no_command_specific_errors];
			        2,  DATA_ENTRY_ERROR, [invalid_selection];
			        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
			        6,  MISC_ERROR,       [xmtr_specific_command_error];
			        7,  MODE_ERROR,       [in_write_protect_mode];
			        16, MODE_ERROR,       [access_restricted];
			        32, MODE_ERROR,       [busy];
			    }
			}


        VARIABLE trigger_level
        {
            REDEFINE LABEL DCT_LABEL_TRIGGER_LEVEL_MESSAGE_N;
						REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE update_period
        {
            REDEFINE LABEL DCT_LABEL_UPDATE_PERIOD_N;
						REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
#ifdef _PDM_
				    REDEFINE TYPE TIME_VALUE
				    {
				        DISPLAY_FORMAT "5.3f" ;
				        TIME_SCALE SECONDS ;
				    }
#endif
        }

        VARIABLE max_update_period
        {
            REDEFINE LABEL DCT_LABEL_MAX_UPDATE_PERIOD_N;
				    REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
#ifdef _PDM_
				    REDEFINE TYPE TIME_VALUE
				    {
				        DISPLAY_FORMAT "5.3f" ;
				        TIME_SCALE SECONDS ;
				    }
#endif
        }

        VARIABLE time_of_first_unack_event
        {
            REDEFINE LABEL DCT_TIME_OF_FIRST_UNACK_EVENT_SWA;
#ifdef _PDM_
				    REDEFINE TYPE TIME_VALUE;
#endif
        }

				VARIABLE event_notification_retry_time_event
				{
						REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
				}

        VARIABLE max_update_time_event
        {
            REDEFINE LABEL DCT_MAX_UPDATE_TIME_EVENT_SWA;
#ifdef _PDM_
						REDEFINE TYPE TIME_VALUE;
#endif
        }

#ifndef _PDM_
#define DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(num) \
    REDEFINE VARIABLE device_specific_status_##num##_mask \
        { \
            LABEL DCT_DEVICE_SPECIFIC_STATUS_##num##_MASK_1_SWA; \
            HELP DCT_DEVICE_SPECIFIC_STATUS_HELP; \
            CLASS LOCAL; \
            HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; } \
            TYPE BIT_ENUMERATED \
            { \
                { 0x01, DCT_BIT_0 }, { 0x02, DCT_BIT_1 }, { 0x04, DCT_BIT_2 }, \
                { 0x08, DCT_BIT_3 }, { 0x10, DCT_BIT_4 }, { 0x20, DCT_BIT_5 }, \
                { 0x40, DCT_BIT_6 }, { 0x80, DCT_BIT_7 } \
            } \
            DEFAULT_VALUE 0; \
        }

        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(0)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(1)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(2)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(3)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(4)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(5)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(14)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(15)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(16)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(17)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(18)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(19)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(20)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(21)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(22)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(23)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(24)
#else
#define DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(num_mask,num_MASK_1_SWA) \
    REDEFINE VARIABLE device_specific_status_##num_mask \
        { \
            LABEL DCT_DEVICE_SPECIFIC_STATUS_##num_MASK_1_SWA ; \
            HELP DCT_DEVICE_SPECIFIC_STATUS_HELP; \
            CLASS LOCAL; \
            HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; } \
            TYPE BIT_ENUMERATED \
            { \
                { 0x01, DCT_BIT_0 }, { 0x02, DCT_BIT_1 }, { 0x04, DCT_BIT_2 }, \
                { 0x08, DCT_BIT_3 }, { 0x10, DCT_BIT_4 }, { 0x20, DCT_BIT_5 }, \
                { 0x40, DCT_BIT_6 }, { 0x80, DCT_BIT_7 } \
            } \
            DEFAULT_VALUE 0; \
        }

        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(0_mask,0_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(1_mask,1_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(2_mask,2_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(3_mask,3_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(4_mask,4_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(5_mask,5_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(14_mask,14_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(15_mask,15_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(16_mask,16_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(17_mask,17_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(18_mask,18_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(19_mask,19_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(20_mask,20_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(21_mask,21_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(22_mask,22_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(23_mask,23_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(24_mask,24_MASK_1_SWA)
#endif








        REDEFINE COMMAND read_burst_mode_configuration
        {
				    NUMBER 105;
				    OPERATION READ;
            TRANSACTION 0
            {
                REQUEST
                {
                    burst_message_number (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    compat_burst_messages[burst_message_number].BURST_MODE_SELECT,
                    0x1F,
                    compat_burst_vars[burst_message_number][0],
                    compat_burst_vars[burst_message_number][1],
                    compat_burst_vars[burst_message_number][2],
                    compat_burst_vars[burst_message_number][3],
                    compat_burst_vars[burst_message_number][4],
                    compat_burst_vars[burst_message_number][5],
                    compat_burst_vars[burst_message_number][6],
                    compat_burst_vars[burst_message_number][7],
                    burst_message_number (INFO, INDEX),
                    total_number_burst_messages,
                    compat_burst_messages[burst_message_number].COMMAND_NUMBER_ENUM,
                    compat_burst_messages[burst_message_number].UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].MAX_UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].BURST_TRIGGER_MODE,
                    compat_burst_messages[burst_message_number].BURST_CLASSIFICATION,
                    compat_burst_messages[burst_message_number].BURST_UNIT,
                    compat_burst_messages[burst_message_number].TRIGGER_LEVEL

                }
            }
            TRANSACTION 1  //same as 0 TODO: must be checked
            {
                REQUEST
                {
                    burst_message_number (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    compat_burst_messages[burst_message_number].BURST_MODE_SELECT,
                    0x1F,
                    compat_burst_vars[burst_message_number][0],
                    compat_burst_vars[burst_message_number][1],
                    compat_burst_vars[burst_message_number][2],
                    compat_burst_vars[burst_message_number][3],
                    compat_burst_vars[burst_message_number][4],
                    compat_burst_vars[burst_message_number][5],
                    compat_burst_vars[burst_message_number][6],
                    compat_burst_vars[burst_message_number][7],
                    burst_message_number (INFO, INDEX),
                    total_number_burst_messages,
                    compat_burst_messages[burst_message_number].COMMAND_NUMBER_ENUM,
                    compat_burst_messages[burst_message_number].UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].MAX_UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].BURST_TRIGGER_MODE,
                    compat_burst_messages[burst_message_number].BURST_CLASSIFICATION,
                    compat_burst_messages[burst_message_number].BURST_UNIT,
                    compat_burst_messages[burst_message_number].TRIGGER_LEVEL

                }
            }
				    RESPONSE_CODES
				    {
				        0,  SUCCESS,          [no_command_specific_errors];
				        6,  MISC_ERROR,       [xmtr_specific_command_error];
				        9,  MISC_ERROR,       "Invalid burst message";
				        32, MODE_ERROR,       [busy];
				    }
        }







		VARIABLE primary_variable_code
		{
    		REDEFINE TYPE INDEX PVVariables;
    		REDEFINE LABEL DCT_PRIMARY_VARIABLE_CODE_SWA;
				REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
		}



		VARIABLE master_mode_code       //Standard dictionary
		{
		    REDEFINE TYPE ENUMERATED
		    {
		        { 0, "Secondary Master", "Secondary Master" },
		        { 1, "Primary Master",   "Primary Master"   }

		    }
		}


		VARIABLE secondary_variable_code {
		   REDEFINE TYPE INDEX PVVariables;
			REDEFINE LABEL DCT_SECONDARY_VARIABLE_CODE_SWA;
			REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
	    REDEFINE POST_EDIT_ACTIONS
	    {
	        warning_message,
	        method_read_non_primary_variable_code
	    }
		}

		VARIABLE tertiary_variable_code {
			REDEFINE LABEL DCT_TERTIARY_VARIABLE_CODE_SWA;
			REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
	    REDEFINE POST_EDIT_ACTIONS
	    {
	        warning_message,
	        method_read_non_primary_variable_code
	    }
		}

      VARIABLE quaternary_variable_code {
         REDEFINE LABEL DCT_QUARTERNARY_VARIABLE_CODE_SWA;
         REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
         REDEFINE POST_EDIT_ACTIONS
         {
	        warning_message,
            method_read_non_primary_variable_code
         }
      }

      VARIABLE current_date
      {
         REDEFINE LABEL DCT_CURRENT_DATE_SWA;
      }

		VARIABLE current_time
		{
         REDEFINE LABEL DCT_CURRENT_TIME_SWA;
#ifdef _PDM_
		    REDEFINE TYPE UNSIGNED_INTEGER(4)
		    {
		        SCALING_FACTOR 0.00003125;
		    }
		    CONSTANT_UNIT "s";
#else
         REDEFINE TYPE TIME_VALUE;
#endif
		}
   }
}

//TODO mus be checked with _PDM_
METHOD method_read_non_primary_variable_code
{
#ifndef _PDM_
    VALIDITY FALSE;
#endif
    LABEL DCT_METHOD_READ_NON_PV_CODE;
    DEFINITION
		{
			char status[STATUS_SIZE];
			long var_id[4];
			int secondary_var;
			int tertiary_var;
			int quaternary_var;

			IGNORE_NO_DEVICE();
			IGNORE_ALL_DEVICE_STATUS();
			IGNORE_ALL_RESPONSE_CODES();
			XMTR_IGNORE_ALL_DEVICE_STATUS();
			XMTR_IGNORE_ALL_RESPONSE_CODES();
			XMTR_IGNORE_NO_DEVICE();

			secondary_var = ivar_value(secondary_variable_code);
			tertiary_var = ivar_value(tertiary_variable_code);
			quaternary_var = ivar_value(quaternary_variable_code);

			send_command_trans(9,0);
			send_command_trans(9,1);

			iassign(secondary_variable_code,0);
			iassign(secondary_variable_code,secondary_var);
			iassign(tertiary_variable_code,0);
			iassign(tertiary_variable_code,tertiary_var);
			iassign(quaternary_variable_code,0);
			iassign(quaternary_variable_code,quaternary_var);

			save_values();
    }
}




// **********************************************************************************
// Wireless
// **********************************************************************************

#ifdef _PDM_
IMPORT "Wireless_wha_ut.ddl"
#else
IMPORT STANDARD _WIRELESS, DEVICE_REVISION 1, DD_REVISION 2
#endif
{
   EVERYTHING;
   REDEFINITIONS
   {
   DELETE COMMAND  read_network_access_mode;
   DELETE COMMAND  write_network_access_mode;
   DELETE VARIABLE network_access_mode;

   DELETE COMMAND  read_network_tag;
   DELETE COMMAND  write_network_tag;
   DELETE VARIABLE network_tag;


   #ifdef _PDM_
   DELETE COMMAND read_join_status;
   #endif

		REDEFINE COMMAND write_join_key
		{
		    NUMBER 768;
		    OPERATION COMMAND;
		    TRANSACTION
		    {
		        REQUEST
		        {
		             join_key_1, join_key_2, join_key_3, join_key_4
		        }
		        REPLY
		        {
		            response_code, device_status,
		            join_key_1, join_key_2, join_key_3, join_key_4
		        }
		    }
		    RESPONSE_CODES
		    {
		        0,  SUCCESS,          [no_command_specific_errors];
		        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
		        6,  MISC_ERROR,       [xmtr_specific_command_error];
		        7,  MODE_ERROR,       [in_write_protect_mode];
		        16, MODE_ERROR,       [access_restricted];
		        32, MODE_ERROR,       [busy];
		        33, MODE_ERROR,       [delayed_response_initiated];
		        34, MODE_ERROR,       [delayed_response_running];
		        35, MODE_ERROR,       [delayed_response_dead];
		        36, MODE_ERROR,       [delayed_response_conflict];
		        65, DATA_ENTRY_ERROR, "Key change failed";
		    }
		}


      REDEFINE COMMAND write_network_id
      {
         NUMBER 773;
         OPERATION WRITE;
         TRANSACTION
         {
            REQUEST
            {
               network_id
            }
            REPLY
            {
                    response_code, device_status, network_id
            }
         }
         RESPONSE_CODES
         {
                0,  SUCCESS,          [no_command_specific_errors];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                8,  MODE_ERROR,       DCT_NETWORK_ID_CHANGE_PENDING;
                16, MODE_ERROR,       [access_restricted];
                32, MODE_ERROR,       [busy];
                33, MODE_ERROR,       [delayed_response_initiated];
                34, MODE_ERROR,       [delayed_response_running];
                35, MODE_ERROR,       [delayed_response_dead];
                36, MODE_ERROR,       [delayed_response_conflict];
                65, DATA_ENTRY_ERROR, DCT_INVALID_NETWORK_ID;
         }
      }

      REDEFINE VARIABLE radio_power_output
      {
         HANDLING
         IF
         (
            (lock_device_status_code__local > 0) ||
#ifdef _PDM_
         JOIN_STATUS_EQU_x079F
#else
         (join_status == 0x079F)
#endif
         )
         {
            READ;
         }
         ELSE
         {
            READ&WRITE;
         }
         TYPE ENUMERATED
         {
#ifdef _PDM_
                    {0, "0 "}
                    {10, "10 "}
#else
                    {0, "0 dBm"}
                    {10, "10 dBm"}
#endif
         }

         LABEL DCT_RADIO_POWER_LABEL;
         HELP "only 0 and 10 allowed";
         CLASS HART;
         #ifdef _PDM_
         CONSTANT_UNIT "dBm";
         #endif
      }

//******************************************************************************

    VARIABLE network_id
		{
		    REDEFINE TYPE UNSIGNED_INTEGER (2);
				REDEFINE LABEL DCT_NETWORK_ID_SWA;
				REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
		}

		VARIABLE join_shed_time
		{
            REDEFINE LABEL DCT_JOIN_SHED_TIME_LABEL;
						REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
#if __TOKVER__ >= 800
            REDEFINE TYPE TIME_VALUE
            {
                DISPLAY_FORMAT ".0f" ;
                EDIT_FORMAT ".0f";
                TIME_SCALE SECONDS ;
            }
#else
            REDEFINE TYPE UNSIGNED_INTEGER(4)
            {
                SCALING_FACTOR 0.00003125;
            }
            REDEFINE CONSTANT_UNIT "s";
#endif
#ifdef _PDM_
				REDEFINE DEFAULT_VALUE 76800000;
#endif
        }

		VARIABLE join_mode
		{
			REDEFINE CLASS DYNAMIC;
            REDEFINE LABEL DCT_JOIN_MODE_SWA;
						REDEFINE HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
            REDEFINE TYPE ENUMERATED
            {
                { 0, DCT_DONT_JOIN },
                { 1, join_mode_codes(1) },
                { 2, DCT_JOIN_ON_POWERUP_OR_RESET }
            }
		}

		REDEFINE VARIABLE join_key_1
		{
		    LABEL DCT_JOIN_KEY_1_SWA;
            CLASS LOCAL;
            //TODO HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
            HANDLING WRITE; //22.07.2010 CW_JO
            TYPE UNSIGNED_INTEGER (4)
            {
                DISPLAY_FORMAT "08X";
                EDIT_FORMAT "08X";
            }
            DEFAULT_VALUE 0;
		}

        REDEFINE VARIABLE join_key_2
		{
		    LABEL DCT_JOIN_KEY_2_SWA;
            CLASS LOCAL;
            //TODO HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
            HANDLING WRITE; //22.07.2010 CW_JO
            TYPE UNSIGNED_INTEGER (4)
            {
                DISPLAY_FORMAT "08X";
                EDIT_FORMAT "08X";
            }
            DEFAULT_VALUE 0;
		}

		REDEFINE VARIABLE join_key_3
		{
		    LABEL DCT_JOIN_KEY_3_SWA;
            CLASS LOCAL;
            //TODO HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
            HANDLING WRITE; //22.07.2010 CW_JO
            TYPE UNSIGNED_INTEGER (4)
            {
                DISPLAY_FORMAT "08X";
                EDIT_FORMAT "08X";
            }
            DEFAULT_VALUE 0;
		}

		REDEFINE VARIABLE join_key_4
		{
		    LABEL DCT_JOIN_KEY_4_SWA;
            CLASS LOCAL;
            //TODO HANDLING IF ( lock_device_status_code__local > 0 ) { READ; } ELSE { READ&WRITE; }
            HANDLING WRITE; //22.07.2010 CW_JO
            TYPE UNSIGNED_INTEGER (4)
            {
                DISPLAY_FORMAT "08X";
                EDIT_FORMAT "08X";
            }
            DEFAULT_VALUE 0;
		}

//********************************************
//*******CHANGE FOR #19123:           ********
//********************************************

        REDEFINE VARIABLE shed_time
        {
            LABEL DCT_SHED_TIME_SWA;
            HELP  "Shed Time";
            CLASS DEVICE;
            HANDLING WRITE;
            TYPE TIME_VALUE
            {
                DISPLAY_FORMAT "5.0f" ;
                EDIT_FORMAT "5.0f";
                TIME_SCALE SECONDS ;
            }
			DEFAULT_VALUE 57600000;
		}

        // Variable Label redefinitions - Wireless
		VARIABLE wireless_mode
		{
			REDEFINE LABEL DCT_WIRELESS_OPERATION_MODE_LABEL;
		}

        VARIABLE join_process_status_codes
        {
            REDEFINE TYPE ENUMERATED (2)
            {
                { 0x0001, DCT_JOIN_STATE_NETWORK_PACKETS_HEARD, DCT_JOIN_STATE_NETWORK_PACKETS_HEARD_HELP },
                { 0x0002, DCT_JOIN_STATE_ASN_ACQUIRED },
                { 0x0004, DCT_JOIN_STATE_SYNCHRONIZED_TO_SLOT_TIME },
                { 0x0008, DCT_JOIN_STATE_ADVERTISEMENT_HEARD },
                { 0x0010, DCT_JOIN_STATE_JOIN_REQUESTED, DCT_JOIN_STATE_JOIN_REQUESTED_HELP },
                { 0x0020, DCT_JOIN_STATE_JOIN_RETRYING , DCT_JOIN_STATE_JOIN_RETRYING_HELP  },
                { 0x0040, DCT_JOIN_STATE_JOIN_FAILED , DCT_JOIN_STATE_JOIN_FAILED_HELP },
                { 0x0080, DCT_JOIN_STATE_AUTHENTICATED, DCT_JOIN_STATE_AUTHENTICATED_HELP },
                { 0x0100, DCT_JOIN_STATE_NETWORK_JOINED, DCT_JOIN_STATE_NETWORK_JOINED_HELP },
                { 0x0200, DCT_JOIN_STATE_NEGOTIATING_NETWORK_PROPERTIES, DCT_JOIN_STATE_NEGOTIATING_NETWORK_PROPERTIES_HELP },
                { 0x0400, DCT_JOIN_STATE_NORMAL_OPERATION_COMMENCING, DCT_JOIN_STATE_NORMAL_OPERATION_COMMENCING_HELP }
            }
        }

				VARIABLE wireless_operation_mode_codes      //Standard dictionary
				{
				    REDEFINE TYPE ENUMERATED
				    {
				        { 0, DCT_WHAUT_OPERATION_MODE_IDLE },
				        { 1, DCT_WHAUT_OPERATION_MODE_ACTIVE_SEARCH, "Looking for neighbors." },
				        { 2, DCT_WHAUT_OPERATION_MODE_NEGOTIATING },
				        { 3, DCT_WHAUT_OPERATION_MODE_QUARANTINED },
				        { 4, DCT_WHAUT_OPERATION_MODE_OPERATIONAL },
				        { 5, DCT_WHAUT_OPERATION_MODE_SUSPENDED },
				        { 6, DCT_WHAUT_OPERATION_MODE_DEEP_SLEEP, "Ultra-low power" }
				    }
				}

		VARIABLE join_status
		{
			REDEFINE LABEL DCT_JOIN_STATUS_SWA;
		}

        REDEFINE VARIABLE neighbor_count
        {
            LABEL DCT_NEIGHBOR_COUNT_SWA;
            HELP "Number of available neighbors.";
            CLASS DEVICE & DYNAMIC;
            HANDLING READ;
            TYPE ENUMERATED (1)
            {
            { 0, "0" },
            { 1, "0" },
            { 2, "1" },
            { 3, "2" },
            { 4, "3" },
            { 5, "4" },
            { 6, "5" },
            { 7, "6" },
            { 8, "7" },
            { 9, "8" },
            { 10, "9" }
            }
        }

		VARIABLE advertisements
		{
			REDEFINE LABEL DCT_NUMBER_OF_ADVERTISING_PACKETS_RECEIVED_LABEL;
		}

		VARIABLE join_attempts
		{
			REDEFINE LABEL DCT_NUMBER_OF_JOIN_ATTEMPTS_LABEL;
		}


		REDEFINE VARIABLE advertised_neighbors_number
		{
				LABEL DCT_ADVERTISED_NEIGHBORS_NUMBER_SWA;
				CLASS LOCAL;
	    	HELP DCT_ADVERTISED_NEIGHBORS_NUMBER_SWA_HELP;
	    	HANDLING READ;
	    	TYPE UNSIGNED_INTEGER (1);
	    	DEFAULT_VALUE 0;
		}

		VARIABLE device_nickname_address
		{
			REDEFINE LABEL DCT_DEVICE_NICKNAME_ADDRESS_SWA;
		}
		// Wireless
    }
}








/*
   Starts joining the network.
   To do this it writes the following valiables:
   - join_key_1 .. join_key_4
   - network_id (will be changed by the user)
   - join_shed_time (will be changed by the user)
   - join_mode
*/


METHOD join_network
{
	LABEL DCT_JOIN_NETWORK;
	HELP  DCT_JOIN_NETWORK_HELP;
	DEFINITION
	{
		char status[STATUS_SIZE];
		char disp_string1[160];
		char disp_string2[160];
		long var_id[3];
		int sel;
		int slen1;
		int slen2;
		long temp;
		int i;

		slen1 = 160;
		slen2 = 160;

		IGNORE_ALL_DEVICE_STATUS();
		IGNORE_ALL_RESPONSE_CODES();
		XMTR_IGNORE_ALL_DEVICE_STATUS();
		XMTR_IGNORE_ALL_RESPONSE_CODES();


		add_abort_method(method_join_network_aborted);
		//Ask if method should be started.
		disp_string1= DCT_WRITE_JOIN_VALUES;
		get_dictionary_string(yes_no, disp_string2, slen2);
		sel= select_from_list(disp_string1, var_id, disp_string2);
		if (sel == 0) /* 0 == yes == start joining */
		{
			GET_DEV_VAR_VALUE(DCT_ENTER_JOINKEY_1, join_key_1);
			GET_DEV_VAR_VALUE(DCT_ENTER_JOINKEY_2, join_key_2);
			GET_DEV_VAR_VALUE(DCT_ENTER_JOINKEY_3, join_key_3);
			GET_DEV_VAR_VALUE(DCT_ENTER_JOINKEY_4, join_key_4);

			//Check if joinkey is set properly
			temp = lvar_value(join_key_1);
			temp += lvar_value(join_key_2);
			temp += lvar_value(join_key_3);
			temp += lvar_value(join_key_4);

			if (temp != 0)
			{
				// Join Key is valid and is written to device
				send_command(768);
				lassign(join_key_1, 0);
				lassign(join_key_2, 0);
				lassign(join_key_3, 0);
				lassign(join_key_4, 0);
				save_values();
			}
			else
			{
				// Join key is not valid. Check if joining must be aborted.
				send_command(48);
				temp = lvar_value(device_specific_status_0);
				if ((temp & 0x08) != 0)
				{    // no join key set error flag
					disp_string1= DCT_JOIN_KEY_EMPTY;
					ACKNOWLEDGE(disp_string1);
					process_abort();
				}
			}
			//ask for new network id and write network id
			GET_DEV_VAR_VALUE(DCT_ENTER_NETWORK_ID, network_id);
			send(773, status);
			if( (status[STATUS_RESPONSE_CODE] != 0) && (status[STATUS_RESPONSE_CODE] != 8) )
			{
				ACKNOWLEDGE(DCT_WRITING_NETWORK_ID_FAILED);
				process_abort();
			}
			if(status[STATUS_RESPONSE_CODE] == 8)
			{
				ACKNOWLEDGE(DCT_NETWORK_ID_CHANGE_PENDING);
			}
			//ask for new join_shed_time
			GET_DEV_VAR_VALUE(DCT_ENTER_JOIN_SHED_TIME, join_shed_time);

			// Set Join mode to "Join Now" if its set to "Attempt to join..."
			temp = lvar_value(join_mode);
			if (temp > 1)
			{
				lassign(join_mode, 1);
			}
			remove_abort_method(method_join_network_aborted);
			//write join_shed_time + join_mode
			send(771, status);

			if(status[STATUS_RESPONSE_CODE] == 0)
			{
				disp_string1= DCT_NORMAL_OPERATION_COMMENCING;
			}
			else
			{
				disp_string1= DCT_JOIN_FAILED;
			}
			ACKNOWLEDGE(disp_string1);
		}
		else
		{
			ACKNOWLEDGE(DCT_JOIN_NETWORK_NO_ACTION_TAKEN);
		}
	}
}

METHOD method_join_network_aborted
{
	LABEL DCT_JOIN_NETWORK_ABORTED;
	DEFINITION
	{
		ACKNOWLEDGE(DCT_JOIN_NETWORK_NO_ACTION_TAKEN);
	}
}

/*
Requests active advertising with time span given by user.
*/
METHOD method_request_active_advertising
{
   LABEL DCT_REQUEST_ACTIVE_ADVERTISING;
   HELP  DCT_REQUEST_ACTIVE_ADVERTISING_HELP;
   DEFINITION
   {
      char status[STATUS_SIZE];
      char disp_string1[260];
      char disp_string2[60];
      long var_id[4];
      int sel;
      int slen1;
      int slen2;
      slen1 = 260;
      slen2 = 60;

      IGNORE_ALL_DEVICE_STATUS();
      IGNORE_ALL_RESPONSE_CODES();
      XMTR_IGNORE_ALL_DEVICE_STATUS();
      XMTR_IGNORE_ALL_RESPONSE_CODES();


#ifndef _PDM_
      if (join_status < 128)
#else
      if (JOIN_STATUS_LSS_128)
#endif
      {
         ACKNOWLEDGE(DCT_DEVICE_IS_NOT_JOINED);
         process_abort();
      }

      if( lvar_value(lock_device_status_code__local) > 0) // check device is locked
      {
         ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
         process_abort();
      }

      disp_string1= DCT_REQUEST_ACTIVE_ADVERTISING_ACTIVATE;
        get_dictionary_string(yes_no, disp_string2, slen2);
        sel= select_from_list(disp_string1, var_id, disp_string2);
      if(sel == 0) // 0 == yes
      {

#ifndef _PDM_
			lassign(shed_time, 32000 * 1800); // => 30minutes, Bugzilla 20577
#else
			lassign(shed_time, 1800); // PDM expects the seconds here.
#endif

         disp_string1= DCT_ENTER_SHED_TIME;
            GET_DEV_VAR_VALUE(disp_string1, shed_time); // time type doesn't work with sdc 4.01 beta
         disp_string1= DCT_WAIT_ACTIVE_ADVERTISING_COMPL;
            
            send(770, status);
            DELAY(2, disp_string1);
            while(status[STATUS_RESPONSE_CODE] != 0)
            {
                send(770, status);
            }
            

         if(status[STATUS_RESPONSE_CODE] == 0)
         {
            var_id[0]= VARID(shed_time);
            var_id[1]= VARID(advertising_period);
            var_id[2]= VARID(advertised_neighbors_number);
            var_id[3]= 0;
            disp_string1= DCT_REQUEST_ACTIVE_ADVERTISING_STATUS;
                acknowledge(disp_string1, var_id);
         }
      }
      if(status[STATUS_RESPONSE_CODE] != 0)
      {
            display_response_status(770, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
      }
      process_abort() ;
   }
}


/*
  Resets the config changed counter in the device
  */
//TODO METHOD method_reset_config_change_counter


// execute device reset and wait for device to recover
METHOD method_reset
{
    LABEL DCT_METHOD_RESET;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[300];
        char disp_string2[60];
        long var_id[4];
        int slen1;
        int slen2;
				int sel;
				int rp;

        IGNORE_NO_DEVICE();
        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_NO_DEVICE();

        slen1= 300;
        slen2= 60;
        disp_string1= DCT_LABEL_CONFIRM_RESET;
				get_dictionary_string(yes_no, disp_string2, slen2);
				sel= select_from_list(disp_string1, var_id, disp_string2);
				if (sel == 0) /* 0 == yes == execute reset */
				{
				    send(42, status);
				    rp=status[STATUS_RESPONSE_CODE];


				    if(status[STATUS_RESPONSE_CODE] != 0)
				    {
				        display_response_status(42, status[STATUS_RESPONSE_CODE]);
				        get_dictionary_string(abort_last_error, disp_string1, slen1);
				        DELAY(2, disp_string1);
				    }
				    else
				    {
				        disp_string1= DCT_LABEL_EXECUTE_RESET;
				        DELAY(30, disp_string1);
				        send(0, status);
				    }
				}
    }
}

// start self test execution
METHOD method_self_test
{
    LABEL DCT_METHOD_SELF_TEST;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[300];
        int slen1;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        slen1= 300;
        send(41, status);
        if(status[STATUS_RESPONSE_CODE] != 0)
        {
            display_response_status(41, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
        }
        else
        {
            disp_string1= DCT_LABEL_SELF_TEST_STARTED;
            ACKNOWLEDGE(disp_string1);
        }
    }
}



//////////////////////////////////////////////////////////////////////////////
//
// BURST MODE
//
//////////////////////////////////////////////////////////////////////////////
//TODO must be checked with PDM
METHOD method_burst_mode_sync_offline
{
	LABEL DCT_METHOD_BURST_MODE_SYNC_OFFLINE;
#ifndef _PDM_
    VALIDITY FALSE;
#endif
    DEFINITION
	{
#ifdef _PDM_
			vassign(compat_burst_messages[0].BURST_MODE_SELECT,compat_burst_mode_local_0);
			vassign(compat_burst_messages[1].BURST_MODE_SELECT,compat_burst_mode_local_1);
			vassign(compat_burst_messages[2].BURST_MODE_SELECT,compat_burst_mode_local_2);
#endif
			save_values();
	}
}



ARRAY OF COLLECTION health_states
{
        ELEMENTS
        {
            0,    health_state0, "Health State 0";
            1,    health_state1, "Health State 1";
            2,    health_state2, "Health State 2";
            3,    health_state3, "Health State 3";
            4,    health_state4, "Health State 4";
            5,    health_state5, "Health State 5";
            6,    health_state6, "Health State 6";
            7,    health_state7, "Health State 7";
            8,    health_state8, "Health State 8";
        }
}

//COLLECTION MACRO

#define HEALTHSTATE(num) COLLECTION OF VARIABLE health_state##num{\
    MEMBERS\
    {\
         NICKNAME_OF_NEIGHBOR,                     nickname_of_neighbor_##num,                           DCT_NICKNAME_OF_NEIGHBOR;\
         NEIGHBOR_FLAGS,                           neighbor_flags_##num,                                 DCT_NEIGHBOR_FLAGS;\
         MEAN_RSL,                                 mean_rsl_##num,                                       DCT_MEAN_RSL;\
         PACKETS_SEND_TO_NEIGHBOR,                 number_of_packets_transmitted_to_this_neighbor_##num, DCT_NUMBER_OF_PACKETS_TRANSMITTED_TO_THIS_NEIGHBOR;\
         FAILED_TRANSMITS,                         number_of_failed_transmits_##num,                     DCT_NUMBER_OF_FAILED_TRANSMITS;\
         PACKETS_RECEIVED_FROM_NEIGHBOR,           number_of_packets_received_from_this_neighbor_##num,  DCT_NUMBER_OF_PACKETS_RECEIVED_FROM_THIS_NEIGHBOR;\
   }\
}

HEALTHSTATE(0)
HEALTHSTATE(1)
HEALTHSTATE(2)
HEALTHSTATE(3)
HEALTHSTATE(4)
HEALTHSTATE(5)
HEALTHSTATE(6)
HEALTHSTATE(7)
HEALTHSTATE(8)





//*********************************************************************
//*********************************************************************
//             END OF NEIGHBOR AREA
//*********************************************************************
//*********************************************************************


//////////////////////////////////////////////////////////////////////////////
//
// device variables array
//
//////////////////////////////////////////////////////////////////////////////



ARRAY OF COLLECTION deviceVariables
{
    ELEMENTS
    {
        /*
         * If you ever change the deviceVariables elements list do not
         * forget to change in METHOD method_simulation the iteration!
         */

         0, dev_var_input_1, DCT_DEV_VAR_TEMPERATURE;
         1, dev_var_input_2, DCT_DEV_VAR_TEMPERATURE;
         2, dev_var_cold_junction_temperature, DCT_DEV_VAR_TEMPERATURE;
         3, dev_var_battery_voltage, DCT_DEV_VAR_BATTERY_VOLTAGE;
         4, dev_var_percent_of_battery_capacity, DCT_DEV_VAR_CONSUMED_ENERGY;
         243, dev_var_estimated_lifetime, DCT_DEV_VAR_ESTIMATED_LIFETIME;
         244, dev_var_percent_range, DCT_DEV_VAR_PERCENT_RANGE;
         245, dev_var_loop_current, DCT_DEV_VAR_LOOP_CURRENT;
    }
}




ARRAY OF COLLECTION PVVariables
{
   ELEMENTS
   {
      0, dev_var_input_1, DCT_DEV_VAR_TEMPERATURE_1;
      1, dev_var_input_2, DCT_DEV_VAR_TEMPERATURE_2;
/*
      2, dev_var_cold_junction_temperature, DCT_DEV_VAR_TEMPERATURE;
      3, dev_var_battery_voltage, DCT_DEV_VAR_BATTERY_VOLTAGE;
      4, dev_var_percent_of_battery_capacity, DCT_DEV_VAR_CONSUMED_ENERGY;
      243, dev_var_estimated_lifetime, DCT_DEV_VAR_ESTIMATED_LIFETIME;
      */
   }
}


ARRAY OF COLLECTION sensorVariables
{
   ELEMENTS
   {
      0, dev_var_sensor_1;
      1, dev_var_sensor_2;
   }
}




UNIT dev_var_input_1_UnitRelation
{
    dev_var_input_1_Unit:
      dev_var_input_1_Value,
      dev_var_input_1_lower_sensor_limit,
      dev_var_input_1_upper_sensor_limit
}

UNIT dev_var_input_2_UnitRelation
{
    dev_var_input_2_Unit:
      dev_var_input_2_Value,
      dev_var_input_2_lower_sensor_limit,
      dev_var_input_2_upper_sensor_limit
}


UNIT dev_var_cold_junction_temperature_unit_relation
{
   dev_var_cold_junction_temperature_Unit:
   dev_var_cold_junction_temperature_Value,
   dev_var_cold_junction_temperature_upper_sensor_limit,
   dev_var_cold_junction_temperature_lower_sensor_limit
}

UNIT dev_var_estimated_lifetime_unit_relation
{
   dev_var_estimated_lifetime_Unit:
   dev_var_estimated_lifetime_Value,
   dev_var_estimated_lifetime_upper_sensor_limit,
   dev_var_estimated_lifetime_lower_sensor_limit
}


UNIT dev_var_battery_voltage_unit_relation
{
   dev_var_battery_voltage_Unit:
   dev_var_battery_voltage_Value,
   dev_var_battery_voltage_upper_sensor_limit,
   dev_var_battery_voltage_lower_sensor_limit
}

UNIT dev_var_percent_of_battery_capacity_unit_relation
{
   dev_var_percent_of_battery_capacity_Unit:
   dev_var_percent_of_battery_capacity_Value,
   dev_var_percent_of_battery_capacity_upper_sensor_limit,
   dev_var_percent_of_battery_capacity_lower_sensor_limit

}

UNIT dev_var_percent_range_UnitRelation
{
    dev_var_percent_range_Unit:
      dev_var_percent_range_Value,
      dev_var_percent_range_upper_sensor_limit,
      dev_var_percent_range_lower_sensor_limit
}

UNIT dev_var_loop_current_UnitRelation
{
    dev_var_loop_current_Unit:
      dev_var_loop_current_Value,
      dev_var_loop_current_upper_sensor_limit,
      dev_var_loop_current_lower_sensor_limit
}




REFRESH dev_var_input_1_lower_sensor_limit_REFRESH
{
    dev_var_input_1_lower_sensor_limit:
   	dev_var_input_1_lower_sensor_limit_SHADOW

}

REFRESH dev_var_input_1_upper_sensor_limit_REFRESH
{
    dev_var_input_1_upper_sensor_limit:
   	dev_var_input_1_upper_sensor_limit_SHADOW

}
REFRESH dev_var_temperature_1_number_of_wires_REFRESH
{
    dev_var_temperature_1_number_of_wires:
   	dev_var_temperature_1_number_of_wires_shadow,
   	dev_var_input_1_probe_type

}
REFRESH dev_var_input_1_probe_type_REFRESH
{
   dev_var_input_1_probe_type:
      dev_var_input_1_lower_sensor_limit,
      dev_var_input_1_upper_sensor_limit,
      dev_var_input_1_lower_sensor_limit_SHADOW,
      dev_var_input_1_upper_sensor_limit_SHADOW,
      dev_var_input_1_Value,
      dev_var_input_1_Unit,
      dev_var_temperature_1_number_of_wires,
      dev_var_temperature_2_2_wire_compensation_status,
      dev_var_temperature_2_2_wire_compensation_value,
      dev_var_temperature_2_2_wire_burnoutmode,
      dev_var_temperature_2_cjc_type
}


REFRESH dev_var_input_2_probe_type_refresh
{
   dev_var_input_2_probe_type:
      dev_var_input_2_lower_sensor_limit,
      dev_var_input_2_upper_sensor_limit,
      dev_var_input_2_Value,
      dev_var_input_2_Unit,
      dev_var_temperature_1_number_of_wires,
      dev_var_temperature_2_2_wire_compensation_status,
      dev_var_temperature_2_2_wire_compensation_value,
      dev_var_temperature_2_2_wire_burnoutmode,
      dev_var_temperature_2_cjc_type
}




REFRESH dev_var_temperature_1_cjc_type_relation
{
   dev_var_temperature_1_cjc_type,
   dev_var_input_1_probe_type:
      dev_var_input_2_probe_type
}


REFRESH dev_var_temperature_1_number_of_wires_refresh
{
   dev_var_temperature_1_number_of_wires:
      dev_var_input_1_probe_type
}



//TEMP
#ifdef _PDM_
MENU download_variables
{
	LABEL DCT_DOWNLOAD_MENU;
    PRE_READ_ACTIONS
    {
		method_read_join_status
        method_Read_Temperature_Configuration

	}
	ITEMS
	{
    dev_var_input_1_Unit,
    dev_var_input_2_Unit,
    dev_var_cold_junction_temperature_Unit,
    dev_var_battery_voltage_Unit,
    dev_var_percent_of_battery_capacity_Unit,
    dev_var_estimated_lifetime_Unit,
    dev_var_percent_range_Unit,
    dev_var_loop_current_Unit,
    dev_var_percent_range_Unit,
    lock_device_status_code,

    IF( TEMPERATURE_1_CJC_AVAILABLE )
    {
      dev_var_temperature_1_cjc_unit
    }

    IF( TEMPERATURE_2_CJC_AVAILABLE )
    {
      dev_var_temperature_2_cjc_unit
    }

		compat_burst_trig_units_0,
		compat_burst_trig_units_1,
		compat_burst_trig_units_2,

    UpdateRate,
    dev_var_input_1_Value,
    dev_var_input_1_sensor_serial_number,
    dev_var_input_1_upper_sensor_limit,
    dev_var_input_1_lower_sensor_limit,
    dev_var_input_1_damping_value,
    dev_var_input_1_minimum_span,
    dev_var_input_1_classification,
    dev_var_input_1_device_family,
    dev_var_input_1_update_time_period,
    dev_var_input_1_data_quality,
    dev_var_input_1_device_family_status,
    dev_var_input_1_limit_status,

    dev_var_input_2_Value,
    dev_var_input_2_sensor_serial_number,
    dev_var_input_2_upper_sensor_limit,
    dev_var_input_2_lower_sensor_limit,
    dev_var_input_2_damping_value,
    dev_var_input_2_minimum_span,
    dev_var_input_2_classification,
    dev_var_input_2_device_family,
    dev_var_input_2_update_time_period,
    dev_var_input_2_data_quality,
    dev_var_input_2_device_family_status,
    dev_var_input_2_limit_status,

    dev_var_cold_junction_temperature_Value,
    dev_var_cold_junction_temperature_sensor_serial_number,
    dev_var_cold_junction_temperature_upper_sensor_limit,
    dev_var_cold_junction_temperature_lower_sensor_limit,
    dev_var_cold_junction_temperature_damping_value,
    dev_var_cold_junction_temperature_minimum_span,
    dev_var_cold_junction_temperature_classification,
    dev_var_cold_junction_temperature_device_family,
    dev_var_cold_junction_temperature_update_time_period,
    dev_var_cold_junction_temperature_data_quality,
    dev_var_cold_junction_temperature_device_family_status,
    dev_var_cold_junction_temperature_limit_status,

    dev_var_battery_voltage_Value,
    dev_var_battery_voltage_sensor_serial_number,
    dev_var_battery_voltage_upper_sensor_limit,
    dev_var_battery_voltage_lower_sensor_limit,
    dev_var_battery_voltage_damping_value,
    dev_var_battery_voltage_minimum_span,
    dev_var_battery_voltage_classification,
    dev_var_battery_voltage_device_family,
    dev_var_battery_voltage_update_time_period,
    dev_var_battery_voltage_data_quality,
    dev_var_battery_voltage_device_family_status,
    dev_var_battery_voltage_limit_status,

    dev_var_percent_of_battery_capacity_Value,
    dev_var_percent_of_battery_capacity_sensor_serial_number,
    dev_var_percent_of_battery_capacity_upper_sensor_limit,
    dev_var_percent_of_battery_capacity_lower_sensor_limit,
    dev_var_percent_of_battery_capacity_damping_value,
    dev_var_percent_of_battery_capacity_minimum_span,
    dev_var_percent_of_battery_capacity_classification,
    dev_var_percent_of_battery_capacity_device_family,
    dev_var_percent_of_battery_capacity_update_time_period,
    dev_var_percent_of_battery_capacity_data_quality,
    dev_var_percent_of_battery_capacity_device_family_status,
    dev_var_percent_of_battery_capacity_limit_status,

      dev_var_estimated_lifetime_Value,
      dev_var_estimated_lifetime_sensor_serial_number,
      dev_var_estimated_lifetime_upper_sensor_limit,
      dev_var_estimated_lifetime_lower_sensor_limit,
      dev_var_estimated_lifetime_damping_value,
      dev_var_estimated_lifetime_minimum_span,
      dev_var_estimated_lifetime_classification,
      dev_var_estimated_lifetime_device_family,
      dev_var_estimated_lifetime_update_time_period,
      dev_var_estimated_lifetime_data_quality,
      dev_var_estimated_lifetime_device_family_status,
      dev_var_estimated_lifetime_limit_status,


      dev_var_percent_range_Value,
      dev_var_percent_range_sensor_serial_number,
      dev_var_percent_range_upper_sensor_limit,
      dev_var_percent_range_lower_sensor_limit,
      dev_var_percent_range_damping_value,
      dev_var_percent_range_minimum_span,
      dev_var_percent_range_classification,
      dev_var_percent_range_device_family,
      dev_var_percent_range_update_time_period,
      dev_var_percent_range_data_quality,
      dev_var_percent_range_device_family_status,
      dev_var_percent_range_limit_status,

      dev_var_loop_current_Value,
      dev_var_loop_current_sensor_serial_number,
      dev_var_loop_current_upper_sensor_limit,
      dev_var_loop_current_lower_sensor_limit,
      dev_var_loop_current_damping_value,
      dev_var_loop_current_minimum_span,
      dev_var_loop_current_classification,
      dev_var_loop_current_device_family,
      dev_var_loop_current_update_time_period,
      dev_var_loop_current_data_quality,
      dev_var_loop_current_device_family_status,
      dev_var_loop_current_limit_status,

      dev_var_input_1_probe_type,
      dev_var_temperature_1_number_of_wires,
      dev_var_temperature_1_2_wire_compensation_status,
      dev_var_temperature_1_2_wire_compensation_value,
      dev_var_temperature_1_2_wire_burnoutmode,
      dev_var_temperature_1_standard


      IF( TEMPERATURE_1_CJC_AVAILABLE )
      {
        dev_var_temperature_1_probe_connection,
   		  dev_var_temperature_1_cjc_type,
      	dev_var_temperature_1_cjc_value
      }

      dev_var_temperature_1_family_status_07,
   	dev_var_temperature_1_family_status_08,
   	dev_var_temperature_1_family_status_30,
   	dev_var_temperature_1_family_status_C0,
   	dev_var_temperature_1_family_status_0,

      dev_var_input_2_probe_type,
      dev_var_temperature_2_2_wire_compensation_status,
      dev_var_temperature_2_2_wire_compensation_value,
      dev_var_temperature_2_2_wire_burnoutmode,
      dev_var_temperature_2_standard


    IF( TEMPERATURE_2_CJC_AVAILABLE )
    {
      dev_var_temperature_2_probe_connection,
      dev_var_temperature_2_cjc_type,
      dev_var_temperature_2_cjc_value
    }

      dev_var_temperature_2_family_status_07,
   	dev_var_temperature_2_family_status_08,
   	dev_var_temperature_2_family_status_30,
   	dev_var_temperature_2_family_status_C0,
   	dev_var_temperature_2_family_status_0,



		//MENU Identification_menu
		longTag,
		descriptor,
		date,
        loopCurrent,
		message,
		polling_address,
		//MENU DeviceVariableMapping_menu
		primary_variable_code,
		secondary_variable_code,
		tertiary_variable_code,
		quaternary_variable_code,

		//MENU root_menu
      //Secondary, tertiary and quaternary values
		dev_var_battery_voltage_Value,//read only
      dev_var_estimated_lifetime_Value,//read only
		loopCurrent,
		//MENU device_info
		tag,
		serial_number,//read only
		//MENU WirelessCommunication_menu
		network_id,
		wireless_mode,//read only
		radio_power_output,
		join_shed_time,
		join_mode,
		join_status_byte_hi,//read only
		join_status_byte_lo,//read only

    IF(JOIN_STATUS_GEQ_128)  //20.09.2010 CW_JO
    {
      total_number_of_neighbors//read only
    }
		advertised_neighbors_number,
		advertisements,//read only
		join_attempts,//read only



		//MENU Burst(parIndex)_menu
		compat_burst_mode_0,
		compat_update_per_0,
		compat_max_update_per_0,
		compat_burst_trig_mode_0,
		compat_burst_trig_class_0,
		compat_burst_trig_level_0,
		compat_burst_cmd_0,
		compat_burst_mode_1,


		compat_update_per_1,
		compat_max_update_per_1,
		compat_burst_trig_mode_1,
		compat_burst_trig_class_1,
		compat_burst_trig_level_1,
		compat_burst_cmd_1,
		compat_burst_mode_2,

		compat_update_per_2,
		compat_max_update_per_2,
		compat_burst_trig_mode_2,
		compat_burst_trig_class_2,
		compat_burst_trig_level_2,
		compat_burst_cmd_2,

		compat_burst_vars_0_0,
		compat_burst_vars_0_1,
		compat_burst_vars_0_2,
		compat_burst_vars_0_3,
		compat_burst_vars_0_4,
		compat_burst_vars_0_5,
		compat_burst_vars_0_6,
		compat_burst_vars_0_7,
		compat_burst_vars_1_0,
		compat_burst_vars_1_1,
		compat_burst_vars_1_2,
		compat_burst_vars_1_3,
		compat_burst_vars_1_4,
		compat_burst_vars_1_5,
		compat_burst_vars_1_6,
		compat_burst_vars_1_7,
		compat_burst_vars_2_0,
		compat_burst_vars_2_1,
		compat_burst_vars_2_2,
		compat_burst_vars_2_3,
		compat_burst_vars_2_4,
		compat_burst_vars_2_5,
		compat_burst_vars_2_6,
		compat_burst_vars_2_7,


		//MENU DiagIdentification_menu
		current_date,//read only
		current_time,//read only


		transmitter_revision//read only

		software_revision,//read only
		hardware_revision,//read only
		universal_revision,//read only

		//MENU DiagWirelessCommunication_menu
		device_nickname_address,//read only


		//MENU HART_menu
		config_change_counter,//read only
		device_status,//read only
		config_changed_flag,//read only
    cold_start_flag,//read only
    device_specific_status_0_mote,//read only



		device_specific_status_0,//read only
		device_specific_status_1,//read only
		device_specific_status_2,//read only
		device_specific_status_3,//read only
		device_specific_status_4,//read only
		extended_fld_device_status,//read only
		standardized_status_0,//read only
		analog_channel_saturated1,//read only
		standardized_status_2,//read only
		standardized_status_3//read only

		//MENU DeviceHealthState0
    IF (JOIN_STATUS_GEQ_128) {
			nickname_of_neighbor_0,//read only
			mean_rsl_0,//read only
			number_of_packets_transmitted_to_this_neighbor_0,//read only
			number_of_failed_transmits_0,//read only
			number_of_packets_received_from_this_neighbor_0,//read only
			nickname_of_neighbor_1,//read only
			mean_rsl_1,//read only
			number_of_packets_transmitted_to_this_neighbor_1,//read only
			number_of_failed_transmits_1,//read only
			number_of_packets_received_from_this_neighbor_1,//read only
			nickname_of_neighbor_2,//read only
			mean_rsl_2,//read only
			number_of_packets_transmitted_to_this_neighbor_2,//read only
			number_of_failed_transmits_2,//read only
			number_of_packets_received_from_this_neighbor_2,//read only
			nickname_of_neighbor_3,//read only
			mean_rsl_3,//read only
			number_of_packets_transmitted_to_this_neighbor_3,//read only
			number_of_failed_transmits_3,//read only
			number_of_packets_received_from_this_neighbor_3,//read only
			nickname_of_neighbor_4,//read only
			mean_rsl_4,//read only
			number_of_packets_transmitted_to_this_neighbor_4,//read only
			number_of_failed_transmits_4,//read only
			number_of_packets_received_from_this_neighbor_4,//read only
			nickname_of_neighbor_5,//read only
			mean_rsl_5,//read only
			number_of_packets_transmitted_to_this_neighbor_5,//read only
			number_of_failed_transmits_5,//read only
			number_of_packets_received_from_this_neighbor_5,//read only
			nickname_of_neighbor_6,//read only
			mean_rsl_6,//read only
			number_of_packets_transmitted_to_this_neighbor_6,//read only
			number_of_failed_transmits_6,//read only
			number_of_packets_received_from_this_neighbor_6,//read only
			nickname_of_neighbor_7,//read only
			mean_rsl_7,//read only
			number_of_packets_transmitted_to_this_neighbor_7,//read only
			number_of_failed_transmits_7,//read only
			number_of_packets_received_from_this_neighbor_7,//read only
			nickname_of_neighbor_8,//read only
			mean_rsl_8,//read only
			number_of_packets_transmitted_to_this_neighbor_8,//read only
			number_of_failed_transmits_8,//read only
			number_of_packets_received_from_this_neighbor_8//read only

		}

	}
}

MENU upload_variables
{
	LABEL DCT_UPLOAD_MENU;
	ITEMS
	{


    dev_var_input_1_probe_type,
    dev_var_temperature_1_number_of_wires,
    dev_var_temperature_1_2_wire_compensation_status,
    dev_var_temperature_1_2_wire_compensation_value,
    dev_var_temperature_1_2_wire_burnoutmode,
    dev_var_input_1_upper_sensor_limit,
    dev_var_input_1_lower_sensor_limit,
    dev_var_input_1_lower_sensor_limit_SHADOW,
    dev_var_input_1_upper_sensor_limit_SHADOW

    IF( TEMPERATURE_1_CJC_AVAILABLE )
    {
      dev_var_temperature_1_probe_connection
      dev_var_temperature_1_cjc_type,
      dev_var_temperature_1_cjc_unit,
      dev_var_temperature_1_cjc_value
   	}

    IF (  dev_var_temperature_1_number_of_wires == 2 ||
                  dev_var_input_1_probe_type == PROBE_TYPE_TC_E ||
                  dev_var_input_1_probe_type == PROBE_TYPE_TC_J ||
                  dev_var_input_1_probe_type == PROBE_TYPE_TC_K ||
                  dev_var_input_1_probe_type == PROBE_TYPE_TC_T )
    {
    dev_var_input_2_probe_type,
    dev_var_temperature_2_2_wire_compensation_status,
    dev_var_temperature_2_2_wire_compensation_value,
    dev_var_temperature_2_2_wire_burnoutmode
    }

    IF( TEMPERATURE_2_CJC_AVAILABLE )
    {
      dev_var_temperature_2_probe_connection,
      dev_var_temperature_2_cjc_type,
      dev_var_temperature_2_cjc_unit,
      dev_var_temperature_2_cjc_value
    }
	  	compat_burst_trig_units_0,
		compat_burst_trig_units_1,
		compat_burst_trig_units_2,

		//MENU Identification_menu
		longTag,
		descriptor,
		date,
		message,
		polling_address,
        burst_mode_select,

		//MENU DeviceVariableMapping_menu
		primary_variable_code,
		secondary_variable_code,
		tertiary_variable_code,
		quaternary_variable_code,


		//MENU WirelessCommunication_menu
		//network_id,
		/*IF ( !(JOIN_STATUS_EQU_x079F_2)) {
			radio_power_output,
		}*/
        IF ( !(JOIN_STATUS_EQU_x079F)) {
			radio_power_output,
		}
		join_mode,

		//MENU Burst(parIndex)_menu
		compat_burst_mode_0
		compat_update_per_0,
		compat_max_update_per_0,
		compat_burst_trig_mode_0,
		compat_burst_trig_class_0,
		compat_burst_trig_level_0,
		compat_burst_cmd_0,
		compat_burst_mode_1

        compat_update_per_1,
		compat_max_update_per_1,
		compat_burst_trig_mode_1,
		compat_burst_trig_class_1,
		compat_burst_trig_level_1,
		compat_burst_cmd_1,
		compat_burst_mode_2,

		compat_update_per_2,
		compat_max_update_per_2,
		compat_burst_trig_mode_2,
		compat_burst_trig_class_2,
		compat_burst_trig_level_2,
		compat_burst_cmd_2,
		compat_burst_vars_0_0,
		compat_burst_vars_0_1,
		compat_burst_vars_0_2,
		compat_burst_vars_0_3,
		compat_burst_vars_0_4,
		compat_burst_vars_0_5,
		compat_burst_vars_0_6,
		compat_burst_vars_0_7,
		compat_burst_vars_1_0,
		compat_burst_vars_1_1,
		compat_burst_vars_1_2,
		compat_burst_vars_1_3,
		compat_burst_vars_1_4,
		compat_burst_vars_1_5,
		compat_burst_vars_1_6,
		compat_burst_vars_1_7,
		compat_burst_vars_2_0,
		compat_burst_vars_2_1,
		compat_burst_vars_2_2,
		compat_burst_vars_2_3,
		compat_burst_vars_2_4,
		compat_burst_vars_2_5,
		compat_burst_vars_2_6,
		compat_burst_vars_2_7,
        compat_burst_cmd_enum_0,
        compat_burst_cmd_enum_1,
        compat_burst_cmd_enum_2


	}


	PRE_WRITE_ACTIONS {
		method_read_join_status
	}
	POST_WRITE_ACTIONS {
		method_burst_mode_and_event_notification_sync_upload
	}

}
#endif


METHOD method_read_join_status
{
#ifndef _PDM_
    VALIDITY FALSE;
#endif
    LABEL "Read Join status upfront the download";
    DEFINITION
		{
			//char status[STATUS_SIZE];
			send_command_trans(769,1);
            //send_command(1025);
			//save_values();
    }
}

METHOD method_Read_Temperature_Configuration
{
#ifndef _PDM_
    VALIDITY FALSE;
#endif
    LABEL "";
    DEFINITION
		{
            send_command(1025);
    }
}

#ifdef _PDM_
COMMAND read_join_status
{
    NUMBER 769;
    OPERATION READ;
    TRANSACTION 1
    {
        REQUEST
        {
        	0x03, 0x01
        }
        REPLY
        {
            0x03, 0x01,
            //dummy_16_bit_command,
            reserved_bits<0xF0>, wireless_mode<0x0F>,
            join_status_byte_hi,
            join_status_byte_lo,
            neighbor_count,
            advertisements,
            join_attempts,
            join_retry_timer,
            network_search_timer
        }
    }
    TRANSACTION 2
    {
        REQUEST
        {
        	0x03, 0x01
        }
        REPLY
        {
            0x03, 0x01,
            //dummy_16_bit_command,
            0x00,
            join_status_byte_hi_2,
            join_status_byte_lo_2,
            0x00,
            0x00,
            0x00,
            0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00
        }
    }
    TRANSACTION 3
    {
        REQUEST
        {
        	0x03, 0x01
        }
        REPLY
        {
            0x03, 0x01,
            //dummy_16_bit_command,
            0x00,
            join_status_ui,
            0x00,
            0x00,
            0x00,
            0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00
        }
    }

    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        32, MODE_ERROR, [busy];
    }
}
#endif




METHOD method_burst_mode_and_event_notification_sync_upload
{
	LABEL DCT_METHOD_BURST_MODE_AND_EVENT_NOTIFICATION_SYNC_UPLOAD;
#ifndef _PDM_
    VALIDITY FALSE;
#endif
    DEFINITION
	{
			// Burst Mode Control
			vassign(compat_burst_mode_local_0,compat_burst_mode_selection_0);
			vassign(compat_burst_mode_local_1,compat_burst_mode_selection_1);
			vassign(compat_burst_mode_local_2,compat_burst_mode_selection_2);
			save_values();
	}
}


METHOD post_edit_dev_var_temperature_1_number_of_wires
{
  LABEL L_DEV_VAR_TEMPERATURE_NUMBER_OF_WIRES;
  DEFINITION
	{
		int number_of_wires;
		int dev_var_input_1_probe_type;


    number_of_wires=igetval();

    if(number_of_wires != 2)
    {
      dev_var_input_1_probe_type=lvar_value(dev_var_input_1_probe_type);
      if( dev_var_input_1_probe_type == PROBE_TYPE_M100_100_MV ||
          dev_var_input_1_probe_type == PROBE_TYPE_0_300_MV ||
          dev_var_input_1_probe_type == PROBE_TYPE_TC_E ||
          dev_var_input_1_probe_type == PROBE_TYPE_TC_J ||
          dev_var_input_1_probe_type == PROBE_TYPE_TC_K ||
          dev_var_input_1_probe_type == PROBE_TYPE_TC_T )
      {
          lassign(dev_var_input_1_probe_type,PROBE_TYPE_RTD_PT100_0_003850);
          save_values();
      }
    }
	}
}


METHOD post_edit_dev_var_input_1_probe_type
{
#ifdef _PDM_
  ACCESS ONLINE;
#endif

  LABEL DCT_DEV_VAR_INPUT_CLASSIFICATION;
  DEFINITION
	{
		int dev_var_input_1_probe_type;
    dev_var_input_1_probe_type=igetval();
    if( dev_var_input_1_probe_type == PROBE_TYPE_M100_100_MV ||
        dev_var_input_1_probe_type == PROBE_TYPE_0_300_MV ||
        dev_var_input_1_probe_type == PROBE_TYPE_TC_E ||
        dev_var_input_1_probe_type == PROBE_TYPE_TC_J ||
        dev_var_input_1_probe_type == PROBE_TYPE_TC_K ||
        dev_var_input_1_probe_type == PROBE_TYPE_TC_T )
    {
        lassign(dev_var_temperature_1_number_of_wires,2);
        save_values();
    }
  }
}
VARIABLE dev_var_input_1_lower_sensor_limit_local
{
    LABEL "";
    HELP  "";
    CLASS LOCAL;
    HANDLING READ;
    TYPE FLOAT;
}
VARIABLE dev_var_input_1_upper_sensor_limit_local
{
    LABEL "";
    HELP  "";
    CLASS LOCAL;
    HANDLING READ;
    TYPE FLOAT;
}

METHOD edit_Probe_type_lower_upper_value
{
    LABEL "Edit sensor type";
    HELP  "";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        long ids[4];
        long dev_var_input_1_probe_type_local;
        int result;
        int sel;
        result = 1;
        sel = 1;

        IGNORE_ALL_DEVICE_STATUS();
		IGNORE_ALL_RESPONSE_CODES();
		XMTR_IGNORE_ALL_DEVICE_STATUS();
		XMTR_IGNORE_ALL_RESPONSE_CODES();

        //add_abort_method(method_join_network_aborted);

        ids[0] = VARID(dev_var_input_1_probe_type);
        ids[1] = VARID(dev_var_input_1_lower_sensor_limit);
        ids[2] = VARID(dev_var_input_1_upper_sensor_limit);

        dev_var_input_1_lower_sensor_limit_local = float_value(dev_var_input_1_lower_sensor_limit);
        dev_var_input_1_upper_sensor_limit_local= float_value(dev_var_input_1_upper_sensor_limit);
        save_values();

            get_dev_var_value("Please select the sensor type",ids,dev_var_input_1_probe_type);
            send_command_trans(1152,0);
            DELAY(2,"\n\n   --PLEASE WAIT--");
            //send_command(1025);
            save_values();
            if((dev_var_input_1_probe_type == 12) ||
   				(dev_var_input_1_probe_type == 15) ||
   				(dev_var_input_1_probe_type == 240) ||
   				(dev_var_input_1_probe_type == 241) ||
   				(dev_var_input_1_probe_type == 1) ||
   				(dev_var_input_1_probe_type == 2))
            {
                get_dev_var_value("Please select the number of wires for sensor Type %{0}",ids,dev_var_temperature_1_number_of_wires);
                send_command_trans(1152,0);
                DELAY(2,"\n\n   --PLEASE WAIT--");
                save_values();
            }
        while ( sel == 1)
        {
            while ( result == 1)
            { 
                get_dev_var_value("Please select the Lower Value for sensor Type %{0}",ids,dev_var_input_1_lower_sensor_limit_local);
                get_dev_var_value("Please select the Upper Value for sensor Type %{0}",ids,dev_var_input_1_upper_sensor_limit_local);
                result = select_from_list("Please select next step",ids,"Write data to device; Edit Values" );
            }

            if(dev_var_input_1_upper_sensor_limit.MIN_VALUE <= dev_var_input_1_upper_sensor_limit_local && dev_var_input_1_upper_sensor_limit.MAX_VALUE >= dev_var_input_1_upper_sensor_limit_local && dev_var_input_1_lower_sensor_limit.MIN_VALUE <= dev_var_input_1_lower_sensor_limit_local && dev_var_input_1_lower_sensor_limit.MAX_VALUE >= dev_var_input_1_lower_sensor_limit_local)
            {
                assign_float(dev_var_input_1_lower_sensor_limit,dev_var_input_1_lower_sensor_limit_local);
                assign_float(dev_var_input_1_upper_sensor_limit,dev_var_input_1_upper_sensor_limit_local);
                send_command(35);
                sel = 0;   
                save_values();
            }
            else
            {
                sel = select_from_list("Value out of range Please select next step",ids,"Abort;Edit Values" );
                result = 1;

            }

    }
            //process_abort();
}


}

