/*****************************************************************************
 *
 * File: Wireless.ddl
 * Version: 1.0.2
 *
 *****************************************************************************
 * Copyright (c) 2009, HART Communication Foundation, All Rights Reserved
 *****************************************************************************
 *
 * Description:  Specification DD that models the Wireless Command Specification
 *
 * Source Specifications
 *      Wireless Command Specification         Rev.1.0
 *
 *****************************************************************************
 */

/*****************************************************************************
 * The manufacturer line
 *****************************************************************************
 */
MANUFACTURER STANDARD, DEVICE_TYPE 0x18, DEVICE_REVISION 1, DD_REVISION 1

/*****************************************************************************
 * Include Files
 *****************************************************************************
 */

#include "macros.h"
#include "methods.h"

/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */

/*
 *****************************************************************************
 * Wireless Command Spec
 * 
 */
VARIABLE join_key_1
{
    LABEL "Key 1";
    HELP  "First key word for joining wirless network";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE UNSIGNED_INTEGER (4)
    {
       DISPLAY_FORMAT "08X";
       EDIT_FORMAT "08X";
    }
#if __TOKVER__ >= 600
    DEFAULT_VALUE 0;
#endif
}

VARIABLE join_key_2
{
    LABEL "Key 2";
    HELP  "Second key word for joining wireless network";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE UNSIGNED_INTEGER (4)
    {
       DISPLAY_FORMAT "08X";
       EDIT_FORMAT "08X";
    }
#if __TOKVER__ >= 600
    DEFAULT_VALUE 0;
#endif
}

VARIABLE join_key_3
{
    LABEL "Key 3";
    HELP  "Third key word for joining wireless network";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE UNSIGNED_INTEGER (4)
    {
       DISPLAY_FORMAT "08X";
       EDIT_FORMAT "08X";
    }
#if __TOKVER__ >= 600
    DEFAULT_VALUE 0;
#endif
}

VARIABLE join_key_4
{
    LABEL "Key 4";
    HELP  "Fourth key word for joining wireless network";
    CLASS DEVICE;
    HANDLING WRITE;
    TYPE UNSIGNED_INTEGER (4)
    {
       DISPLAY_FORMAT "08X";
       EDIT_FORMAT "08X";
    }
#if __TOKVER__ >= 600
    DEFAULT_VALUE 0;
#endif
}

COMMAND write_join_key
{
    NUMBER 768;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
             join_key_1, join_key_2, join_key_3, join_key_4
        }
        REPLY
        {
            response_code, device_status, 
            join_key_1, join_key_2, join_key_3, join_key_4
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
        33, MODE_ERROR,       [delayed_response_initiated];
        34, MODE_ERROR,       [delayed_response_running];
        35, MODE_ERROR,       [delayed_response_dead];
        36, MODE_ERROR,       [delayed_response_conflict];
        65, DATA_ENTRY_ERROR, "Key change failed";
    }
}

VARIABLE neighbor_count
{
    LABEL "Neighbor Count";
    HELP "Number of available neighbors.";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1);
}

VARIABLE advertisements
{
    LABEL "Advertisement Count";
    HELP "Number of Advertisement packets received.";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1);
}

VARIABLE join_attempts
{
    LABEL "Join Attempts";
    HELP "Number of Join Attempts. Too many join attempts will result in the device considering the join failed.";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1);
}

VARIABLE join_retry_timer
{
    LABEL "Join Retry Timer";
    HELP "The Join Retry Timer indicates the amount of time since the last join request was sent.";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
#if __TOKVER__ >= 800
    TYPE TIME_VALUE
    { 
        DISPLAY_FORMAT "5.3f" ;
        TIME_SCALE SECONDS ;
    }
#else
    TYPE UNSIGNED_INTEGER(4)
    {
        SCALING_FACTOR 0.00003125;
    }
    CONSTANT_UNIT "s";
#endif
}

VARIABLE network_search_timer
{
    LABEL "Network Search Time";
    HELP  "The Network Search Timer indicates the amount of time listening for first advertisement.";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
#if __TOKVER__ >= 800
    TYPE TIME_VALUE
    { 
        DISPLAY_FORMAT "5.3f" ;
        TIME_SCALE SECONDS ;
    }
#else
    TYPE UNSIGNED_INTEGER(4)
    {
        SCALING_FACTOR 0.00003125;
    }
    CONSTANT_UNIT "s";
#endif
}

VARIABLE reserved_bits
{
    /* LABEL and HELP not defined because these are never
       displayed to a user. */
    CLASS LOCAL;
    TYPE UNSIGNED_INTEGER;
}

COMMAND read_join_status
{
    NUMBER 769;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, 
            reserved_bits<0xF0>, wireless_mode<0x0F>,
            join_status,
            neighbor_count,
            advertisements,
            join_attempts,
            join_retry_timer,
            network_search_timer
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        32, MODE_ERROR, [busy];
    }
}

VARIABLE join_shed_time
{
    LABEL "Network Search Time";
    HELP  "Minimum time a device must stay in active search mode when forced to join a newtork";
    CLASS DEVICE;
    HANDLING READ & WRITE;
#if __TOKVER__ >= 800
    TYPE TIME_VALUE
    { 
        DISPLAY_FORMAT "5.3f" ;
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS ;
    }
#else
    TYPE UNSIGNED_INTEGER(4)
    {
        SCALING_FACTOR 0.00003125;
    }
    CONSTANT_UNIT "s";
#endif
#if __TOKVER__ >= 600
    DEFAULT_VALUE 60 * 60 * 32000;
#endif
}

VARIABLE shed_time
{
    LABEL "Shed Time";
    HELP  "Shed Time";
    CLASS DEVICE;
    HANDLING WRITE;
#if __TOKVER__ >= 800
    TYPE TIME_VALUE
    { 
        DISPLAY_FORMAT "5.3f" ;
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS ;
    }
#else
    TYPE UNSIGNED_INTEGER(4)
    {
        SCALING_FACTOR 0.00003125;
    }
    CONSTANT_UNIT "s";
#endif
#if __TOKVER__ >= 600
    DEFAULT_VALUE 60 * 60 * 32000;
#endif
}

VARIABLE advertising_period 
{
    LABEL "Adv. Period";
    HELP "Advertising period";
    CLASS HART;
    HANDLING READ;
#if __TOKVER__ >= 800
    TYPE TIME_VALUE
    { 
        DISPLAY_FORMAT "5.3f" ;
        TIME_SCALE SECONDS ;
    }
#else
    TYPE UNSIGNED_INTEGER(4)
    {
        SCALING_FACTOR 0.00003125;
    }
    CONSTANT_UNIT "s";
#endif
}

VARIABLE advertised_neighbors_number
{
    LABEL "Advertised Neighbors Number";
    HELP "Number of neighbors advertising in addition to this device.";
    CLASS HART;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1);
}

COMMAND request_active_advertising
{
    NUMBER 770;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            shed_time (INFO)
        }
        REPLY
        {
            response_code, device_status,
            shed_time (INFO), advertising_period, advertised_neighbors_number
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR,   [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR,   [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        8,  DATA_ENTRY_WARNING, [set_to_nearest_possible_value]; 
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy];
        33, MODE_ERROR,         [delayed_response_initiated];
        34, MODE_ERROR,         [delayed_response_running];
        35, MODE_ERROR,         [delayed_response_dead];
        36, MODE_ERROR,         [delayed_response_conflict];
        65, DATA_ENTRY_ERROR,   "Request declined";    
    }
}

COMMAND force_join_mode
{
    NUMBER 771;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            join_mode, join_shed_time
        }
        REPLY
        {
            response_code, device_status, 
            join_mode, join_shed_time
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR,   [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR,   [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        8,  DATA_ENTRY_WARNING, [set_to_nearest_possible_value]; 
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy];
        33, MODE_ERROR,         [delayed_response_initiated];
        34, MODE_ERROR,         [delayed_response_running];
        35, MODE_ERROR,         [delayed_response_dead];
        36, MODE_ERROR,         [delayed_response_conflict];
        65, DATA_ENTRY_ERROR,   "Force join declined";
    }
}

COMMAND read_join_mode_configuration
{
    NUMBER 772;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, 
            join_mode, join_shed_time
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        32, MODE_ERROR, [busy];
    }
}

VARIABLE network_id
{
    LABEL "Network ID";
    HELP  "";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER (2)
    {
        MAX_VALUE  0x8FFF;   // reference Spec 75, TDMA Data Link Layer
    }
}

COMMAND write_network_id
{
    NUMBER 773;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            network_id
        }
        REPLY
        {
            response_code, device_status, network_id
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        8,  MISC_WARNING,     "Network ID change pending";  
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
        33, MODE_ERROR,       [delayed_response_initiated];
        34, MODE_ERROR,       [delayed_response_running];
        35, MODE_ERROR,       [delayed_response_dead];
        36, MODE_ERROR,       [delayed_response_conflict];
        65, DATA_ENTRY_ERROR, "Invalid network ID";
    }
}

COMMAND read_network_id
{
    NUMBER 774;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, network_id
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        32, MODE_ERROR, [busy];
    }
}

VARIABLE network_tag
{
    LABEL "Network Tag";
    HELP "The network tag is a proxy for the Network Id so that the network can be identified in a text form at the application level.";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE ASCII (32);
}

COMMAND write_network_tag
{
    NUMBER 775;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            network_tag
        }
        REPLY
        {
            response_code, device_status, network_tag
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
        33, MODE_ERROR,       [delayed_response_initiated];
        34, MODE_ERROR,       [delayed_response_running];
        35, MODE_ERROR,       [delayed_response_dead];
        36, MODE_ERROR,       [delayed_response_conflict];
    }
}

COMMAND read_network_tag
{
    NUMBER 776;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, network_tag
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        32, MODE_ERROR, [busy];
    }
}

/* Command 777 **********************************************/

VARIABLE peak_packets_per_second
{
    LABEL "Peak packets/s";
    HELP "Peak packets per second.";
    CLASS DEVICE;
    HANDLING READ;
    TYPE FLOAT;
}

VARIABLE peak_packet_load
{
    LABEL "Peak packet load";
    HELP "Duration at peak packet load before power drained. Must be at least "
         "one hour (set to 24 hours if not applicable)";
    CLASS DEVICE;
    HANDLING READ;
#if __TOKVER__ >= 800
    TYPE TIME_VALUE
    { 
        DISPLAY_FORMAT "5.3f";
        TIME_SCALE HOURS;
    }
#else
    TYPE UNSIGNED_INTEGER(4)
    {
        SCALING_FACTOR 0.00003125;
    }
    CONSTANT_UNIT "s";
#endif
}

VARIABLE recover_time
{
    LABEL "Recover time";
    HELP "Time to recover from power drain (set to zero if not applicable). "
         "While recovering, the device must be able to route 1 packet per second.";
    CLASS DEVICE;
    HANDLING READ;
#if __TOKVER__ >= 800
    TYPE TIME_VALUE
    { 
        DISPLAY_FORMAT "5.3f";
        TIME_SCALE MINUTES;
    }
#else
    TYPE UNSIGNED_INTEGER(4)
    {
        SCALING_FACTOR 0.00003125;
    }
    CONSTANT_UNIT "s";
#endif
}

VARIABLE receive_signal_level
{
    LABEL "Receive signal level";
    HELP "RSL (receive signal level in dB) threshold 'good' connection.";
    CLASS DEVICE;
    HANDLING READ;
    TYPE INTEGER (1);
    CONSTANT_UNIT "dBm";
}

VARIABLE keep_alive_min_time
{
    LABEL "Keep-alive time";
    HELP "Minimum required keep-alive time.";
    CLASS DEVICE;
    HANDLING READ;
#if __TOKVER__ >= 800
    TYPE TIME_VALUE
    { 
        DISPLAY_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
#else
    TYPE UNSIGNED_INTEGER(4)
    {
        SCALING_FACTOR 0.00003125;
    }
    CONSTANT_UNIT "s";
#endif
}

VARIABLE max_neighbors
{
    LABEL "Max Neighbors";
    HELP "Maximum number of neighbors";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (2);
}

VARIABLE max_packet_buffers
{
    LABEL "Max Packetbuffer";
    HELP "Maximum number of packet buffers";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (2);
}

COMMAND read_wireless_device_capabilities
{
    NUMBER 777;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            reserved_bits<0xF0>, power_source<0x0F>, peak_packets_per_second, 
            peak_packet_load, recover_time, receive_signal_level, 
            keep_alive_min_time, max_neighbors, max_packet_buffers
        }
    }
    RESPONSE_CODES
    {
        0, SUCCESS, [no_command_specific_errors];
    }
}

VARIABLE battery_life
{
    LABEL "Battery Life Remaining";
    HELP  "Remaining battery life";
    CLASS CORRECTION;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (2);
    CONSTANT_UNIT "Days";
}

COMMAND read_battery_life
{
    NUMBER 778;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, battery_life
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        32, MODE_ERROR, [busy];
    }
}

VARIABLE generated_packets
{
    LABEL "Generated packets";
    HELP "Number of packets generated by this device";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (2);
}

VARIABLE terminated_packets
{
    LABEL "Terminated packets";
    HELP "Number of packets terminated by this device";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (2);
}

VARIABLE detected_dll_mic_failures
{
    LABEL "DLL failures";
    HELP "Number of Data-Link Layer MIC failures detected";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE detected_nl_mic_failures
{
    LABEL "NL(Session) failures";
    HELP "Number of Network Layer (Session) MIC failures detected";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE detected_crc_errors
{
    LABEL "CRC errors";
    HELP "Number of CRC errors detected.";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE not_received_nonce_counter_values
{
    LABEL "Nonce Counter Values";
    HELP "Number of nonce counter values not received. This value is "
         "incremented for each zero in nonce counter history that "
         "underflows.";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

COMMAND report_device_health
{
    NUMBER 779;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status,
            generated_packets, terminated_packets, detected_dll_mic_failures,
            detected_nl_mic_failures, device_power_status, detected_crc_errors,
            not_received_nonce_counter_values
        }
    }
    RESPONSE_CODES
    {
        0, SUCCESS, [no_command_specific_errors];
    }
}


// Command 780, not needed by DD host application


VARIABLE device_nickname_address
{
    LABEL "Device Nickname Address";
    HELP "The 2-byte nickname is assigned and managed by the Network Manager. "
         "Consequently, it is only locally unique (i.e., within the network the device belongs to).";
    CLASS HART;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}

COMMAND read_device_nickname_address    //Is this really needed for DD hosts??
{
    NUMBER 781;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, device_nickname_address
        }
    }
    RESPONSE_CODES
    {
        0, SUCCESS, [no_command_specific_errors];
    }
}


// Command 782, not needed by DD host application

// Command 783, not needed by DD host application

// Command 784, not needed by DD host application

// Command 785, not needed by DD host application

// Command 786, not needed by DD host application

// Command 787, not needed by DD host application

// Command 788, not needed by DD host application

// Command 789, not needed by DD host application

// Command 790, not needed by DD host application

// Command 791, not needed by DD host application

// Command 792, not defined

// Commands 793 and 794 are used internally by the Network Manager only and therefore shouldn't be in a DD

// Command 795, not needed by DD host application

// Command 796, not needed by DD host application

VARIABLE radio_power_output
{
    LABEL "Output Power";
    HELP "";
    CLASS HART;
    TYPE INTEGER;
    CONSTANT_UNIT "dBm";
}

COMMAND write_radio_power_output
{
    NUMBER 797;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            radio_power_output
        }
        REPLY
        {
            response_code, device_status, radio_power_output
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,            [no_command_specific_errors];
        3,  DATA_ENTRY_ERROR,   [passed_parameter_too_large];
        4,  DATA_ENTRY_ERROR,   [passed_parameter_too_small];
        5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
        6,  MISC_ERROR,         [xmtr_specific_command_error];
        7,  MODE_ERROR,         [in_write_protect_mode];
        8,  DATA_ENTRY_WARNING, [set_to_nearest_possible_value]; 
        16, MODE_ERROR,         [access_restricted];
        32, MODE_ERROR,         [busy];
        33, MODE_ERROR,         [delayed_response_initiated];
        34, MODE_ERROR,         [delayed_response_running];
        35, MODE_ERROR,         [delayed_response_dead];
        36, MODE_ERROR,         [delayed_response_conflict];
    }
}

COMMAND read_radio_output_power
{
    NUMBER 798;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status
            radio_power_output
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        32, MODE_ERROR, [busy];
    }
}


// Command 799, not needed by DD host application

// Command 800, not needed by DD host application

// Command 801, not needed by DD host application

// Command 802, not needed by DD host application

// Command 803, not needed by DD host application

// Commands 804 & 805 are Network Manager commands only and do not belong in a DD

// Command 806, not needed by DD host application

// Command 807, not needed by DD host application

// Command 808, not needed by DD host application

// Command 809, not needed by DD host application

// Command 812, not needed by DD host application

// Command 813, not needed by DD host application

// Command 814, not needed by DD host application

// Command 815, not needed by DD host application
                           
// Command 816, not needed by DD host application

// Command 817, not needed by DD host application

// Command 818, not needed by DD host application

// Command 819, not needed by DD host application

// Command 820, not needed by DD host application

COMMAND write_network_access_mode
{
    NUMBER 821;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            network_access_mode
        }
        REPLY
        {
            response_code, device_status, network_access_mode
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];  
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
        33, MODE_ERROR,       [delayed_response_initiated];
        34, MODE_ERROR,       [delayed_response_running];
        35, MODE_ERROR,       [delayed_response_dead];
        36, MODE_ERROR,       [delayed_response_conflict];
    }
}

COMMAND read_network_access_mode
{
    NUMBER 822;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, network_access_mode
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, [xmtr_specific_command_error];
        32, MODE_ERROR, [busy];
    }
}

// Command 823, not needed by DD host application

// Command 824 - 831, not defined

// Command 832 - 845, Gateway DD, would require lists to model. Skipped for now

// Command 846 -959, not defined

// Command 960 - 976, not needed by DD host application

VARIABLE wireless_module_transmitter_revision
{
    LABEL "Wireless Module Device Revision";
    HELP "";
    CLASS HART;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}


VARIABLE wireless_module_software_revision
{
    LABEL "Wireless Module Software Revision";
    HELP "";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

VARIABLE wireless_module_hardware_revision
{
    LABEL "Wireless Module Hardware Revision";
    HELP "";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}

COMMAND read_wireless_module_revision
{
    NUMBER 64512;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST {}
        REPLY
        {
            response_code, device_status, 
            wireless_module_device_type, wireless_module_manufacturer_id, 
            wireless_module_transmitter_revision, wireless_module_software_revision, 
            wireless_module_hardware_revision
        }
    }
    RESPONSE_CODES
    {
        0, SUCCESS, [no_command_specific_errors];
    }
}

