/*****************************************************************************
 *
 * File: LWA.ddl
 * Version: 0.1
 * Date: 05/30/08
 * Copyright (c) 2008, CodeWrights GmbH Germany
 *
 * Description:
 * Local Wireless Adapater Main
 *
 *****************************************************************************
 */

#ifdef _PDM_
#define DUMMY_16BITS_PDM 0x00, 0x00,
#else
#define DUMMY_16BITS_PDM
#endif
// GEQ = Greater than or equal
#define JOIN_STATUS_GEQ_128\
		( (join_status_byte_lo >= 128 ) || (join_status_byte_hi >= 1) )

// LSS = Less than
#define JOIN_STATUS_LSS_128\
		( (join_status_byte_lo < 128 ) || (join_status_byte_hi < 1) )

#define JOIN_STATUS_EQU_x079F\
		( (join_status_byte_lo == 0x9F ) && (join_status_byte_hi == 0x07) )

#define JOIN_STATUS_EQU_x079F_2\
		( (join_status_byte_lo_2 == 0x9F ) && (join_status_byte_hi_2 == 0x07) )
/*****************************************************************************
 * Include Files
 *****************************************************************************
 */
#include "lclDef.h"
#include "lwa.h"
#include "macros.h"
#include "methods.h"
#include "Menus.dd"
#include "MenusAMS.dd"
#include "Burst.dd"
#include "Event.dd"
#include "Subdevice.dd"


/*****************************************************************************
 * PDM IDENTCHECK
 *****************************************************************************
 */
#ifdef _PDM_
#include "identcheck.inc"
#endif

/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */

#define SEND_TO_DEVICE11(x) \
  LABEL "SEND_TO_DEVICE11"; \
  CLASS COMPUTATION; \
  DEFINITION \
		{ \
		ACKNOWLEDGE("NOTICE - Sending Parameter to device to update dependencies.|de|HINWEIS - Parameter wird zum Gert gesendet um Abhngigkeiten zu lsen."); \
		iassign(variable_array_index, x); \
		send_command_trans(173,0); \
		}

#define SEND_TRIGGER_TO_DEVICE11(x) \
  LABEL "SEND_TO_DEVICE11"; \
  CLASS COMPUTATION; \
  DEFINITION \
		{ \
		iassign(variable_array_index, x); \
		send_command_trans(173,0); \
		}

#define MODULE_INDEX_MASTER 0 // Module Index value
#define MODULE_INDEX_SUB    1 // Module Index value
#define READ_JOIN_STATUS_CMD_NBR 769

#define ENUM_LIMIT_STATUS TYPE ENUMERATED \
    { \
        { 0, [not_limited] },   \
        { 1, [low_limited] }, \
        { 2, [high_limited] }, \
        { 3, [constant] } \
    }


#define ENUM_PROCESS_DATA_STATUS TYPE ENUMERATED \
    { \
        { 0, [bad] },   \
        { 1, [manual_fixed] }, \
        { 2, [poor_accuracy] }, \
        { 3, [good] }  \
    }


#define ENUM_DEVICE_FAMILY_STATUS TYPE ENUMERATED \
    { \
        { 0x08, [more_device_family_status_available]} \
    }


#define ENUM_TABLE_21 TYPE ENUMERATED \
    { \
{ 81, [analytical_classification] },        /* 0x51 */ \
{ 86, [angle_classification] },             /* 0x56 */ \
{ 75, [angular_velocity_classification] },  /* 0x4B */ \
{ 76, [area_classification] },              /* 0x4C */ \
{ 82, [capacitance_classification] },       /* 0x52 */ \
{ 90, [concentration_classification] },     /* 0x5A */ \
{ 87, [conductance_classification] },       /* 0x57 */ \
{ 95, DCT_CORIOLIS_FLOW_CLASSIFICATION },     /* 0x5F */ \
{ 84, [current_classification] },           /* 0x54 */ \
{ 0,  [device_variable_not_classified] },      \
{ 83, [emf_classification] },               /* 0x53 */ \
{ 77, [energy_work_classification] },       /* 0x4D */ \
{ 78, [force_classification] },             /* 0x4E */ \
{ 80, [frequency_classification] },         /* 0x50 */ \
{ 69, [length_classification] },            /* 0x45 */ \
{ 92, DCT_LEVEL_CLASSIFICATION},             /* 0x5C */ \
{ 94, DCT_MAG_FLOW_CLASSIFICATION },          /* 0x5E */ \
{ 71, [mass_classification] },              /* 0x47 */ \
{ 72, [mass_flow_classification] },         /* 0x48 */ \
{ 73, [mass_per_volume_classification] },   /* 0x49 */ \
{ 79, [power_classification] },             /* 0x4F */ \
{ 65, [pressure_classification] },          /* 0x41 */ \
{ 85, [resistance_classification] },        /* 0x55 */ \
{ 64, [temperature_classification] },       /* 0x40 */ \
{ 70, [time_classification] },              /* 0x46 */ \
{ 91, [valve_actuator_classification] },    /* 0x5B */ \
{ 67, [velocity_classification] },          /* 0x43 */ \
{ 74, [viscosity_classification] },         /* 0x4A */ \
{ 68, [volume_classification] },            /* 0x44 */ \
{ 89, [volume_per_mass_classification] },   /* 0x59 */ \
{ 88, [volume_per_volume_classification] }, /* 0x58 */ \
{ 66, [volumetric_flow_classification] },   /* 0x42 */ \
{ 93, DCT_VORTEX_FLOW_CLASSIFICATION }       /* 0x5D */ \
    }

#define ENUM_TABLE_20 TYPE ENUMERATED \
    { \
         \
        /* codes 0-3 are reserved and must never be used */ \
         \
        { 4,  [temperature_family] },   \
        { 5,  [pressure_family] },   \
        { 6,  [actuator_family] },   \
        { 7,  [simple_pid_family] }, \
        { 8,  [ph_family] }, \
        { 9,  [conductivity_family] }, \
        { 10, [totalizer_family] }, \
        { 11, [level_family] },             \
        { 12, [vortex_flow_family] }, \
        { 13, [mag_flow_family] }, \
        { 14, [coriolis_flow_family] } \
 \
        /* codes 132-249 are reserved and must never be used */ \
 \
        { 250, DCT_NOT_USED } \
    }

/* defines a NON SI region - which is active only if non si is allowed. */
/* At the moment the device doesn't support this command. So non si (units) are disabled all the time. */
#define NONSIBEGIN IF(si_control == 0) {
#define NONSIEND }

/* Temperature */
/**********************************************************************
 * Table 2.64 (0x40) Temperature
 */
#define ENUM_TEMPERATURE_UNIT TYPE ENUMERATED       \
{                                                   \
    IF (si_control == 1)  /* Restricted to SI units */\
    {                                                 \
        { 0x20,  [degC],  [degrees_celsius_help]  }   \
        { 0x23,  [Kelvin],[degrees_kelvin_help]   }   \
    }                                               \
    ELSE                                            \
    {                                               \
        { 0x20,  [degC],  [degrees_celsius_help]    } \
        { 0x21,  [degF],  [degrees_fahrenheit_help] } \
        { 0x23,  [Kelvin],[degrees_kelvin_help]     } \
    }                                               \
}

/*****************************************************************************
 * Common Tables
 */

#ifndef _PDM_
IMPORT STANDARD _TABLES, DEVICE_REVISION 20, DD_REVISION 1
#else
IMPORT "Tabl20_1.ddl"
#endif
{
    //    ALL_TABLES;
    EVERYTHING;

    REDEFINITIONS
    {

        REDEFINE VARIABLE device_status
        {
            LABEL DCT_CONFIGURATIONCHANGEDFLAG_LABEL;
            CLASS HART & DYNAMIC & DIAGNOSTIC;
            HANDLING READ;
            TYPE BIT_ENUMERATED
            {
                { 0x40, [configuration_changed],        MISC }
            }
        }

        // This must be redefined if the Device Type is not listed
        // in Common Tables DD


        VARIABLE device_type
        {
            TYPE ENUMERATED
            {
                MYDEVICETYPE
            }
        }

        // This must be redefined if the Manufacturer is not listed
        // in the Common Tables DD
        VARIABLE manufacturer_id
        {
            REDEFINE TYPE ENUMERATED(2)
            {
                // replace the 0xf9 with your manufactures id and appropriete label
                MYMANUFACTURERID
            }
        }

        VARIABLE burst_trigger_classification
        {
            REDEFINE LABEL DCT_LABEL_BURST_TRIGGER_CLASSIFICATION_MESSAGE_N;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            REDEFINE ENUM_TABLE_21
        }

        VARIABLE burst_trigger_units
        {
            REDEFINE LABEL DCT_LABEL_BURST_TRIGGER_UNITS_MESSAGE_N;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            REDEFINE TYPE ENUMERATED
            {
                ALL_UNITS_SORTED_EXTENDED
            }
        }

        VARIABLE burst_message_trigger_mode
        {
            REDEFINE LABEL DCT_LABEL_BURST_MESSAGE_TRIGGER_MODE_MESSASGE_N;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE burst_mode_select
        {
            REDEFINE LABEL DCT_LABEL_BURST_MODE_SELECT_N;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            REDEFINE TYPE ENUMERATED
            {
                BURST_MODE_CONTROL_CODE(0),  /* Off */
		        		{2, DCT_WIRELESS}
            }
        }

        VARIABLE event_notification_control_n
        {
            REDEFINE LABEL DCT_EVENT_NOTIFICATION_CONTROL_SWA;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            REDEFINE TYPE ENUMERATED
            {
		        {0, event_nofication_control_codes(0) },
		        {2, DCT_WIRELESS}
            }
        }

        REDEFINE VARIABLE device_status_mask
        {
            LABEL DCT_STANDARD_DEVICE_STATUS;
            HELP DCT_STANDARD_DEVICE_STATUS_HELP;
            CLASS DEVICE;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE BIT_ENUMERATED
            {
                { 0x01, "Primary Variable Out of Limits",     DV1 (AUTO & BAD) },
                { 0x02, "Non-Primary Variable Out of Limits", DV2 (AUTO & BAD) },
                { 0x04, "Loop Current Saturated",             AO1 (AUTO & BAD) },
                { 0x08, "Loop Current Fixed",                 AO1 (MANUAL & GOOD) },
                { 0x10, "More Status Available",              MORE },
                { 0x20, "Cold Start",                         MISC },
                { 0x40, "Configuration Changed",              MISC },
                { 0x80, "Device Malfunction",                 MORE }
            }
            DEFAULT_VALUE 0;
        }

        REDEFINE VARIABLE extended_fld_device_status_mask
        {
            LABEL DCT_EXTENDED_FLD_DEVICE_STATUS_MASK_SWA;
            HELP DCT_EXTENDED_FLD_DEVICE_STATUS_MASK_SWA_HELP;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_EXTENDED_DEVICE_STATUS_6_REMAINING_BATTERY_LIFETIME, HARDWARE },
                { 0x02, DCT_EXTENDED_DEVICE_STATUS_6_DEVICE_VARIABLE_ALERT, SOFTWARE },
                { 0x04, DCT_EXTENDED_DEVICE_STATUS_6_BATTERY_FAILURE }
            }
            DEFAULT_VALUE 0;
        }

        VARIABLE standardized_status_0_codes        //Standard dictionary
        {
            REDEFINE TYPE ENUMERATED //Note:  All tables are described using ENUMERATED even though they will be used in actual DD as a BIT_ENUMERATED
            {
                { 0x01, DCT_STANDARDIZED_STATUS_8_SIMULATION_ACTIVE},
                { 0x02, DCT_STANDARDIZED_STATUS_8_NON_VOLATILE_MEMORY_DEFECT},
                { 0x04, DCT_STANDARDIZED_STATUS_8_VOLATILE_MEMORY_DEFECT},
                { 0x08, DCT_STANDARDIZED_STATUS_8_WATCHDOG_RESET_EXECUTED },
                { 0x10, DCT_STANDARDIZED_STATUS_8_VOLTAGE_CONDITIONS_OUT_OF_RANGE},
                { 0x20, DCT_STANDARDIZED_STATUS_8_ENVIRONMENTAL_CONDITIONS_OUT_OF_RANGE},
                { 0x40, DCT_STANDARDIZED_STATUS_8_ELECTRONIC_DEFECT}
            }
        }

        REDEFINE VARIABLE standardized_status_0_mask
        {
            LABEL DCT_STANDARDIZED_STATUS_0_MASK;
            HELP DCT_STANDARDIZED_STATUS_0_MASK_HELP;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE BIT_ENUMERATED
            {
                { 0x01, standardized_status_0_codes(0x01), MODE },
                { 0x02, standardized_status_0_codes(0x02), HARDWARE },
                { 0x04, standardized_status_0_codes(0x04), HARDWARE },
                { 0x08, standardized_status_0_codes(0x08), HARDWARE & SOFTWARE }, // MISC if this doesn't work
                { 0x10, standardized_status_0_codes(0x10), HARDWARE },
                { 0x20, standardized_status_0_codes(0x20), MISC },
                { 0x40, standardized_status_0_codes(0x40), HARDWARE }
            }
            DEFAULT_VALUE 0;
        }

        REDEFINE VARIABLE standardized_status_1_mask
        {
            LABEL DCT_STANDARDIZED_STATUS_1_MASK;
            HELP DCT_STANDARDIZED_STATUS_1_MASK_HELP;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE BIT_ENUMERATED
            {
                { 0x01, standardized_status_1_codes(0x01), MISC }
            }
            DEFAULT_VALUE 0;
       }

        REDEFINE VARIABLE standardized_status_2_mask
        {
            LABEL DCT_STANDARDIZED_STATUS_2_MASK;
            HELP DCT_STANDARDIZED_STATUS_2_MASK_HELP;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_STANDARDIZED_STATUS_2_SUBDEVICE_LIST_CHANGED_SWA, DATA },
                { 0x02, DCT_STANDARDIZED_STATUS_2_SUBDEVICE_ADAPTER_DISCOVERED_SWA, MISC }
            }
            DEFAULT_VALUE 0;
        }

        REDEFINE VARIABLE standardized_status_3_mask
        {
            LABEL DCT_STANDARDIZED_STATUS_3_MASK;
            HELP DCT_STANDARDIZED_STATUS_3_MASK_HELP;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_STANDARDIZED_STATUS_3_ADAPTER_UNABLE_GET_BANDWIDTH, DATA },
                { 0x04, DCT_STANDARDIZED_STATUS_3_ADAPTER_ASKED_FOR_BANDWIDTH, MISC }
                //{ 0x08, DCT_STANDARDIZED_STATUS_3_FIRMWARE_UPDATE_IN_PROCESS, MISC}
            }
            DEFAULT_VALUE 0;
        }

        REDEFINE VARIABLE analog_channel_saturated1_mask
        {
            LABEL DCT_ANALOG_CHANNEL_SATURATED1_MASK_1_SWA;
            HELP DCT_ANALOG_CHANNEL_SATURATED1_MASK_1_SWA_HELP;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE BIT_ENUMERATED
            {
		        {0x01, DCT_ANALOG_CHANNEL1},
		        {0x02, DCT_ANALOG_CHANNEL2},
		        {0x04, DCT_ANALOG_CHANNEL3},
		        {0x08, DCT_ANALOG_CHANNEL4}
            }
            DEFAULT_VALUE 0;
        }

        REDEFINE VARIABLE analog_channel_fixed1_mask
        {
            LABEL DCT_ANALOG_CHANNEL_FIXED_SWA;
            HELP DCT_ANALOG_CHANNEL_FIXED_SWA_HELP;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE BIT_ENUMERATED
            {
		        {0x01, DCT_ANALOG_CHANNEL1},
		        {0x02, DCT_ANALOG_CHANNEL2},
		        {0x04, DCT_ANALOG_CHANNEL3},
		        {0x08, DCT_ANALOG_CHANNEL4}
            }
            DEFAULT_VALUE 0;
        }

        REDEFINE VARIABLE dev_operating_mode_mask
        {
            LABEL DCT_OPERATING_MODE_MASK;
            HELP DCT_OPERATING_MODE_MASK_HELP;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE BIT_ENUMERATED           //MASKS should be bit-enum not enum
            {
                {0x01, "Not Defined"}
            }
            DEFAULT_VALUE 0;
        }

    }
}

/*****************************************************************************
 * Universal Commmands
 */

#ifndef _PDM_
IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 7, DD_REVISION 1
#else
IMPORT "Univ7.ddl"
#endif
{
    // All HART devices must support all Universal Commands
    EVERYTHING;

    REDEFINITIONS
    {
        REDEFINE VARIABLE universal_revision
        {
            LABEL DCT_UNIVERSAL_REVISION_SWA;
            HELP [universal_revision_help];
            CLASS HART;
            HANDLING READ;
            TYPE UNSIGNED_INTEGER
            {
                MIN_VALUE  7;
            }
            DEFAULT_VALUE 7;
        }

        VARIABLE device_specific_status_0
        {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_0_SWA;
            REDEFINE  TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_DEV_SPEC_STAT0_0, DCT_DEV_SPEC_STAT0_0_HELP, MISC },
                { 0x02, DCT_DEV_SPEC_STAT0_1, DCT_DEV_SPEC_STAT0_1_HELP, MISC },
                { 0x04, DCT_DEV_SPEC_STAT0_2, DCT_DEV_SPEC_STAT0_2_HELP, MISC },
                { 0x08, DCT_DEV_SPEC_STAT0_3, DCT_DEV_SPEC_STAT0_3_HELP, MISC }
                { 0x10, DCT_DEV_SPEC_STAT0_4, DCT_DEV_SPEC_STAT0_4_HELP, MISC }
            }
        }

        VARIABLE device_specific_status_1
        {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_1_SWA;
            REDEFINE  TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_DEV_SPEC_STAT1_0, DCT_DEV_SPEC_STAT1_0_HELP, MISC },
                { 0x02, DCT_DEV_SPEC_STAT1_1, DCT_DEV_SPEC_STAT1_1_HELP, MISC },
                { 0x04, DCT_DEV_SPEC_STAT1_2, DCT_DEV_SPEC_STAT1_2_HELP, MISC },
                { 0x08, DCT_DEV_SPEC_STAT1_3, DCT_DEV_SPEC_STAT1_3_HELP, MISC },
                { 0x10, DCT_DEV_SPEC_STAT1_4, DCT_DEV_SPEC_STAT1_4_HELP, MISC },
                { 0x20, DCT_DEV_SPEC_STAT1_5, DCT_DEV_SPEC_STAT1_5_HELP, MISC },
                { 0x40, DCT_DEV_SPEC_STAT1_6, DCT_DEV_SPEC_STAT1_6_HELP, MISC },
                { 0x80, DCT_DEV_SPEC_STAT1_7, DCT_DEV_SPEC_STAT1_7_HELP, MISC }
            }
        }

        VARIABLE device_specific_status_2
        {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_2_SWA;
            REDEFINE  TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_DEV_SPEC_STAT2_0, DCT_DEV_SPEC_STAT2_0_HELP, MISC },
                { 0x02, DCT_DEV_SPEC_STAT2_1, DCT_DEV_SPEC_STAT2_1_HELP, MISC },
                { 0x04, DCT_DEV_SPEC_STAT2_2, DCT_DEV_SPEC_STAT2_2_HELP, MISC },
                { 0x08, DCT_DEV_SPEC_STAT2_3, DCT_DEV_SPEC_STAT2_3_HELP, MISC },
                { 0x10, DCT_DEV_SPEC_STAT2_4, DCT_DEV_SPEC_STAT2_4_HELP, MISC },
                { 0x20, DCT_DEV_SPEC_STAT2_5, DCT_DEV_SPEC_STAT2_5_HELP, MISC }
                { 0x80, DCT_DEV_SPEC_STAT2_7, DCT_DEV_SPEC_STAT2_7_HELP, MISC }
            }
        }

        VARIABLE device_specific_status_3
        {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_3_SWA;
            REDEFINE  TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_DEV_SPEC_STAT3_0, DCT_DEV_SPEC_STAT3_0_HELP, MISC },
                { 0x02, DCT_DEV_SPEC_STAT3_1, DCT_DEV_SPEC_STAT3_1_HELP, MISC },
                { 0x04, DCT_DEV_SPEC_STAT3_2, DCT_DEV_SPEC_STAT3_2_HELP, MISC },
                { 0x08, DCT_DEV_SPEC_STAT3_3, DCT_DEV_SPEC_STAT3_3_HELP, MISC }
                { 0x10, DCT_DEV_SPEC_STAT3_4, DCT_DEV_SPEC_STAT3_4_HELP, MISC }
                { 0x20, DCT_DEV_SPEC_STAT3_5, DCT_DEV_SPEC_STAT3_5_HELP, MISC }
            }
        }

				// Variable labels redefinitions - Universal 7
        VARIABLE longTag
        {
            REDEFINE LABEL DCT_LONG_TAG_SWA;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE tag
        {
            REDEFINE LABEL DCT_TAG_SWA;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE descriptor
        {
            REDEFINE LABEL DCT_DESCRIPTOR_SWA;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE date
        {
            REDEFINE LABEL DCT_DATE_SWA;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE message
        {
            REDEFINE LABEL DCT_MESSAGE_SWA;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE polling_address
        {
            REDEFINE LABEL DCT_POLLING_ADDRESS_SWA;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE response_preambles
        {
            REDEFINE LABEL DCT_RESPONSE_PREAMBLES_SWA;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE transmitter_revision
        {
            REDEFINE LABEL DCT_TRANSMITTER_REVISION_SWA;
        }

        VARIABLE software_revision
        {
            REDEFINE LABEL DCT_SOFTWARE_REVISION_SWA;
        }

        VARIABLE hardware_revision
        {
            REDEFINE LABEL DCT_HARDWARE_REVISION_SWA;
        }

        VARIABLE config_change_counter
        {
            REDEFINE LABEL DCT_CONFIG_CHANGE_COUNTER_SWA;
        }

        VARIABLE device_specific_status_4
        {
            REDEFINE LABEL DCT_DEVICE_SPECIFIC_STATUS_4_SWA;

            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_DEVICE_SPECIFIC_STATUS_4_DEVICE_1_MORE_STATUS_AVAILABLE },
                { 0x02, DCT_DEVICE_SPECIFIC_STATUS_4_DEVICE_1_MALFUNCTION },
                { 0x04, DCT_DEVICE_SPECIFIC_STATUS_4_DEVICE_2_MORE_STATUS_AVAILABLE },
                { 0x08, DCT_DEVICE_SPECIFIC_STATUS_4_DEVICE_2_MALFUNCTION },
                { 0x10, DCT_DEVICE_SPECIFIC_STATUS_4_DEVICE_3_MORE_STATUS_AVAILABLE },
                { 0x20, DCT_DEVICE_SPECIFIC_STATUS_4_DEVICE_3_MALFUNCTION },
                { 0x40, DCT_DEVICE_SPECIFIC_STATUS_4_DEVICE_4_MORE_STATUS_AVAILABLE },
                { 0x80, DCT_DEVICE_SPECIFIC_STATUS_4_DEVICE_4_MALFUNCTION }
            }
        }

        VARIABLE extended_fld_device_status
        {
            REDEFINE LABEL DCT_EXTENDED_FLD_DEVICE_STATUS_SWA;
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "The battery unit has energy for less than 30 days.", HARDWARE },
                { 0x02, "One of the device variables is not valid.", SOFTWARE },
                { 0x04, "Battery failure: field device no longer powered." }
            }
        }

        VARIABLE standardized_status_0
        {
            REDEFINE LABEL DCT_STANDARDIZED_STATUS_0_SWA;
        }

        VARIABLE standardized_status_1
        {
            REDEFINE LABEL DCT_STANDARDIZED_STATUS_1_SWA;
        }

        VARIABLE analog_channel_saturated1
        {
            REDEFINE LABEL DCT_ANALOG_CHANNEL_SATURATED1_SWA;
				    REDEFINE TYPE BIT_ENUMERATED
				    {
				        {0x01, DCT_ANALOG_CHANNEL1, MISC}
				    }
        }

        VARIABLE standardized_status_2
        {
            REDEFINE LABEL DCT_STANDARDIZED_STATUS_2_SWA;
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_STANDARDIZED_STATUS_2_SUBDEVICE_LIST_CHANGED_SWA, PROCESS},
                { 0x02, DCT_STANDARDIZED_STATUS_2_SUBDEVICE_ADAPTER_DISCOVERED_SWA, MISC}
            }
        }

        VARIABLE standardized_status_3
        {
            REDEFINE LABEL DCT_STANDARDIZED_STATUS_3_SWA;
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, DCT_STANDARDIZED_STATUS_3_ADAPTER_UNABLE_GET_BANDWIDTH, DATA},
                { 0x04, DCT_STANDARDIZED_STATUS_3_ADAPTER_ASKED_FOR_BANDWIDTH, MISC}
                //{ 0x08, DCT_STANDARDIZED_STATUS_3_FIRMWARE_UPDATE_IN_PROCESS, MISC}
            }
        }
				// Universal 7

		VARIABLE analog_channel_fixed1
		{
		    REDEFINE TYPE BIT_ENUMERATED
		    {
		        {0x01, [ac1_fixed], MISC},
		        {0x02, [ac2_fixed], MISC},
		        {0x04, [ac3_fixed], MISC},
		        {0x08, [ac4_fixed], MISC}
		    }
		}

        // redefine this to get the unit codes.
        COMMAND read_dynamic_variables_and_pv_current
        {
            REDEFINE TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status,
                    PV.DAQ.ANALOG_VALUE,
                    PV.DEVICE_VARIABLE.DIGITAL_UNITS <0xFF>, PV.DEVICE_VARIABLE.DIGITAL_VALUE,
                    SV.DEVICE_VARIABLE.DIGITAL_UNITS <0xFF>, SV.DEVICE_VARIABLE.DIGITAL_VALUE,
                    TV.DEVICE_VARIABLE.DIGITAL_UNITS <0xFF>, TV.DEVICE_VARIABLE.DIGITAL_VALUE,
                    QV.DEVICE_VARIABLE.DIGITAL_UNITS <0xFF>, QV.DEVICE_VARIABLE.DIGITAL_VALUE
                }
            }
        }

        /*****************************************************************************
        * Command 48
        */
        REDEFINE COMMAND read_additional_device_status
        {
            NUMBER 48;
            OPERATION READ;
            TRANSACTION 0
            {
                REQUEST
                {}
                REPLY
                {
                    response_code, device_status,
                    device_specific_status_0, device_specific_status_1, device_specific_status_2,
                    device_specific_status_3, device_specific_status_4, device_specific_status_5,

                    extended_fld_device_status, 0x00,

                    standardized_status_0, standardized_status_1, analog_channel_saturated1,
                    standardized_status_2, standardized_status_3
                }
            }

            TRANSACTION 1
            {
                REQUEST
                {
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),

                    extended_fld_device_status (INFO), 0x00,

                    standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                    standardized_status_2 (INFO), standardized_status_3 (INFO)
                }
                REPLY
                {
                    response_code, device_status,
                    device_specific_status_0, device_specific_status_1, device_specific_status_2,
                    device_specific_status_3, device_specific_status_4, device_specific_status_5,

                    extended_fld_device_status, 0x00,

                    standardized_status_0, standardized_status_1, analog_channel_saturated1,
                    standardized_status_2, standardized_status_3
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,            [no_command_specific_errors];
                5,  DATA_ENTRY_ERROR,   [too_few_data_bytes_recieved];
                6,  MISC_ERROR,         [xmtr_specific_command_error];
                8,  MISC_WARNING,       [update_in_progress];
                14, MISC_WARNING,       "Status bytes mismatch";
                16, MODE_ERROR,         [access_restricted];
            }
        }


#define CMD_9_DEVICE_VAR_RESPONSE_BLOCK(num) \
                    num, \
                    deviceVariables[num].CLASSIFICATION, \
                    deviceVariables[num].DIGITAL_UNITS <0xFF>, \
                    deviceVariables[num].DIGITAL_VALUE, \
                    deviceVariables[num].DATA_QUALITY <0xC0>, \
                    deviceVariables[num].LIMIT_STATUS <0x30>, \
                    deviceVariables[num].DEVICE_FAMILY_STATUS <0x0F>


        REDEFINE COMMAND read_device_variables_and_status
        {
            NUMBER 9;
            OPERATION READ;

            // To minimize read cycles by AMS Device Diagnostics you have to pack all vars shown
            // in Diagnostics in one transaction and be shure these are not included in another transaction.
            // If all device vars are in a single transaction only you can prevent flickerering in AMS and
            // you prevent useless multi cmd 9 transactions.

            // TRANSACTION 0 includes all transactions shown in Device Diagnostics.
            TRANSACTION 0
            {
                REQUEST
                {
                    0,
                    1,
                    2,
                    3,
                    4,
                    9,
                    243,
                    243
                }
                REPLY
                {
                    response_code, device_status,
                    extended_fld_device_status,

                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(0),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(1),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(2),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(3),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(4),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(9),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(243),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(243),

                    time_stamp
                }
            }

            // TRANSACTION 1 includes all other transactions.
            TRANSACTION 1
            {
                REQUEST
                {
                    5,
                    6,
                    7,
                    8,
                    8,
                    8,
                    8,
                    245
                }
                REPLY
                {
                    response_code, device_status,
                    extended_fld_device_status,

                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(5),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(6),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(7),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(8),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(8),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(8),
                    CMD_9_DEVICE_VAR_RESPONSE_BLOCK(8),

                    245,
                    PVVariables[245].CLASSIFICATION,
                    PVVariables[245].DIGITAL_UNITS <0xFF>,
                    PVVariables[245].DIGITAL_VALUE,
                    PVVariables[245].DATA_QUALITY <0xC0>,
                    PVVariables[245].LIMIT_STATUS <0x30>,
                    PVVariables[245].DEVICE_FAMILY_STATUS <0x0F>,

                    time_stamp
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                8,  MISC_WARNING,     [update_failure];
                14, MISC_WARNING,     "Dynamic Variables Returned for Device Variables"; // CPK - add this to standard dictionary
                16, MODE_ERROR,       [access_restricted];
                30, MISC_WARNING,     "Command Response Truncated"; // CPK  - add this to standard dictionary
            }
        }
    }
}

/*****************************************************************************
 * PV, when the primary variable is not mappable we can import the PV
 * standard DD to take care of the dynamic variable array, loop current
 * and ranging functions
 */
#ifndef _PDM_
IMPORT STANDARD _PV, DEVICE_REVISION 1, DD_REVISION 1
#else
IMPORT "PV1_1.ddl"
#endif
{
    EVERYTHING;


    REDEFINITIONS
    {

        COLLECTION OF COLLECTION primary_variable
        {
            MEMBERS
            {
            	REDEFINE DEVICE_VARIABLE, PVVariables[primary_variable_code];
            }
        }

        COLLECTION OF COLLECTION secondary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE, deviceVariables[secondary_variable_code];
            }
        }

        COLLECTION OF COLLECTION tertiary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE, deviceVariables[tertiary_variable_code];
            }
        }

        COLLECTION OF COLLECTION quaternary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE, deviceVariables[quaternary_variable_code];
            }
        }


        REDEFINE UNIT scaling_units_relation
        {
            PVVariables[primary_variable_code].DIGITAL_UNITS:
                upperRange_value,
                lowerRange_value
        }

        COLLECTION OF VARIABLE scaling
        {
            MEMBERS
            {
                ADD 			RANGE_UNITS, 				PVVariables[primary_variable_code].DIGITAL_UNITS,	[range_units], 										[range_units_help];
                REDEFINE 	UPPER_RANGE_VALUE,  upperRange_value,   															DCT_DEV_VAR_PV_UPPER_RANGE_VALUE,	[upper_range_value_help];
                REDEFINE 	LOWER_RANGE_VALUE,  lowerRange_value,   															DCT_DEV_VAR_PV_LOWER_RANGE_VALUE,	[lower_range_value_help];
            }
        }

        VARIABLE transfer_function
        {
      	  	REDEFINE LABEL DCT_VAR_PV_TRANSFER_FUNCTION_CODE;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            REDEFINE TYPE ENUMERATED(1)
            {
                { 0, DCT_TRANSFER_FUNCTION_LINEAR }
                { 4, DCT_TRANSFER_FUNCTION_CURVE }
            }
        }

        VARIABLE loopCurrent
        {
            REDEFINE LABEL DCT_SIMULATE_CURRENT;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            REDEFINE POST_EDIT_ACTIONS
            {
                method_write_simulation_value
            }
        }

				ARRAY OF COLLECTION dynamic_variables
				{
				    REDEFINE ELEMENTS
				    {
				        PRIMARY,        primary_variable;
				        SECONDARY,      secondary_variable;
				        TERTIARY,       tertiary_variable;
				        QUATERNARY,     quaternary_variable;
				    }
				}

				VARIABLE upperRange_value {
					REDEFINE LABEL DCT_UPPERRANGE_VALUE_SWA;
					REDEFINE HANDLING IF (( (lock_device_status_code & 1) != 0)||(transfer_function == 4)) {READ;} ELSE {READ&WRITE;}
				}

				VARIABLE lowerRange_value {
					REDEFINE LABEL DCT_LOWERRANGE_VALUE_SWA;
					REDEFINE HANDLING IF (( (lock_device_status_code & 1) != 0)||(transfer_function == 4)) {READ;} ELSE {READ&WRITE;}
				}
    }
}

/*****************************************************************************
 * Common Practice Commands
 */
#ifndef _PDM_
IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 9, DD_REVISION 1
#else
IMPORT "Common9.ddl"
#endif
{
    COMMAND write_device_variable;                  // Command  9

    COMMAND write_pv_damping_value;                 // Command 34
    COMMAND write_pv_range_values;                  // Command 35
    COMMAND set_pv_upper_range_value;               // Command 36
    COMMAND set_pv_lower_range_value;               // Command 37

    COMMAND enter_exit_fixed_pv_current_mode;       // Command 40
    COMMAND perform_self_test;                      // Command 41
    COMMAND perform_device_reset;                   // Command 42
    COMMAND set_pv_zero;                            // Command 43
    COMMAND write_pv_units;                         // Command 44
    COMMAND trim_pv_current_dac_zero;               // Command 45
    COMMAND trim_pv_current_dac_gain;               // Command 46
    COMMAND write_pv_transfer_function;             // COMMAND 47

    COMMAND read_device_variable_assignments;       // Command 50
    COMMAND write_device_variable_assignments;      // Command 51

    COMMAND write_device_variable_units;            // COMMAND 53
    //COMMAND read_device_variable_information;       // Command 54  // unused
    COMMAND write_number_of_response_preambles;     // Command 59

    COMMAND lock_device;                            // Command 71

    COMMAND read_io_system_capabilities;            // Command 74 (manually added to lib)

    COMMAND read_lock_device_state;                 // Command 76

    COMMAND write_io_system_master_mode;            // Command 87 (manually added to lib)
    COMMAND write_io_system_retry_count;			// Command 88

    COMMAND read_real_time_clock;                   // Command 90

    COMMAND write_burst_period;						// Command 103
    COMMAND write_burst_trigger;					// Command 104
    COMMAND read_burst_mode_configuration;			// Command 105
    COMMAND write_burst_device_variables;			// Command 107
    COMMAND write_burst_command_number;				// Command 108
    COMMAND burst_mode_control;						// Command 109

    COMMAND read_event_notification_summary;            // Command 115
    COMMAND write_event_notification_bit_mask;          // Command 116
    COMMAND write_event_notification_timing;            // Command 117
    COMMAND event_notification_control;                 // Command 118

    COMMAND read_country_code;                      // COMMAND 512
    COMMAND write_country_code;						// Command 513

    METHOD applied_rerange;
    METHOD zero_trim;
    METHOD return_to_normal;
    METHOD warning_message;
    METHOD device_self_test;
    METHOD device_master_reset;
    METHOD transmitter_loop_test;
    METHOD leave_fixed_current_mode;
    METHOD transmitter_dac_trim;
    METHOD lock_unlock_device;

	VARIABLE total_number_burst_messages;

    VARIABLE device_variable_code;

    VARIABLE max_io_cards;
    VARIABLE max_channels_per_io_card;
    VARIABLE max_sub_dev_per_channel;
    VARIABLE devices_detected;
    VARIABLE max_dr;
    VARIABLE retry_count;

    VARIABLE primary_variable_code;
    VARIABLE secondary_variable_code;
    VARIABLE tertiary_variable_code;
    VARIABLE quaternary_variable_code;

    VARIABLE current_date;
    VARIABLE current_time;
    VARIABLE last_clock_date;
    VARIABLE last_clock_time;

    VARIABLE burst_message_number;

    VARIABLE subdev_IO_card;
    VARIABLE subdev_channel;
    VARIABLE subdev_device_id;
    VARIABLE subdev_universal_command_revision;
    VARIABLE subdev_long_tag;
    VARIABLE stx_sent_to_subdev;
    VARIABLE ack_received_from_subdev;
    VARIABLE back_received_from_subdev;

    COLLECTION subdev_identity;
    COLLECTION subdev_stats;
    COLLECTION sub_device;


	COMMAND read_subdev_id_summary;
	COMMAND read_subdevice_burst_map;
	COMMAND write_subdevice_burst_map;
    COMMAND read_subdevice_statistics;   // command 86

    VARIABLE constZero;
    VARIABLE constOne;
 VARIABLE number_events_supported;

    VARIABLE time_of_first_unack_event;
    VARIABLE event_notification_retry_time_event;
    VARIABLE max_update_time_event;
    VARIABLE event_debounce_interval_event;
    VARIABLE device_specific_status_0_mask;
    VARIABLE device_specific_status_1_mask;
    VARIABLE device_specific_status_2_mask;
    VARIABLE device_specific_status_3_mask;
    VARIABLE device_specific_status_4_mask;
    VARIABLE device_specific_status_5_mask;
    VARIABLE device_specific_status_14_mask;
    VARIABLE device_specific_status_15_mask;
    VARIABLE device_specific_status_16_mask;
    VARIABLE device_specific_status_17_mask;
    VARIABLE device_specific_status_18_mask;
    VARIABLE device_specific_status_19_mask;
    VARIABLE device_specific_status_20_mask;
    VARIABLE device_specific_status_21_mask;
    VARIABLE device_specific_status_22_mask;
    VARIABLE device_specific_status_23_mask;
    VARIABLE device_specific_status_24_mask;
    VARIABLE device_specific_status_0_latched_value;
    VARIABLE device_specific_status_1_latched_value;
    VARIABLE device_specific_status_2_latched_value;
    VARIABLE device_specific_status_3_latched_value;
    VARIABLE device_specific_status_4_latched_value;
    VARIABLE device_specific_status_5_latched_value;
    VARIABLE device_specific_status_14_latched_value;
    VARIABLE device_specific_status_15_latched_value;
    VARIABLE device_specific_status_16_latched_value;
    VARIABLE device_specific_status_17_latched_value;
    VARIABLE device_specific_status_18_latched_value;
    VARIABLE device_specific_status_19_latched_value;
    VARIABLE device_specific_status_20_latched_value;
    VARIABLE device_specific_status_21_latched_value;
    VARIABLE device_specific_status_22_latched_value;
    VARIABLE device_specific_status_23_latched_value;
    VARIABLE device_specific_status_24_latched_value;
    VARIABLE config_change_counter_latched_value;


    VARIABLE burst_command_number;
    VARIABLE update_period;
    VARIABLE max_update_period;
    VARIABLE trigger_level;
    VARIABLE subdevice_missing;

    REDEFINITIONS
    {

#ifdef _PDM_
			REDEFINE COMMAND write_device_variable_assignments
			{
			    NUMBER 51;
			    OPERATION WRITE;
			    TRANSACTION
			    {
			        REQUEST
			        {
			            0xF5, secondary_variable_code,
			            tertiary_variable_code, quaternary_variable_code
			        }
			        REPLY
			        {
			            response_code, device_status,
			            0xF5, secondary_variable_code,
			            tertiary_variable_code, quaternary_variable_code
			        }
			    }
			    RESPONSE_CODES
			    {
			        0,  SUCCESS,          [no_command_specific_errors];
			        2,  DATA_ENTRY_ERROR, [invalid_selection];
			        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
			        6,  MISC_ERROR,       [xmtr_specific_command_error];
			        7,  MODE_ERROR,       [in_write_protect_mode];
			        16, MODE_ERROR,       [access_restricted];
			        32, MODE_ERROR,       [busy];
			    }
			}
#endif

#ifndef _PDM_
REDEFINE COMMAND write_country_code
{
    NUMBER 513;
    OPERATION WRITE;
    TRANSACTION 1
    {
        REQUEST
        {
            country_code, si_control        // see common tables DD
        }

        REPLY
        {
            response_code, device_status,
            country_code, si_control
        }
    }
    TRANSACTION 2
    {
        REQUEST
        {
            0x4A, 0x50, 0x01        // see common tables DD
        }

        REPLY
        {
            response_code, device_status,
            country_code, si_control
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        7,  MODE_ERROR,       [in_write_protect_mode];
        16, MODE_ERROR,       [access_restricted];
        32, MODE_ERROR,       [busy];
        33, MODE_ERROR,       [delayed_response_initiated];
        34, MODE_ERROR,       [delayed_response_running];
        35, MODE_ERROR,       [delayed_response_dead];
        36, MODE_ERROR,       [delayed_response_conflict];
    }
}
#endif

        VARIABLE trigger_level
        {
            REDEFINE LABEL DCT_LABEL_TRIGGER_LEVEL_MESSAGE_N;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
        }

        VARIABLE update_period
        {
            REDEFINE LABEL DCT_LABEL_UPDATE_PERIOD_N;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
#ifdef _PDM_
				    REDEFINE TYPE TIME_VALUE
				    {
				        DISPLAY_FORMAT "5.3f" ;
				        TIME_SCALE SECONDS ;
				    }
#endif
        }

        VARIABLE max_update_period
        {
            REDEFINE LABEL DCT_LABEL_MAX_UPDATE_PERIOD_N;
				    REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
#ifdef _PDM_
				    REDEFINE TYPE TIME_VALUE
				    {
				        DISPLAY_FORMAT "5.3f" ;
				        TIME_SCALE SECONDS ;
				    }
#endif
        }

        VARIABLE time_of_first_unack_event
        {
            REDEFINE LABEL DCT_TIME_OF_FIRST_UNACK_EVENT_SWA;
#ifdef _PDM_
				    REDEFINE TYPE TIME_VALUE;
#endif
        }

				VARIABLE event_notification_retry_time_event
				{
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
				}

        VARIABLE max_update_time_event
        {
            REDEFINE LABEL DCT_MAX_UPDATE_TIME_EVENT_SWA;
#ifdef _PDM_
						REDEFINE TYPE TIME_VALUE;
#endif
        }

#ifndef _PDM_
#define DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(num) \
    REDEFINE VARIABLE device_specific_status_##num##_mask \
        { \
            LABEL DCT_DEVICE_SPECIFIC_STATUS_##num##_MASK_1_SWA; \
            HELP DCT_DEVICE_SPECIFIC_STATUS_HELP; \
            CLASS LOCAL; \
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; } \
            TYPE BIT_ENUMERATED \
            { \
                { 0x01, DCT_BIT_0 }, { 0x02, DCT_BIT_1 }, { 0x04, DCT_BIT_2 }, \
                { 0x08, DCT_BIT_3 }, { 0x10, DCT_BIT_4 }, { 0x20, DCT_BIT_5 }, \
                { 0x40, DCT_BIT_6 }, { 0x80, DCT_BIT_7 } \
            } \
            DEFAULT_VALUE 0; \
        }

        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(0)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(1)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(2)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(3)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(4)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(5)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(14)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(15)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(16)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(17)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(18)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(19)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(20)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(21)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(22)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(23)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(24)
#else
#define DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(num_mask,num_MASK_1_SWA) \
    REDEFINE VARIABLE device_specific_status_##num_mask \
        { \
            LABEL DCT_DEVICE_SPECIFIC_STATUS_##num_MASK_1_SWA ; \
            HELP DCT_DEVICE_SPECIFIC_STATUS_HELP; \
            CLASS LOCAL; \
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; } \
            TYPE BIT_ENUMERATED \
            { \
                { 0x01, DCT_BIT_0 }, { 0x02, DCT_BIT_1 }, { 0x04, DCT_BIT_2 }, \
                { 0x08, DCT_BIT_3 }, { 0x10, DCT_BIT_4 }, { 0x20, DCT_BIT_5 }, \
                { 0x40, DCT_BIT_6 }, { 0x80, DCT_BIT_7 } \
            } \
            DEFAULT_VALUE 0; \
        }

        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(0_mask,0_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(1_mask,1_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(2_mask,2_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(3_mask,3_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(4_mask,4_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(5_mask,5_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(14_mask,14_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(15_mask,15_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(16_mask,16_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(17_mask,17_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(18_mask,18_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(19_mask,19_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(20_mask,20_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(21_mask,21_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(22_mask,22_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(23_mask,23_MASK_1_SWA)
        DEVICE_SPECIFIC_STATUS_MASK_REDEFINE(24_mask,24_MASK_1_SWA)
#endif

        VARIABLE burst_message_number
        {
            REDEFINE TYPE INDEX compat_burst_messages;
        }

      // cmd 79
        REDEFINE COMMAND write_device_variable
        {
            NUMBER 79;
            OPERATION COMMAND;
            TRANSACTION 0
            {
                REQUEST
                {
                    device_variable_code_simulation_enum (INFO, INDEX),
                    write_device_variable_code,
                    deviceVariablesSimulation[device_variable_code_simulation_enum].DIGITAL_UNITS <0xFF> (INFO),
                    deviceVariablesSimulation[device_variable_code_simulation_enum].DIGITAL_VALUE,
                    deviceVariablesSimulation[device_variable_code_simulation_enum].DATA_QUALITY <0xC0>,
                    deviceVariablesSimulation[device_variable_code_simulation_enum].LIMIT_STATUS <0x30>,
                    deviceVariablesSimulation[device_variable_code_simulation_enum].DEVICE_FAMILY_STATUS <0x0F>

                }
                REPLY
                {
                    response_code, device_status,
                    device_variable_code_simulation_enum (INFO,INDEX),
                    write_device_variable_code,
                    deviceVariablesSimulation[device_variable_code_simulation_enum].DIGITAL_UNITS <0xFF> (INFO),
                    deviceVariablesSimulation[device_variable_code_simulation_enum].DIGITAL_VALUE,
                    deviceVariablesSimulation[device_variable_code_simulation_enum].DATA_QUALITY <0xC0>,
                    deviceVariablesSimulation[device_variable_code_simulation_enum].LIMIT_STATUS <0x30>,
                    deviceVariablesSimulation[device_variable_code_simulation_enum].DEVICE_FAMILY_STATUS <0x0F>
                }
            }
            TRANSACTION 1
            {
                REQUEST
                {
                    243,
                    write_device_variable_code,
                    0x35,
                    deviceVariablesSimulation[243].DIGITAL_VALUE,
                    deviceVariablesSimulation[243].DATA_QUALITY <0xC0>,
                    deviceVariablesSimulation[243].LIMIT_STATUS <0x30>,
                    deviceVariablesSimulation[243].DEVICE_FAMILY_STATUS <0x0F>

                }
                REPLY
                {
                    response_code, device_status,
                    243,
                    write_device_variable_code,
                    0x35,
                    deviceVariablesSimulation[243].DIGITAL_VALUE,
                    deviceVariablesSimulation[243].DATA_QUALITY <0xC0>,
                    deviceVariablesSimulation[243].LIMIT_STATUS <0x30>,
                    deviceVariablesSimulation[243].DEVICE_FAMILY_STATUS <0x0F>
                }
            }
            TRANSACTION 2
            {
                REQUEST
                {
                    245,
                    write_device_variable_code,
                    0x27,
                    deviceVariablesSimulation[245].DIGITAL_VALUE,
                    deviceVariablesSimulation[245].DATA_QUALITY <0xC0>,
                    deviceVariablesSimulation[245].LIMIT_STATUS <0x30>,
                    deviceVariablesSimulation[245].DEVICE_FAMILY_STATUS <0x0F>

                }
                REPLY
                {
                    response_code, device_status,
                    245,
                    write_device_variable_code,
                    0x27,
                    deviceVariablesSimulation[245].DIGITAL_VALUE,
                    deviceVariablesSimulation[245].DATA_QUALITY <0xC0>,
                    deviceVariablesSimulation[245].LIMIT_STATUS <0x30>,
                    deviceVariablesSimulation[245].DEVICE_FAMILY_STATUS <0x0F>
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                10, DATA_ENTRY_ERROR, [invalid_write_device_variable];
                16, MODE_ERROR,       [access_restricted];
                17, DATA_ENTRY_ERROR, [invalid_device_variable];
                18, DATA_ENTRY_ERROR, [invalid_units];
                19, DATA_ENTRY_ERROR, [device_variable_not_allowed];
                32, MODE_ERROR,       [busy];
                33, MODE_ERROR,       [delayed_response_initiated];
                34, MODE_ERROR,       [delayed_response_running];
                35, MODE_ERROR,       [delayed_response_dead];
                36, MODE_ERROR,       [delayed_response_conflict];
            }
        }

      // cmd 84
        COMMAND read_subdev_id_summary
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    subdevice_index (INFO, INDEX)
                }
                REPLY
                {
                     response_code, device_status,
                     subdevice_index (INFO, INDEX),
                     subdevice_variables[subdevice_index].IO_CARD,
                     subdevice_variables[subdevice_index].CHANNEL,
                     subdevice_variables[subdevice_index].MANUFID,
                     subdevice_variables[subdevice_index].EXP_DEV_TYPE,
                     subdevice_variables[subdevice_index].DEV_ID,
                     subdevice_variables[subdevice_index].UNIV_CMD_REV,
                     subdevice_variables[subdevice_index].LONG_TAG
                }
            }
        }


        // cmd 86 Read Subdevice Statistics - Command
        COMMAND read_subdevice_statistics
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    subdevice_index (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    subdevice_index (INFO, INDEX),
                    subdevice_variables[subdevice_index].STAT_STX_CNT,
                    subdevice_variables[subdevice_index].STAT_ACK_CNT,
                    subdevice_variables[subdevice_index].STAT_BACK_CNT
                }
            }
        }

        //CMD 101
        REDEFINE COMMAND read_subdevice_burst_map
        {
            NUMBER 101;
            OPERATION READ;
            TRANSACTION 0  //For Burst messages
            {
                REQUEST
                {
                    constZero <0x80> (INFO), // MSB = 0 for burst messages
                    burst_message_number <0x7F> (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    constZero <0x80> (INFO),
                    burst_message_number <0x7F> (INFO, INDEX),
                    compat_burst_messages[burst_message_number].SUB_DEVICE_MISSING <0x8000>,
                    compat_burst_messages[burst_message_number].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 1  //For event 1
            {
                REQUEST
                {
                    0x80
                }
                REPLY
                {
                    response_code, device_status,
                    0x80,
                    compat_events[0].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[0].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 2  //For event 2
            {
                REQUEST
                {
                    0x81
                }
                REPLY
                {
                    response_code, device_status,
                    0x81,
                    compat_events[1].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[1].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 3  //For event 3
            {
                REQUEST
                {
                    0x82
                }
                REPLY
                {
                    response_code, device_status,
                    0x82,
                    compat_events[2].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[2].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 4  //For event 4
            {
                REQUEST
                {
                    0x83
                }
                REPLY
                {
                    response_code, device_status,
                    0x83,
                    compat_events[3].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[3].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 5  //For event 5
            {
                REQUEST
                {
                    0x84
                }
                REPLY
                {
                    response_code, device_status,
                    0x84,
                    compat_events[4].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[4].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
            }
        }

        //CMD 102
        REDEFINE COMMAND write_subdevice_burst_map
        {
            NUMBER 102;
            OPERATION WRITE;
            TRANSACTION 0  //For Burst messages
            {
                REQUEST
                {
                    constZero <0x80> (INFO), // MSB = 0 for burst messages
                    burst_message_number <0x7F> (INFO, INDEX),
                    constZero <0x8000>,
                    compat_burst_messages[burst_message_number].SUB_DEVICE_MAPPING <0x7FFF>
                }
                REPLY
                {
                    response_code, device_status,
                    constZero <0x80> (INFO),
                    burst_message_number <0x7F> (INFO, INDEX),
                    compat_burst_messages[burst_message_number].SUB_DEVICE_MISSING <0x8000>,
                    compat_burst_messages[burst_message_number].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 1  //For event 1
            {
                REQUEST
                {
                    0x80,
                    constZero <0x8000>,
                    compat_events[0].SUB_DEVICE_MAPPING <0x7FFF>
                }
                REPLY
                {
                    response_code, device_status,
                    0x80,
                    compat_events[0].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[0].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 2  //For event 2
            {
                REQUEST
                {
                    0x81,
                    constZero <0x8000>,
                    compat_events[1].SUB_DEVICE_MAPPING <0x7FFF>
                }
                REPLY
                {
                    response_code, device_status,
                    0x81,
                    compat_events[1].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[1].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 3  //For event 3
            {
                REQUEST
                {
                    0x82,
                    constZero <0x8000>,
                    compat_events[2].SUB_DEVICE_MAPPING <0x7FFF>
                }
                REPLY
                {
                    response_code, device_status,
                    0x82,
                    compat_events[2].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[2].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 4  //For event 4
            {
                REQUEST
                {
                    0x83,
                    constZero <0x8000>,
                    compat_events[3].SUB_DEVICE_MAPPING <0x7FFF>
                }
                REPLY
                {
                    response_code, device_status,
                    0x83,
                    compat_events[3].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[3].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            TRANSACTION 5  //For event 5
            {
                REQUEST
                {
                    0x84,
                    constZero <0x8000>,
                    compat_events[4].SUB_DEVICE_MAPPING <0x7FFF>
                }
                REPLY
                {
                    response_code, device_status,
                    0x84,
                    compat_events[4].SUB_DEVICE_MISSING <0x8000>,
                    compat_events[4].SUB_DEVICE_MAPPING <0x7FFF>
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                9,  DATA_ENTRY_ERROR, "Invalid subdevice index";
                16, MODE_ERROR,       [access_restricted];
            }
        }

      //CMD 103
        COMMAND write_burst_period
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    burst_message_number (INFO, INDEX),
                    compat_burst_messages[burst_message_number].UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].MAX_UPDATE_PERIOD
                }
                REPLY
                {
                    response_code, device_status,
                    burst_message_number (INFO, INDEX),
                    compat_burst_messages[burst_message_number].UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].MAX_UPDATE_PERIOD
                }
            }
        }

      //CMD 104
        COMMAND write_burst_trigger
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    burst_message_number (INFO, INDEX),
                    compat_burst_messages[burst_message_number].BURST_TRIGGER_MODE,
                    compat_burst_messages[burst_message_number].BURST_CLASSIFICATION,
                    compat_burst_messages[burst_message_number].BURST_UNIT,
                    compat_burst_messages[burst_message_number].TRIGGER_LEVEL
                }
                REPLY
                {
                    response_code, device_status,
                    burst_message_number (INFO, INDEX),
                    compat_burst_messages[burst_message_number].BURST_TRIGGER_MODE,
                    compat_burst_messages[burst_message_number].BURST_CLASSIFICATION,
                    compat_burst_messages[burst_message_number].BURST_UNIT,
                    compat_burst_messages[burst_message_number].TRIGGER_LEVEL
                }
            }
        }


        REDEFINE COMMAND read_burst_mode_configuration
        {
				    NUMBER 105;
				    OPERATION READ;
            TRANSACTION 0
            {
                REQUEST
                {
                    burst_message_number (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    compat_burst_messages[burst_message_number].BURST_MODE_SELECT,
                    0x1F,
                    compat_burst_vars[burst_message_number][0],
                    compat_burst_vars[burst_message_number][1],
                    compat_burst_vars[burst_message_number][2],
                    compat_burst_vars[burst_message_number][3],
                    compat_burst_vars[burst_message_number][4],
                    compat_burst_vars[burst_message_number][5],
                    compat_burst_vars[burst_message_number][6],
                    compat_burst_vars[burst_message_number][7],
                    burst_message_number (INFO, INDEX),
                    total_number_burst_messages,
                    compat_burst_messages[burst_message_number].COMMAND_NUMBER,
                    compat_burst_messages[burst_message_number].UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].MAX_UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].BURST_TRIGGER_MODE,
                    compat_burst_messages[burst_message_number].BURST_CLASSIFICATION,
                    compat_burst_messages[burst_message_number].BURST_UNIT,
                    compat_burst_messages[burst_message_number].TRIGGER_LEVEL

                }
            }
            TRANSACTION 1
            {
                REQUEST
                {
                    burst_message_number (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    compat_burst_messages[burst_message_number].BURST_MODE_SELECT,
                    0x1F,
                    compat_burst_vars[burst_message_number][0],
                    compat_burst_vars[burst_message_number][1],
                    compat_burst_vars[burst_message_number][2],
                    compat_burst_vars[burst_message_number][3],
                    compat_burst_vars[burst_message_number][4],
                    compat_burst_vars[burst_message_number][5],
                    compat_burst_vars[burst_message_number][6],
                    compat_burst_vars[burst_message_number][7],
                    burst_message_number (INFO, INDEX),
                    total_number_burst_messages,
                    compat_burst_messages[burst_message_number].COMMAND_NUMBER_ENUM,
                    compat_burst_messages[burst_message_number].UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].MAX_UPDATE_PERIOD,
                    compat_burst_messages[burst_message_number].BURST_TRIGGER_MODE,
                    compat_burst_messages[burst_message_number].BURST_CLASSIFICATION,
                    compat_burst_messages[burst_message_number].BURST_UNIT,
                    compat_burst_messages[burst_message_number].TRIGGER_LEVEL

                }
            }
				    RESPONSE_CODES
				    {
				        0,  SUCCESS,          [no_command_specific_errors];
				        6,  MISC_ERROR,       [xmtr_specific_command_error];
				        9,  MISC_ERROR,       "Invalid burst message";
				        32, MODE_ERROR,       [busy];
				    }
        }

        REDEFINE COMMAND write_burst_device_variables
        {
            NUMBER 107;
            OPERATION WRITE;
            TRANSACTION 1
            {
                REQUEST
                {
                     compat_burst_vars[burst_message_number][0],
                     compat_burst_vars[burst_message_number][1],
                     compat_burst_vars[burst_message_number][2],
                     compat_burst_vars[burst_message_number][3],
                     compat_burst_vars[burst_message_number][4],
                     compat_burst_vars[burst_message_number][5],
                     compat_burst_vars[burst_message_number][6],
                     compat_burst_vars[burst_message_number][7],
                     burst_message_number (INFO, INDEX)
                }

                REPLY
                {
                     response_code, device_status,
                     compat_burst_vars[burst_message_number][0],
                     compat_burst_vars[burst_message_number][1],
                     compat_burst_vars[burst_message_number][2],
                     compat_burst_vars[burst_message_number][3],
                     compat_burst_vars[burst_message_number][4],
                     compat_burst_vars[burst_message_number][5],
                     compat_burst_vars[burst_message_number][6],
                     compat_burst_vars[burst_message_number][7],
                     burst_message_number (INFO, INDEX)
                }
            }

            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, DCT_BurstVarsWriteErrorSWA;
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                8,  MISC_WARNING,     "Burst condition conflict";
                9,  DATA_ENTRY_ERROR, "Invalid burst message";
            }
        }

        REDEFINE COMMAND write_burst_command_number
        {
            NUMBER 108;
            OPERATION WRITE;

            TRANSACTION 1
            {
               REQUEST
                {
                    compat_burst_messages[burst_message_number].COMMAND_NUMBER,
                    burst_message_number (INFO, INDEX)
                }

                REPLY
                {
                    response_code, device_status,
                    compat_burst_messages[burst_message_number].COMMAND_NUMBER,
                    burst_message_number (INFO, INDEX)
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, DCT_BurstCmdWriteErrorSWA;
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                8,  MISC_WARNING,     "Burst condition conflict";
                9,  DATA_ENTRY_ERROR, "Invalid burst message";
            }
        }

        //CMD 109
        COMMAND burst_mode_control
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    compat_burst_messages[burst_message_number].BURST_MODE_SELECT,
                    burst_message_number (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    compat_burst_messages[burst_message_number].BURST_MODE_SELECT,
                    burst_message_number (INFO, INDEX)
                }
            }
        }

        //CMD 115
        REDEFINE COMMAND read_event_notification_summary
        {
            NUMBER 115;
            OPERATION READ;
            TRANSACTION
            {
                REQUEST
                {
                    compat_eventNumber (INFO, INDEX)
                }
                REPLY
                {
                    response_code, device_status,
                    compat_eventNumber (INFO, INDEX),
                    number_events_supported,
                    compat_events[compat_eventNumber].EVENT_STATUS<0xf0>,
                    compat_events[compat_eventNumber].EVENT_NOTIFICATION_CONTROL<0x0f>,
                    compat_events[compat_eventNumber].TIME_OF_FIRST_UNACK_EVENT,
                    compat_events[compat_eventNumber].EVENT_NOTIFICATION_RETRY_TIME,
                    compat_events[compat_eventNumber].MAX_UPDATE_TIME,
                    compat_events[compat_eventNumber].EVENT_DEBOUNCE_INTERVAL,
                    compat_events[compat_eventNumber].DEVICE_STATUS_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_0_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_1_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_2_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_3_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_4_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_5_MASK,
                    compat_events[compat_eventNumber].EXTENDED_FLD_DEVICE_STATUS_MASK,
                    compat_events[compat_eventNumber].DEV_OPERATING_MODE_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_0_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_1_MASK,
                    compat_events[compat_eventNumber].ANALOG_CHANNEL_SATURATED1_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_2_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_3_MASK,
                    compat_events[compat_eventNumber].ANALOG_CHANNEL_FIXED1_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_14_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_15_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_16_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_17_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_18_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_19_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_20_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_21_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_22_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_23_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_24_MASK
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                2,  DATA_ENTRY_ERROR, [invalid_selection];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
            }
        }

        //CMD 116
        COMMAND write_event_notification_bit_mask
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    compat_eventNumber (INFO, INDEX),
                    compat_events[compat_eventNumber].DEVICE_STATUS_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_0_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_1_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_2_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_3_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_4_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_5_MASK,
                    compat_events[compat_eventNumber].EXTENDED_FLD_DEVICE_STATUS_MASK,
                    compat_events[compat_eventNumber].DEV_OPERATING_MODE_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_0_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_1_MASK,
                    compat_events[compat_eventNumber].ANALOG_CHANNEL_SATURATED1_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_2_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_3_MASK,
                    compat_events[compat_eventNumber].ANALOG_CHANNEL_FIXED1_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_14_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_15_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_16_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_17_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_18_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_19_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_20_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_21_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_22_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_23_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_24_MASK
                }
                REPLY
                {
                    response_code, device_status,
                    compat_eventNumber (INFO, INDEX),
                    compat_events[compat_eventNumber].DEVICE_STATUS_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_0_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_1_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_2_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_3_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_4_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_5_MASK,
                    compat_events[compat_eventNumber].EXTENDED_FLD_DEVICE_STATUS_MASK,
                    compat_events[compat_eventNumber].DEV_OPERATING_MODE_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_0_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_1_MASK,
                    compat_events[compat_eventNumber].ANALOG_CHANNEL_SATURATED1_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_2_MASK,
                    compat_events[compat_eventNumber].STANDARDIZED_STATUS_3_MASK,
                    compat_events[compat_eventNumber].ANALOG_CHANNEL_FIXED1_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_14_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_15_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_16_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_17_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_18_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_19_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_20_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_21_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_22_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_23_MASK,
                    compat_events[compat_eventNumber].DEVICE_SPECIFIC_STATUS_24_MASK
                }
            }
        }

        //CMD 117
        COMMAND write_event_notification_timing
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    compat_eventNumber (INFO, INDEX),
                    compat_events[compat_eventNumber].EVENT_NOTIFICATION_RETRY_TIME,
                    compat_events[compat_eventNumber].MAX_UPDATE_TIME,
                    compat_events[compat_eventNumber].EVENT_DEBOUNCE_INTERVAL
                }
                REPLY
                {
                    response_code, device_status,
                    compat_eventNumber (INFO, INDEX),
                    compat_events[compat_eventNumber].EVENT_NOTIFICATION_RETRY_TIME,
                    compat_events[compat_eventNumber].MAX_UPDATE_TIME,
                    compat_events[compat_eventNumber].EVENT_DEBOUNCE_INTERVAL
                }
            }
        }

        //CMD 118
        COMMAND event_notification_control
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    compat_eventNumber (INFO, INDEX),
                    compat_events[compat_eventNumber].EVENT_NOTIFICATION_CONTROL
                }
                REPLY
                {
                    response_code, device_status,
                    compat_eventNumber (INFO, INDEX),
                    compat_events[compat_eventNumber].EVENT_NOTIFICATION_CONTROL
                }
            }
        }

		VARIABLE primary_variable_code
		{
    		REDEFINE TYPE INDEX PVVariables;
    		REDEFINE LABEL DCT_PRIMARY_VARIABLE_CODE_SWA;
				REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
		}

				VARIABLE si_control_codes       //Standard dictionary
				{
				    REDEFINE TYPE ENUMERATED
				    {
				        { 0, DCT_EN_SI_CONTROL_RESTRICTIONABSENT, "No restrictions" },
				        { 1, DCT_EN_SI_CONTROL_RESTRICTIONPRESENT, "Restricted to SI Unit codes only" }

				    }
				}

       REDEFINE VARIABLE si_control
        {
            LABEL DCT_SI_CONTROL_SWA;
            HELP  DCT_SI_CONTROL_HELP;
            CLASS DEVICE;
            HANDLING IF (((lock_device_status_code & 1) != 0)||(country_code == en_country_code_Japan)) {READ;} ELSE {READ&WRITE;}
            TYPE ENUMERATED
            {
                { 0, DCT_EN_SI_CONTROL_RESTRICTIONABSENT  },  // No restriction
                { 1, DCT_EN_SI_CONTROL_RESTRICTIONPRESENT }   // Restricted to SI units
            }
            DEFAULT_VALUE 1;
        }

		REDEFINE VARIABLE devices_detected
		{
            LABEL DCT_DEVICES_DETECTED_SWA;
            HANDLING READ;
			TYPE ENUMERATED (2)
			{
				{1, "0"}
				{2, "1"}
				{3, "2"}
				{4, "3"}
				{5, "4"}
            }
			CLASS DYNAMIC;
            DEFAULT_VALUE 1;
		}

		// Variable Labels redefinitios - Common 9
		VARIABLE country_code {
			REDEFINE LABEL DCT_CONTRY_CODE_SWA;
			REDEFINE TYPE ENUMERATED(2)
		   {
		    {16710, "Afghanistan"},
		    {16716, "Albania"},
		    {17498, "Algeria"},
		    {16723, "American Samoa"},
		    {16708, "Andorra"},
		    {16719, "Angola"},
		    {16713, "Anguilla"},
		    {16721, "Antarctica"},
		    {16711, "Antigua And Barbuda"},
		    {16722, "Argentina"},
		    {16717, "Armenia"},
		    {16727, "Aruba"},
		    {16725, "Australia"},
		    {16724, "Austria"},
		    {16730, "Azerbaijan"},
		    {16979, "Bahamas"},
		    {16968, "Bahrain"},
		    {16964, "Bangladesh"},
		    {16962, "Barbados"},
		    {16985, "Belarus"},
		    {16965, "Belgium"},
		    {16986, "Belize"},
		    {16970, "Benin"},
		    {16973, "Bermuda"},
		    {16980, "Bhutan"},
		    {16975, "Bolivia"},
		    {16961, "Bosnia And Herzegowina"},
		    {16983, "Botswana"},
		    {16982, "Bouvet Island"},
		    {16978, "Brazil"},
		    {18767, "British Indian Ocean Territory"},
		    {16974, "Brunei Darussalam"},
		    {16967, "Bulgaria"},
		    {16966, "Burkina Faso"},
		    {16969, "Burundi"},
		    {19272, "Cambodia"},
		    {17229, "Cameroon"},
		    {17217, "Canada"},
		    {17238, "Cape Verde"},
		    {19289, "Cayman Islands"},
		    {17222, "Central African Republic"},
		    {21572, "Chad"},
		    {17228, "Chile"},
		    {17230, "China"},
		    {17240, "Christmas Island"},
		    {17219, "Cocos (Keeling) Islands"},
		    {17231, "Colombia"},
		    {19277, "Comoros"},
		    {17223, "Congo"},
		    {17227, "Cook Islands"},
		    {17234, "Costa Rica"},
		    {17225, "Cote d'Ivoire"},
		    {18514, "Croatia (local name: Hrvatska)"},
		    {17237, "Cuba"},
		    {17241, "Cyprus"},
		    {17242, "Czech Republic"},
		    {17483, "Denmark"},
		    {17482, "Djibouti"},
		    {17485, "Dominica"},
		    {17487, "Dominican Republic"},
		    {21584, "East Timor"},
		    {17731, "Ecuador"},
		    {17735, "Egypt"},
		    {21334, "El Salvador"},
		    {18257, "Equatorial Guinea"},
		    {17746, "Eritrea"},
		    {17733, "Estonia"},
		    {17748, "Ethiopia"},
		    {17749, "Europe (SSGFI only)"},
		    {17995, "Falkland Islands (Malvinas)"},
		    {17999, "Faroe Islands"},
		    {17994, "Fiji"},
		    {17993, "Finland"},
		    {18002, "France"},
		    {18008, "France, Metropolitan"},
		    {18246, "French Guiana"},
		    {20550, "French Polynesia"},
		    {21574, "French Southern Territories"},
		    {18241, "Gabon"},
		    {18253, "Gambia"},
		    {18245, "Georgia"},
		    {17477, "Germany"},
		    {18248, "Ghana"},
		    {18249, "Gibraltar"},
		    {18258, "Greece"},
		    {18252, "Greenland"},
		    {18244, "Grenada"},
		    {18256, "Guadeloupe"},
		    {18261, "Guam"},
		    {18260, "Guatemala"},
		    {18254, "Guinea"},
		    {18263, "Guinea-Bissau"},
		    {18265, "Guyana"},
		    {18516, "Haiti"},
		    {18509, "Heard And Mc Donald Islands"},
		    {18510, "Honduras"},
		    {18507, "Hong Kong"},
		    {18517, "Hungary"},
		    {18771, "Iceland"},
		    {18766, "India"},
		    {18756, "Indonesia"},
		    {18761, "International (SSGFI only)"},
		    {18770, "Iran (Islamic Republic Of)"},
		    {18769, "Iraq"},
		    {18757, "Ireland"},
		    {18764, "Israel"},
		    {18772, "Italy"},
		    {19021, "Jamaica"},
		    {19024, "Japan"},
		    {19023, "Jordan"},
		    {19290, "Kazakhstan"},
		    {19269, "Kenya"},
		    {19273, "Kiribati"},
		    {19280, "Korea, Democratic People's Republic Of"},
		    {19282, "Korea, Republic Of"},
		    {19287, "Kuwait"},
		    {19271, "Kyrgyzstan"},
		    {19521, "Lao People's Democratic Republic"},
		    {19542, "Latvia"},
		    {19522, "Lebanon"},
		    {19539, "Lesotho"},
		    {19538, "Liberia"},
		    {19545, "Libyan Arab Jamahiriya"},
		    {19529, "Liechtenstein"},
		    {19540, "Lithuania"},
		    {19541, "Luxembourg"},
		    {19791, "Macau"},
		    {19787, "Macedonia, The Former Yugoslav Republic Of"},
		    {19783, "Madagascar"},
		    {19799, "Malawi"},
		    {19801, "Malaysia"},
		    {19798, "Maldives"},
		    {19788, "Mali"},
		    {19796, "Malta"},
		    {19784, "Marshall Islands"},
		    {19793, "Martinique"},
		    {19794, "Mauritania"},
		    {19797, "Mauritius"},
		    {22868, "Mayotte"},
		    {19800, "Mexico"},
		    {17997, "Micronesia, Federated States Of"},
		    {19780, "Moldova, Republic Of"},
		    {19779, "Monaco"},
		    {19790, "Mongolia"},
		    {19795, "Montserrat"},
		    {19777, "Morocco"},
		    {19802, "Mozambique"},
		    {19789, "Myanmar"},
		    {20033, "Namibia"},
		    {20050, "Nauru"},
		    {20048, "Nepal"},
		    {20044, "Netherlands"},
		    {16718, "Netherlands Antilles"},
		    {20035, "New Caledonia"},
		    {20058, "New Zealand"},
		    {20041, "Nicaragua"},
		    {20037, "Niger"},
		    {20039, "Nigeria"},
		    {20053, "Niue"},
		    {20038, "Norfolk Island"},
		    {19792, "Northern Mariana Islands"},
		    {20047, "Norway"},
		    {20301, "Oman"},
		    {20555, "Pakistan"},
		    {20567, "Palau"},
		    {20545, "Panama"},
		    {20551, "Papua New Guinea"},
		    {20569, "Paraguay"},
		    {20549, "Peru"},
		    {20552, "Philippines"},
		    {20558, "Pitcairn"},
		    {20556, "Poland"},
		    {20564, "Portugal"},
		    {20562, "Puerto Rico"},
		    {20801, "Qatar"},
		    {21061, "Reunion"},
		    {21071, "Romania"},
		    {21077, "Russian Federation"},
		    {21079, "Rwanda"},
		    {19278, "Saint Kitts And Nevis"},
		    {19523, "Saint Lucia"},
		    {22083, "Saint Vincent And The Grenadines"},
		    {22355, "Samoa"},
		    {21325, "San Marino"},
		    {21332, "Sao Tome And Principe"},
		    {21313, "Saudi Arabia"},
		    {21326, "Senegal"},
		    {21315, "Seychelles"},
		    {21324, "Sierra Leone"},
		    {21319, "Singapore"},
		    {21323, "Slovakia (Slovak Republic)"},
		    {21321, "Slovenia"},
		    {21314, "Solomon Islands"},
		    {21327, "Somalia"},
		    {23105, "South Africa"},
		    {18259, "South Georgia And The South Sandwich Islands"},
		    {17747, "Spain"},
		    {19531, "Sri Lanka"},
		    {21320, "St. Helena"},
		    {20557, "St. Pierre And Miquelon"},
		    {21316, "Sudan"},
		    {21330, "Suriname"},
		    {21322, "Svalbard And Jan Mayen Islands"},
		    {21338, "Swaziland"},
		    {21317, "Sweden"},
		    {17224, "Switzerland"},
		    {21337, "Syrian Arab Republic"},
		    {21591, "Taiwan, Province Of China"},
		    {21578, "Tajikistan"},
		    {21594, "Tanzania, United Republic Of"},
		    {21576, "Thailand"},
		    {21575, "Togo"},
		    {21579, "Tokelau"},
		    {21583, "Tonga"},
		    {21588, "Trinidad And Tobago"},
		    {21582, "Tunisia"},
		    {21586, "Turkey"},
		    {21581, "Turkmenistan"},
		    {21571, "Turks And Caicos Islands"},
		    {21590, "Tuvalu"},
		    {21831, "Uganda"},
		    {21825, "Ukraine"},
		    {16709, "United Arab Emirates"},
		    {18242, "United Kingdom (UK)"},
		    {21843, "United States"},
		    {21837, "United States Minor Outlying Islands"},
		    {21849, "Uruguay"},
		    {21850, "Uzbekistan"},
		    {22101, "Vanuatu"},
		    {22081, "Vatican City State (Holy See)"},
		    {22085, "Venezuela"},
		    {22094, "Viet Nam"},
		    {22087, "Virgin Islands (British)"},
		    {22089, "Virgin Islands (U.S.)"},
		    {22342, "Wallis And Futuna Islands"},
		    {17736, "Western Sahara"},
		    {22853, "Yemen"},
		    {22869, "Yugoslavia"},
		    {23122, "Zaire"},
		    {23117, "Zambia"},
		    {23127, "Zimbabwe"}
		   }
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }

            REDEFINE POST_EDIT_ACTIONS
            {
                method_country_code_edited
            }


  }
		VARIABLE master_mode_code       //Standard dictionary
		{
		    REDEFINE TYPE ENUMERATED
		    {
		        { 0, "Secondary Master", "Secondary Master" },
		        { 1, "Primary Master",   "Primary Master"   }

		    }
		}
		VARIABLE master_mode {
			REDEFINE LABEL DCT_MASTER_MODE_SWA;
			REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
		}

		VARIABLE retry_count {
			REDEFINE LABEL DCT_RETRY_COUNT_SWA;
				REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
		}

		VARIABLE secondary_variable_code {
			REDEFINE LABEL DCT_SECONDARY_VARIABLE_CODE_SWA;
			REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
	    REDEFINE POST_EDIT_ACTIONS
	    {
	        warning_message,
	        method_read_non_primary_variable_code
	    }
		}

		VARIABLE tertiary_variable_code {
			REDEFINE LABEL DCT_TERTIARY_VARIABLE_CODE_SWA;
			REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
	    REDEFINE POST_EDIT_ACTIONS
	    {
	        warning_message,
	        method_read_non_primary_variable_code
	    }
		}

		VARIABLE quaternary_variable_code {
			REDEFINE LABEL DCT_QUARTERNARY_VARIABLE_CODE_SWA;
			REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
	    REDEFINE POST_EDIT_ACTIONS
	    {
	        warning_message,
	        method_read_non_primary_variable_code
	    }
		}

        VARIABLE current_date
        {
            REDEFINE LABEL DCT_CURRENT_DATE_SWA;
        }

				VARIABLE current_time
				{
						REDEFINE LABEL DCT_CURRENT_TIME_SWA;
#ifdef _PDM_
				    REDEFINE TYPE UNSIGNED_INTEGER(4)
				    {
				        SCALING_FACTOR 0.00003125;
				    }
				    CONSTANT_UNIT "s";
#else
						REDEFINE TYPE TIME_VALUE;
#endif

				}

        REDEFINE COLLECTION OF VARIABLE subdev_identity
        {
            MEMBERS
            {
                IO_CARD,            subdev_IO_card,                        "IO Card";
                CHANNEL,            subdev_channel,                        "Channel";
                MANUFID,            subdev_manufacturer_id,                "Mfg ID";
                EXP_DEV_TYPE,       subdev_expanded_device_type,           "Device Type";
                DEV_ID,             subdev_device_id,                      "Device ID";
                UNIV_CMD_REV,       subdev_universal_command_revision,     "Univ Rev";
                LONG_TAG,           subdev_long_tag,                       "Long Tag or Message";
                POLL_ADR,           subdev_poll_adr,                       "Polling Address";
                SHORT_TAG,          subdev_short_tag,                      "Tag";
            }
        }

      VARIABLE lock_device_status_code
      {
          REDEFINE LABEL DCT_LOCK_STATUS;
          REDEFINE CLASS DYNAMIC;

          REDEFINE TYPE BIT_ENUMERATED
          {
              { 0x01, lock_device_status_codes(0x01) },
              { 0x02, lock_device_status_codes(0x02) },
              { 0x04, lock_device_status_codes(0x04) },
              { 0x08, lock_device_status_codes(0x08) },
              { 0x10, lock_device_status_codes(0x10) }
          }

      }
      VARIABLE lock_device_status_codes
      {
			    REDEFINE TYPE ENUMERATED
			    {
			        { 0x01, "Device Locked" },
			        { 0x02, "Lock is Permanent" },
			        { 0x04, "Locked by Primary Master (Reset if Secondary Master)" },
			        { 0x08, "Configuration Cannot Be Changed" },
			        { 0x10, "Locked by Gateway" }
			    }
      }
   }
}

METHOD method_read_non_primary_variable_code
{
#ifndef _PDM_
    VALIDITY FALSE;
#endif
    LABEL DCT_METHOD_READ_NON_PV_CODE;
    DEFINITION
		{
			char status[STATUS_SIZE];
			long var_id[4];
			int secondary_var;
			int tertiary_var;
			int quaternary_var;

			IGNORE_NO_DEVICE();
			IGNORE_ALL_DEVICE_STATUS();
			IGNORE_ALL_RESPONSE_CODES();
			XMTR_IGNORE_ALL_DEVICE_STATUS();
			XMTR_IGNORE_ALL_RESPONSE_CODES();
			XMTR_IGNORE_NO_DEVICE();

			secondary_var = ivar_value(secondary_variable_code);
			tertiary_var = ivar_value(tertiary_variable_code);
			quaternary_var = ivar_value(quaternary_variable_code);

			send_command_trans(9,0);
			send_command_trans(9,1);

			iassign(secondary_variable_code,0);
			iassign(secondary_variable_code,secondary_var);
			iassign(tertiary_variable_code,0);
			iassign(tertiary_variable_code,tertiary_var);
			iassign(quaternary_variable_code,0);
			iassign(quaternary_variable_code,quaternary_var);

			save_values();
    }
}

/*
 * Device Variables Configuration
 */

//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_temperature
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_temperature_Value
{
    LABEL DCT_DEV_VAR_TEMPERATURE_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_Unit
{
    LABEL DCT_DEV_VAR_TEMPERATURE_UNIT;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TEMPERATURE_UNIT
}


VARIABLE dev_var_temperature_sensor_serial_number
{
    LABEL DCT_DEV_VAR_TEMPERATURE_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_temperature_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_TEMPERATURE_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_TEMPERATURE_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_damping_value
{
    LABEL DCT_DEV_VAR_TEMPERATURE_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_minimum_span
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_classification
{
    LABEL DCT_DEV_VAR_TEMPERATURE_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_temperature_device_family
{
    LABEL DCT_DEV_VAR_TEMPERATURE_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_temperature_update_time_period
{
    LABEL DCT_DEV_VAR_TEMPERATURE_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_temperature_data_quality
{
    LABEL DCT_DEV_VAR_TEMPERATURE_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_temperature_device_family_status
{
    LABEL DCT_DEV_VAR_TEMPERATURE_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_temperature_limit_status
{
    LABEL DCT_DEV_VAR_TEMPERATURE_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_temperature
{
    LABEL DCT_DEV_VAR_TEMPERATURE;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_temperature_Unit,                  DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_temperature_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_temperature_sensor_serial_number,  [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_temperature_upper_sensor_limit,    DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_temperature_lower_sensor_limit,    DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_temperature_damping_value,         DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_temperature_minimum_span,          DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_temperature_classification,        [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_temperature_device_family,         [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_temperature_update_time_period;
        DATA_QUALITY,              dev_var_temperature_data_quality,          [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_temperature_device_family_status,  [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_temperature_limit_status,          [limit_status], [limit_status_help];
    }
}

//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_temperature_min
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_temperature_min_Value
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_min_Unit
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_UNIT;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TEMPERATURE_UNIT
}


VARIABLE dev_var_temperature_min_sensor_serial_number
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_temperature_min_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_min_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_min_damping_value
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_min_minimum_span
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_min_classification
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_temperature_min_device_family
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_temperature_min_update_time_period
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_temperature_min_data_quality
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_temperature_min_device_family_status
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_temperature_min_limit_status
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_temperature_min
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MIN;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_temperature_min_Unit,                     DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_temperature_min_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_temperature_min_sensor_serial_number,     [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_temperature_min_upper_sensor_limit,       DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_temperature_min_lower_sensor_limit,       DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_temperature_min_damping_value,            DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_temperature_min_minimum_span,             DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_temperature_min_classification,           [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_temperature_min_device_family,            [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_temperature_min_update_time_period;
        DATA_QUALITY,              dev_var_temperature_min_data_quality,             [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_temperature_min_device_family_status,     [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_temperature_min_limit_status,             [limit_status], [limit_status_help];
    }
}


//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_temperature_max
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_temperature_max_Value
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_max_Unit
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_UNIT;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TEMPERATURE_UNIT
}


VARIABLE dev_var_temperature_max_sensor_serial_number
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_temperature_max_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_max_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_max_damping_value
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_max_minimum_span
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_temperature_max_classification
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_temperature_max_device_family
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_temperature_max_update_time_period
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_temperature_max_data_quality
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_temperature_max_device_family_status
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_temperature_max_limit_status
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_temperature_max
{
    LABEL DCT_DEV_VAR_TEMPERATURE_MAX;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_temperature_max_Unit,                     DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_temperature_max_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_temperature_max_sensor_serial_number,     [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_temperature_max_upper_sensor_limit,       DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_temperature_max_lower_sensor_limit,       DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_temperature_max_damping_value,            DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_temperature_max_minimum_span,             DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_temperature_max_classification,           [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_temperature_max_device_family,            [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_temperature_max_update_time_period;
        DATA_QUALITY,              dev_var_temperature_max_data_quality,             [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_temperature_max_device_family_status,     [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_temperature_max_limit_status,             [limit_status], [limit_status_help];
    }
}


//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_battery_voltage
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_battery_voltage_Value
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_Unit
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_UNIT;
    HANDLING READ;
    TYPE ENUMERATED  {
    	{0x3A, [v], [volts_help]}
    }
}


VARIABLE dev_var_battery_voltage_sensor_serial_number
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_battery_voltage_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_damping_value
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_minimum_span
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_classification
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_battery_voltage_device_family
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_battery_voltage_update_time_period
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_battery_voltage_data_quality
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_battery_voltage_device_family_status
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_battery_voltage_limit_status
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_battery_voltage
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE;
    MEMBERS
    {
        DIGITAL_UNITS, dev_var_battery_voltage_Unit, DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE, dev_var_battery_voltage_Value;
        SENSOR_SERIAL_NUMBER, dev_var_battery_voltage_sensor_serial_number, [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT, dev_var_battery_voltage_upper_sensor_limit, DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT, dev_var_battery_voltage_lower_sensor_limit, DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE, dev_var_battery_voltage_damping_value,     DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN, dev_var_battery_voltage_minimum_span, DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION, dev_var_battery_voltage_classification, [classification], [classification_help];
        DEVICE_FAMILY, dev_var_battery_voltage_device_family, [device_family], [device_family_help];
        UPDATE_TIME_PERIOD, dev_var_battery_voltage_update_time_period;
        DATA_QUALITY, dev_var_battery_voltage_data_quality, [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS, dev_var_battery_voltage_device_family_status, [family_status], [family_status_help];
        LIMIT_STATUS, dev_var_battery_voltage_limit_status, [limit_status], [limit_status_help];
    }
}


//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_consumed_energy
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_consumed_energy_Value
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_consumed_energy_Unit
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_UNIT;
    HANDLING READ;
    TYPE ENUMERATED {
    	{0xF0, DCT_AH}
    }
}


VARIABLE dev_var_consumed_energy_sensor_serial_number
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_consumed_energy_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_consumed_energy_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_consumed_energy_damping_value
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_consumed_energy_minimum_span
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_consumed_energy_classification
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_consumed_energy_device_family
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_consumed_energy_update_time_period
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_consumed_energy_data_quality
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_consumed_energy_device_family_status
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_consumed_energy_limit_status
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_consumed_energy
{
    LABEL DCT_DEV_VAR_CONSUMED_ENERGY;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_consumed_energy_Unit,                   DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_consumed_energy_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_consumed_energy_sensor_serial_number,   [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_consumed_energy_upper_sensor_limit,     DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_consumed_energy_lower_sensor_limit,     DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_consumed_energy_damping_value,          DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_consumed_energy_minimum_span,           DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_consumed_energy_classification,         [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_consumed_energy_device_family,          [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_consumed_energy_update_time_period;
        DATA_QUALITY,              dev_var_consumed_energy_data_quality,           [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_consumed_energy_device_family_status,   [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_consumed_energy_limit_status,           [limit_status], [limit_status_help];
    }
}


//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_estimated_lifetime
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_estimated_lifetime_Value
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_estimated_lifetime_Unit
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_UNIT;
    HANDLING READ;
    TYPE ENUMERATED {
    	{0x35, [day], [days_help]}
    }
}


VARIABLE dev_var_estimated_lifetime_sensor_serial_number
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_estimated_lifetime_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_estimated_lifetime_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_estimated_lifetime_damping_value
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_estimated_lifetime_minimum_span
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_estimated_lifetime_classification
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_estimated_lifetime_device_family
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_estimated_lifetime_update_time_period
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_estimated_lifetime_data_quality
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_estimated_lifetime_device_family_status
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_estimated_lifetime_limit_status
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_estimated_lifetime
{
    LABEL DCT_DEV_VAR_ESTIMATED_LIFETIME;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_estimated_lifetime_Unit,                      DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_estimated_lifetime_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_estimated_lifetime_sensor_serial_number,      [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_estimated_lifetime_upper_sensor_limit,        DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_estimated_lifetime_lower_sensor_limit,        DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_estimated_lifetime_damping_value,             DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_estimated_lifetime_minimum_span,              DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_estimated_lifetime_classification,            [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_estimated_lifetime_device_family,             [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_estimated_lifetime_update_time_period;
        DATA_QUALITY,              dev_var_estimated_lifetime_data_quality,              [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_estimated_lifetime_device_family_status,      [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_estimated_lifetime_limit_status,              [limit_status], [limit_status_help];
    }
}


//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_percent_range
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_percent_range_Value
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_percent_range_Unit
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_UNIT;
    HANDLING READ;
    TYPE ENUMERATED
    {
        ALL_UNITS_SORTED_EXTENDED
    }
}


VARIABLE dev_var_percent_range_sensor_serial_number
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_percent_range_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_percent_range_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_percent_range_damping_value
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_percent_range_minimum_span
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_percent_range_classification
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_percent_range_device_family
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_percent_range_update_time_period
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_percent_range_data_quality
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_percent_range_device_family_status
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_percent_range_limit_status
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_percent_range
{
    LABEL DCT_DEV_VAR_PERCENT_RANGE;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_percent_range_Unit,                    DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_percent_range_Value,                   [blank];
        SENSOR_SERIAL_NUMBER,      dev_var_percent_range_sensor_serial_number,    [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_percent_range_upper_sensor_limit,      DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_percent_range_lower_sensor_limit,      DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_percent_range_damping_value,           DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_percent_range_minimum_span,            DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_percent_range_classification,          [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_percent_range_device_family,           [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_percent_range_update_time_period;
        DATA_QUALITY,              dev_var_percent_range_data_quality,            [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_percent_range_device_family_status,    [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_percent_range_limit_status,            [limit_status], [limit_status_help];
    }
}


//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_current
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_current_Value
{
    LABEL DCT_DEV_VAR_CURRENT_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_current_Unit
{
    LABEL DCT_DEV_VAR_CURRENT_UNIT;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE ENUMERATED
    {
        ALL_UNITS_SORTED_EXTENDED
    }
}


VARIABLE dev_var_current_sensor_serial_number
{
    LABEL DCT_DEV_VAR_CURRENT_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_current_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_CURRENT_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_current_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_CURRENT_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_current_damping_value
{
    LABEL DCT_DEV_VAR_CURRENT_DAMPING_VALUE;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    CONSTANT_UNIT [sec];
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
        MIN_VALUE 0;
        MAX_VALUE 100;
    }
}

VARIABLE dev_var_current_minimum_span
{
    LABEL DCT_DEV_VAR_CURRENT_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_current_classification
{
    LABEL DCT_DEV_VAR_CURRENT_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_current_device_family
{
    LABEL DCT_DEV_VAR_CURRENT_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_current_update_time_period
{
    LABEL DCT_DEV_VAR_CURRENT_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_current_data_quality
{
    LABEL DCT_DEV_VAR_CURRENT_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_current_device_family_status
{
    LABEL DCT_DEV_VAR_CURRENT_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_current_limit_status
{
    LABEL DCT_DEV_VAR_CURRENT_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_current
{
    LABEL DCT_DEV_VAR_CURRENT;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_current_Unit,                 DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_current_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_current_sensor_serial_number, [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_current_upper_sensor_limit,   DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_current_lower_sensor_limit,   DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_current_damping_value,        DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_current_minimum_span,         DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_current_classification,       [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_current_device_family,        [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_current_update_time_period;
        DATA_QUALITY,              dev_var_current_data_quality,         [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_current_device_family_status, [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_current_limit_status,         [limit_status], [limit_status_help];
    }
}

//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_best_neighbour
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_best_neighbour_Value
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_best_neighbour_Unit
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_UNIT;
    HANDLING READ;
    TYPE ENUMERATED {
    	{0xF1, DCT_DB}
    }
}


VARIABLE dev_var_best_neighbour_sensor_serial_number
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_best_neighbour_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_best_neighbour_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_best_neighbour_damping_value
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_best_neighbour_minimum_span
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_best_neighbour_classification
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_best_neighbour_device_family
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_best_neighbour_update_time_period
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_best_neighbour_data_quality
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_best_neighbour_device_family_status
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_best_neighbour_limit_status
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_best_neighbour
{
    LABEL DCT_DEV_VAR_BEST_NEIGHBOUR;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_best_neighbour_Unit,                 DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_best_neighbour_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_best_neighbour_sensor_serial_number, [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_best_neighbour_upper_sensor_limit,   DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_best_neighbour_lower_sensor_limit,   DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_best_neighbour_damping_value,        DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_best_neighbour_minimum_span,         DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_best_neighbour_classification,       [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_best_neighbour_device_family,        [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_best_neighbour_update_time_period;
        DATA_QUALITY,              dev_var_best_neighbour_data_quality,         [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_best_neighbour_device_family_status, [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_best_neighbour_limit_status,         [limit_status], [limit_status_help];
    }
}

//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_second_best_neighbour
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_second_best_neighbour_Value
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_second_best_neighbour_Unit
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_UNIT;
    HANDLING READ;
    TYPE ENUMERATED {
    	{0xF1, DCT_DB}
    }
}


VARIABLE dev_var_second_best_neighbour_sensor_serial_number
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_second_best_neighbour_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_second_best_neighbour_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_second_best_neighbour_damping_value
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_second_best_neighbour_minimum_span
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_second_best_neighbour_classification
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_second_best_neighbour_device_family
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_second_best_neighbour_update_time_period
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_second_best_neighbour_data_quality
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_second_best_neighbour_device_family_status
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_second_best_neighbour_limit_status
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_second_best_neighbour
{
    LABEL DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_second_best_neighbour_Unit,                 DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_second_best_neighbour_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_second_best_neighbour_sensor_serial_number, [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_second_best_neighbour_upper_sensor_limit,   DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_second_best_neighbour_lower_sensor_limit,   DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_second_best_neighbour_damping_value,        DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_second_best_neighbour_minimum_span,         DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_second_best_neighbour_classification,       [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_second_best_neighbour_device_family,        [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_second_best_neighbour_update_time_period;
        DATA_QUALITY,              dev_var_second_best_neighbour_data_quality,         [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_second_best_neighbour_device_family_status, [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_second_best_neighbour_limit_status,         [limit_status], [limit_status_help];
    }
}

//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_battery_voltage_w_load
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_battery_voltage_w_load_Value
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_w_load_Unit
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_UNIT;
    HANDLING READ;
    TYPE ENUMERATED {
    	{58, [v], [volts_help]}
    }
}


VARIABLE dev_var_battery_voltage_w_load_sensor_serial_number
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_battery_voltage_w_load_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_w_load_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_w_load_damping_value
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_w_load_minimum_span
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_w_load_classification
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_battery_voltage_w_load_device_family
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_battery_voltage_w_load_update_time_period
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_battery_voltage_w_load_data_quality
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_battery_voltage_w_load_device_family_status
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_battery_voltage_w_load_limit_status
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_battery_voltage_w_load
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_battery_voltage_w_load_Unit,                       DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_battery_voltage_w_load_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_battery_voltage_w_load_sensor_serial_number,       [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_battery_voltage_w_load_upper_sensor_limit,         DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_battery_voltage_w_load_lower_sensor_limit,         DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_battery_voltage_w_load_damping_value,              DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_battery_voltage_w_load_minimum_span,               DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_battery_voltage_w_load_classification,             [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_battery_voltage_w_load_device_family,              [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_battery_voltage_w_load_update_time_period;
        DATA_QUALITY,              dev_var_battery_voltage_w_load_data_quality,               [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_battery_voltage_w_load_device_family_status,       [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_battery_voltage_w_load_limit_status,               [limit_status], [limit_status_help];
    }
}

//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_battery_voltage_wo_load_test
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_battery_voltage_wo_load_test_Value
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_wo_load_test_Unit
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_UNIT;
    HANDLING READ;
    TYPE ENUMERATED {
    	{58, [v], [volts_help]}
    }
}


VARIABLE dev_var_battery_voltage_wo_load_test_sensor_serial_number
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_battery_voltage_wo_load_test_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_wo_load_test_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_wo_load_test_damping_value
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_DAMPING_VALUE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_wo_load_test_minimum_span
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_battery_voltage_wo_load_test_classification
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_battery_voltage_wo_load_test_device_family
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_battery_voltage_wo_load_test_update_time_period
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_battery_voltage_wo_load_test_data_quality
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_battery_voltage_wo_load_test_device_family_status
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_battery_voltage_wo_load_test_limit_status
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_battery_voltage_wo_load_test
{
    LABEL DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_battery_voltage_wo_load_test_Unit,                       DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_battery_voltage_wo_load_test_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_battery_voltage_wo_load_test_sensor_serial_number,       [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_battery_voltage_wo_load_test_upper_sensor_limit,         DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_battery_voltage_wo_load_test_lower_sensor_limit,         DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_battery_voltage_wo_load_test_damping_value,              DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_battery_voltage_wo_load_test_minimum_span,               DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_battery_voltage_wo_load_test_classification,             [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_battery_voltage_wo_load_test_device_family,              [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_battery_voltage_wo_load_test_update_time_period;
        DATA_QUALITY,              dev_var_battery_voltage_wo_load_test_data_quality,               [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_battery_voltage_wo_load_test_device_family_status,       [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_battery_voltage_wo_load_test_limit_status,               [limit_status], [limit_status_help];
    }
}

//////////////////////////////////////////////////////////////////////////////
// device variable dev_var_normalized_consumed_energy
//////////////////////////////////////////////////////////////////////////////

VARIABLE dev_var_normalized_consumed_energy_Value
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_VALUE;
    CLASS CORRECTION & DYNAMIC;
    HANDLING READ;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_normalized_consumed_energy_Unit
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_UNIT;
    HANDLING READ;
    TYPE ENUMERATED  {
    	{0xF0, DCT_AH}
    }
}


VARIABLE dev_var_normalized_consumed_energy_sensor_serial_number
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_SENSOR_SERIAL_NUMBER;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE INTEGER (3);
}

VARIABLE dev_var_normalized_consumed_energy_upper_sensor_limit
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_UPPER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_normalized_consumed_energy_lower_sensor_limit
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_LOWER_SENSOR_LIMIT;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_normalized_consumed_energy_damping_value
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_DAMPING_VALUE;
    						HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_normalized_consumed_energy_minimum_span
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_MINIMUM_SPAN;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE dev_var_normalized_consumed_energy_classification
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_CLASSIFICATION;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_21
}

VARIABLE dev_var_normalized_consumed_energy_device_family
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_DEVICE_FAMILY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_TABLE_20
}

VARIABLE dev_var_normalized_consumed_energy_update_time_period
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_UPDATE_TIME_PERIOD;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE TIME_VALUE
    {
        DISPLAY_FORMAT "5.3f";
        EDIT_FORMAT "5.3f";
        TIME_SCALE SECONDS;
    }
    DEFAULT_VALUE 115200000; //60 * 60 * 32000;
}

VARIABLE dev_var_normalized_consumed_energy_data_quality
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_DATA_QUALITY;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_PROCESS_DATA_STATUS
}

VARIABLE dev_var_normalized_consumed_energy_device_family_status
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_DEVICE_FAMILY_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_DEVICE_FAMILY_STATUS
}

VARIABLE dev_var_normalized_consumed_energy_limit_status
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY_LIMIT_STATUS;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    ENUM_LIMIT_STATUS
}

COLLECTION OF VARIABLE dev_var_normalized_consumed_energy
{
    LABEL DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY;
    MEMBERS
    {
        DIGITAL_UNITS,             dev_var_normalized_consumed_energy_Unit,                   DCT_DEV_VAR_UNITS;
        DIGITAL_VALUE,             dev_var_normalized_consumed_energy_Value;
        SENSOR_SERIAL_NUMBER,      dev_var_normalized_consumed_energy_sensor_serial_number,   [sensor_serial_number], [sensor_serial_number_help];
        UPPER_SENSOR_LIMIT,        dev_var_normalized_consumed_energy_upper_sensor_limit,     DCT_DEV_VAR_UPPER_RANGE_LIMIT;
        LOWER_SENSOR_LIMIT,        dev_var_normalized_consumed_energy_lower_sensor_limit,     DCT_DEV_VAR_LOWER_RANGE_LIMIT;
        DAMPING_VALUE,             dev_var_normalized_consumed_energy_damping_value,          DCT_DEV_VAR_DAMPING_VALUE;
        MINIMUM_SPAN,              dev_var_normalized_consumed_energy_minimum_span,           DCT_DEV_VAR_MINIMUM_SPAN;
        CLASSIFICATION,            dev_var_normalized_consumed_energy_classification,         [classification], [classification_help];
        DEVICE_FAMILY,             dev_var_normalized_consumed_energy_device_family,          [device_family], [device_family_help];
        UPDATE_TIME_PERIOD,        dev_var_normalized_consumed_energy_update_time_period;
        DATA_QUALITY,              dev_var_normalized_consumed_energy_data_quality,           [process_data_quality], [process_data_quality_help];
        DEVICE_FAMILY_STATUS,      dev_var_normalized_consumed_energy_device_family_status,   [family_status], [family_status_help];
        LIMIT_STATUS,              dev_var_normalized_consumed_energy_limit_status,           [limit_status], [limit_status_help];
    }
}


//////////////////////////////////////////////////////////////////////////////
//
// device variables array
//
//////////////////////////////////////////////////////////////////////////////

ARRAY OF COLLECTION deviceVariables
{
    ELEMENTS
    {
        /*
         * If you ever change the deviceVariables elements list do not
         * forget to change in METHOD method_simulation the iteration!
         */
         3, dev_var_battery_voltage, DCT_DEV_VAR_BATTERY_VOLTAGE;
         7, dev_var_battery_voltage_w_load, DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD;
         8, dev_var_battery_voltage_wo_load_test, DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST;
         4, dev_var_consumed_energy, DCT_DEV_VAR_CONSUMED_ENERGY;
         243, dev_var_estimated_lifetime, DCT_DEV_VAR_ESTIMATED_LIFETIME;
         9, dev_var_normalized_consumed_energy, DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY;
         5, dev_var_best_neighbour, DCT_DEV_VAR_BEST_NEIGHBOUR;
         6, dev_var_second_best_neighbour, DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR;
         0, dev_var_temperature, DCT_DEV_VAR_TEMPERATURE;
         2, dev_var_temperature_max, DCT_DEV_VAR_TEMPERATURE_MAX;
         1, dev_var_temperature_min, DCT_DEV_VAR_TEMPERATURE_MIN;
    }
}

ARRAY OF COLLECTION deviceVariablesSimulation
{
    ELEMENTS
    {
        /*
         * If you ever change the deviceVariables elements list do not
         * forget to change in METHOD method_simulation the iteration!
         */
         3, dev_var_battery_voltage, DCT_DEV_VAR_BATTERY_VOLTAGE;
         7, dev_var_battery_voltage_w_load, DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD;
         8, dev_var_battery_voltage_wo_load_test, DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST;
         4, dev_var_consumed_energy, DCT_DEV_VAR_CONSUMED_ENERGY;
         245, dev_var_current, DCT_DEV_VAR_CURRENT;
         243, dev_var_estimated_lifetime, DCT_DEV_VAR_ESTIMATED_LIFETIME;
         9, dev_var_normalized_consumed_energy, DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY;
         5, dev_var_best_neighbour, DCT_DEV_VAR_BEST_NEIGHBOUR;
         6, dev_var_second_best_neighbour, DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR;
         0, dev_var_temperature, DCT_DEV_VAR_TEMPERATURE;
         2, dev_var_temperature_max, DCT_DEV_VAR_TEMPERATURE_MAX;
         1, dev_var_temperature_min, DCT_DEV_VAR_TEMPERATURE_MIN;
    }
}

VARIABLE device_variable_code_simulation
{
    HELP [index_into_dynamic_variables_array_help];
/*  APPHELP "This variable can represent measured or calculated type variables."
            " The numeric assigned to a Field Device variable can never be"
            " changed. If it is changed this would be a violation of the Forward"
            " Compatibility rules."
            " HART_VAR:XMTR VAR CODE"; */
    CLASS LOCAL;
    TYPE INDEX deviceVariablesSimulation;
}

VARIABLE device_variable_code_simulation_enum
{
    HELP [index_into_dynamic_variables_array_help];
    CLASS LOCAL;
    TYPE ENUMERATED {
        { 3, DCT_DEV_VAR_BATTERY_VOLTAGE }
        { 7, DCT_DEV_VAR_BATTERY_VOLTAGE_W_LOAD }
        { 8, DCT_DEV_VAR_BATTERY_VOLTAGE_WO_LOAD_TEST }
        { 4, DCT_DEV_VAR_CONSUMED_ENERGY }
        { 243, DCT_DEV_VAR_ESTIMATED_LIFETIME }
        { 245, DCT_DEV_VAR_CURRENT }
        { 9, DCT_DEV_VAR_NORMALIZED_CONSUMED_ENERGY }
        { 5, DCT_DEV_VAR_BEST_NEIGHBOUR }
        { 6, DCT_DEV_VAR_SECOND_BEST_NEIGHBOUR }
        { 0, DCT_DEV_VAR_TEMPERATURE }
        { 2, DCT_DEV_VAR_TEMPERATURE_MAX }
        { 1, DCT_DEV_VAR_TEMPERATURE_MIN }
    }
}

ARRAY OF COLLECTION PVVariables
{
 ELEMENTS
 {
		245, dev_var_current, DCT_DEV_VAR_CURRENT;
 }
}

/*
 * Relationships
 */

REFRESH table_index_rfsh
{
 table_index:
 y_value,
 x_value
}



UNIT dev_var_temperature_UnitRelation
{
    dev_var_temperature_Unit:
        dev_var_temperature_Value,
        dev_var_temperature_upper_sensor_limit,
        dev_var_temperature_lower_sensor_limit,
        dev_var_temperature_damping_value,
        dev_var_temperature_minimum_span
}

UNIT dev_var_temperature_min_UnitRelation
{
    dev_var_temperature_min_Unit:
        dev_var_temperature_min_Value,
        dev_var_temperature_min_upper_sensor_limit,
        dev_var_temperature_min_lower_sensor_limit,
        dev_var_temperature_min_damping_value,
        dev_var_temperature_min_minimum_span
}

UNIT dev_var_temperature_max_UnitRelation
{
    dev_var_temperature_max_Unit:
        dev_var_temperature_max_Value,
        dev_var_temperature_max_upper_sensor_limit,
        dev_var_temperature_max_lower_sensor_limit,
        dev_var_temperature_max_damping_value,
        dev_var_temperature_max_minimum_span
}

UNIT dev_var_battery_voltage_UnitRelation
{
    dev_var_battery_voltage_Unit:
        dev_var_battery_voltage_Value,
        dev_var_battery_voltage_upper_sensor_limit,
        dev_var_battery_voltage_lower_sensor_limit,
        dev_var_battery_voltage_damping_value,
        dev_var_battery_voltage_minimum_span
}

UNIT dev_var_consumed_energy_UnitRelation
{
    dev_var_consumed_energy_Unit:
        dev_var_consumed_energy_Value,
        dev_var_consumed_energy_upper_sensor_limit,
        dev_var_consumed_energy_lower_sensor_limit,
        dev_var_consumed_energy_damping_value,
        dev_var_consumed_energy_minimum_span
}

UNIT dev_var_estimated_lifetime_UnitRelation
{
    dev_var_estimated_lifetime_Unit:
        dev_var_estimated_lifetime_Value,
        dev_var_estimated_lifetime_upper_sensor_limit,
        dev_var_estimated_lifetime_lower_sensor_limit,
        dev_var_estimated_lifetime_damping_value,
        dev_var_estimated_lifetime_minimum_span
}

UNIT dev_var_percent_range_UnitRelation
{
    dev_var_percent_range_Unit:
        dev_var_percent_range_Value,
        dev_var_percent_range_upper_sensor_limit,
        dev_var_percent_range_lower_sensor_limit,
        dev_var_percent_range_damping_value,
        dev_var_percent_range_minimum_span
}

UNIT dev_var_current_UnitRelation
{
    dev_var_current_Unit:
        dev_var_current_Value,
        dev_var_current_upper_sensor_limit,
        dev_var_current_lower_sensor_limit,
        dev_var_current_minimum_span,
        y_value
}

UNIT dev_var_best_neighbour_UnitRelation
{
    dev_var_best_neighbour_Unit:
        dev_var_best_neighbour_Value,
        dev_var_best_neighbour_upper_sensor_limit,
        dev_var_best_neighbour_lower_sensor_limit,
        dev_var_best_neighbour_damping_value,
        dev_var_best_neighbour_minimum_span
}

UNIT dev_var_second_best_neighbour_UnitRelation
{
    dev_var_second_best_neighbour_Unit:
        dev_var_second_best_neighbour_Value,
        dev_var_second_best_neighbour_upper_sensor_limit,
        dev_var_second_best_neighbour_lower_sensor_limit,
        dev_var_second_best_neighbour_damping_value,
        dev_var_second_best_neighbour_minimum_span
}

UNIT dev_var_battery_voltage_w_load_UnitRelation
{
    dev_var_battery_voltage_w_load_Unit:
        dev_var_battery_voltage_w_load_Value,
        dev_var_battery_voltage_w_load_upper_sensor_limit,
        dev_var_battery_voltage_w_load_lower_sensor_limit,
        dev_var_battery_voltage_w_load_damping_value,
        dev_var_battery_voltage_w_load_minimum_span
}

UNIT dev_var_battery_voltage_wo_load_test_UnitRelation
{
    dev_var_battery_voltage_wo_load_test_Unit:
        dev_var_battery_voltage_wo_load_test_Value,
        dev_var_battery_voltage_wo_load_test_upper_sensor_limit,
        dev_var_battery_voltage_wo_load_test_lower_sensor_limit,
        dev_var_battery_voltage_wo_load_test_damping_value,
        dev_var_battery_voltage_wo_load_test_minimum_span
}

UNIT dev_var_normalized_consumed_energy_UnitRelation
{
    dev_var_normalized_consumed_energy_Unit:
        dev_var_normalized_consumed_energy_Value,
        dev_var_normalized_consumed_energy_upper_sensor_limit,
        dev_var_normalized_consumed_energy_lower_sensor_limit,
        dev_var_normalized_consumed_energy_damping_value,
        dev_var_normalized_consumed_energy_minimum_span
}


//#ifdef _PDM_
#define BURST_TRIGGER_UNIT_RELATION(num,num_UnitRelation)\
UNIT burst_trigger_units_##num_UnitRelation \
{\
		compat_burst_trig_units_##num :\
				compat_burst_trig_level_##num \
}
BURST_TRIGGER_UNIT_RELATION(0,0_UnitRelation)
BURST_TRIGGER_UNIT_RELATION(1,1_UnitRelation)
BURST_TRIGGER_UNIT_RELATION(2,2_UnitRelation)
BURST_TRIGGER_UNIT_RELATION(3,3_UnitRelation)
BURST_TRIGGER_UNIT_RELATION(4,4_UnitRelation)
BURST_TRIGGER_UNIT_RELATION(5,5_UnitRelation)
BURST_TRIGGER_UNIT_RELATION(6,6_UnitRelation)
BURST_TRIGGER_UNIT_RELATION(7,7_UnitRelation)
BURST_TRIGGER_UNIT_RELATION(8,8_UnitRelation)
BURST_TRIGGER_UNIT_RELATION(9,9_UnitRelation)
//#endif

// **********************************************************************************
// Wireless
// **********************************************************************************

#ifndef _PDM_
IMPORT STANDARD _WIRELESS, DEVICE_REVISION 1, DD_REVISION 1
#else
IMPORT "Wireless.ddl"
#endif
{
    EVERYTHING;
    REDEFINITIONS
    {
        DELETE COMMAND  read_network_access_mode;
        DELETE COMMAND  write_network_access_mode;
        DELETE VARIABLE network_access_mode;

        DELETE COMMAND  read_network_tag;
        DELETE COMMAND  write_network_tag;
        DELETE VARIABLE network_tag;

#ifdef _PDM_
				DELETE COMMAND read_join_status;
/*
				REDEFINE COMMAND read_join_status
				{
				    NUMBER 769;
				    OPERATION READ;
				    TRANSACTION
				    {
				        REQUEST {}
				        REPLY
				        {
				            response_code, device_status,
				            //dummy_16_bit_command,
				            reserved_bits<0xF0>, wireless_mode<0x0F>,
				            join_status_byte_hi,
				            join_status_byte_lo,
				            neighbor_count,
				            advertisements,
				            join_attempts,
				            join_retry_timer,
				            network_search_timer
				        }
				    }
				    RESPONSE_CODES
				    {
				        0,  SUCCESS,    [no_command_specific_errors];
				        6,  MISC_ERROR, [xmtr_specific_command_error];
				        32, MODE_ERROR, [busy];
				    }
				}
*/
#endif

		REDEFINE COMMAND write_join_key
		{
		    NUMBER 768;
		    OPERATION COMMAND;
		    TRANSACTION
		    {
		        REQUEST
		        {
		             join_key_1, join_key_2, join_key_3, join_key_4
		        }
		        REPLY
		        {
		            response_code, device_status,
		            join_key_1, join_key_2, join_key_3, join_key_4
		        }
		    }
		    RESPONSE_CODES
		    {
		        0,  SUCCESS,          [no_command_specific_errors];
		        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
		        6,  MISC_ERROR,       [xmtr_specific_command_error];
		        7,  MODE_ERROR,       [in_write_protect_mode];
		        16, MODE_ERROR,       [access_restricted];
		        32, MODE_ERROR,       [busy];
		        33, MODE_ERROR,       [delayed_response_initiated];
		        34, MODE_ERROR,       [delayed_response_running];
		        35, MODE_ERROR,       [delayed_response_dead];
		        36, MODE_ERROR,       [delayed_response_conflict];
		        65, DATA_ENTRY_ERROR, "Key change failed";
		    }
		}

        REDEFINE COMMAND write_network_id
        {
            NUMBER 773;
            OPERATION WRITE;
            TRANSACTION
            {
                REQUEST
                {
                    network_id
                }
                REPLY
                {
                    response_code, device_status, network_id
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,          [no_command_specific_errors];
                5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
                6,  MISC_ERROR,       [xmtr_specific_command_error];
                7,  MODE_ERROR,       [in_write_protect_mode];
                8,  MODE_ERROR,       DCT_NETWORK_ID_CHANGE_PENDING;
                16, MODE_ERROR,       [access_restricted];
                32, MODE_ERROR,       [busy];
                33, MODE_ERROR,       [delayed_response_initiated];
                34, MODE_ERROR,       [delayed_response_running];
                35, MODE_ERROR,       [delayed_response_dead];
                36, MODE_ERROR,       [delayed_response_conflict];
                65, DATA_ENTRY_ERROR, DCT_INVALID_NETWORK_ID;
            }
        }

//********************************************
//*******CHANGE FOR #18993: NOT TESTED********
//********************************************
//ORIGINAL CODE
/*
        VARIABLE radio_power_output
        {
            REDEFINE TYPE INTEGER
            {
                MIN_VALUE1   0;
                MAX_VALUE1   0;
                MIN_VALUE2  IF(si_control == 0) { 10; } ELSE { 0; }
                MAX_VALUE2  IF(si_control == 0) { 10; } ELSE { 0; }
            }
            REDEFINE LABEL DCT_RADIO_POWER_OUTPUT_SWA;
            REDEFINE HELP "only 0 and 10 allowed";
        }

*/

//MODIFIED
        REDEFINE VARIABLE radio_power_output
        {
						HANDLING
								IF
								(
									((lock_device_status_code & 1) != 0) ||
									(country_code == en_country_code_Japan) ||
#ifdef _PDM_
									JOIN_STATUS_EQU_x079F
#else
									(join_status == 0x079F)
#endif
								)
								{
									READ;
								}
								ELSE
								{
									READ&WRITE;
								}
            TYPE ENUMERATED
            {
                    {0, "0"}
                    {10, "10"}
            }

            LABEL DCT_RADIO_POWER_OUTPUT_SWA;
            HELP "only 0 and 10 allowed";
            CLASS HART;
            CONSTANT_UNIT "dBm";
        }

//******************************************************************************

    VARIABLE network_id
		{
		    REDEFINE TYPE UNSIGNED_INTEGER (2);
				REDEFINE LABEL DCT_NETWORK_ID_SWA;
				REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
		}

		VARIABLE join_shed_time
		{
            REDEFINE LABEL DCT_JOIN_SHED_TIME_LABEL;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
#if __TOKVER__ >= 800
            REDEFINE TYPE TIME_VALUE
            {
                DISPLAY_FORMAT ".0f" ;
                EDIT_FORMAT ".0f";
                TIME_SCALE SECONDS ;
            }
#else
            REDEFINE TYPE UNSIGNED_INTEGER(4)
            {
                SCALING_FACTOR 0.00003125;
            }
            REDEFINE CONSTANT_UNIT "s";
#endif
#ifdef _PDM_
						REDEFINE DEFAULT_VALUE 76800000;
#endif
        }

		VARIABLE join_mode
		{
			REDEFINE CLASS DYNAMIC;
            REDEFINE LABEL DCT_JOIN_MODE_SWA;
						REDEFINE HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            REDEFINE TYPE ENUMERATED
            {
                { 0, DCT_DONT_JOIN },
                { 1, join_mode_codes(1) },
                { 2, DCT_JOIN_ON_POWERUP_OR_RESET }
            }
		}

		REDEFINE VARIABLE join_key_1
		{
		    LABEL DCT_JOIN_KEY_1_SWA;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE UNSIGNED_INTEGER (4)
            {
                DISPLAY_FORMAT "08X";
                EDIT_FORMAT "08X";
            }
            DEFAULT_VALUE 0;
		}

        REDEFINE VARIABLE join_key_2
		{
		    LABEL DCT_JOIN_KEY_2_SWA;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE UNSIGNED_INTEGER (4)
            {
                DISPLAY_FORMAT "08X";
                EDIT_FORMAT "08X";
            }
            DEFAULT_VALUE 0;
		}

		REDEFINE VARIABLE join_key_3
		{
		    LABEL DCT_JOIN_KEY_3_SWA;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE UNSIGNED_INTEGER (4)
            {
                DISPLAY_FORMAT "08X";
                EDIT_FORMAT "08X";
            }
            DEFAULT_VALUE 0;
		}

		REDEFINE VARIABLE join_key_4
		{
		    LABEL DCT_JOIN_KEY_4_SWA;
            CLASS LOCAL;
            HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
            TYPE UNSIGNED_INTEGER (4)
            {
                DISPLAY_FORMAT "08X";
                EDIT_FORMAT "08X";
            }
            DEFAULT_VALUE 0;
		}

//********************************************
//*******CHANGE FOR #19123:           ********
//********************************************

        REDEFINE VARIABLE shed_time
        {
            LABEL DCT_SHED_TIME_SWA;
            HELP  "Shed Time";
            CLASS DEVICE;
            HANDLING WRITE;
            TYPE TIME_VALUE
            {
                DISPLAY_FORMAT "5.0f" ;
                EDIT_FORMAT "5.0f";
                TIME_SCALE SECONDS ;
            }
			DEFAULT_VALUE 57600000;
		}

        // Variable Label redefinitions - Wireless
		VARIABLE wireless_mode
		{
			REDEFINE LABEL DCT_WIRELESS_MODE_SWA;
		}

        VARIABLE join_process_status_codes
        {
            REDEFINE TYPE ENUMERATED (2)
            {
                { 0x0001, DCT_JOIN_STATE_NETWORK_PACKETS_HEARD, DCT_JOIN_STATE_NETWORK_PACKETS_HEARD_HELP },
                { 0x0002, DCT_JOIN_STATE_ASN_ACQUIRED },
                { 0x0004, DCT_JOIN_STATE_SYNCHRONIZED_TO_SLOT_TIME },
                { 0x0008, DCT_JOIN_STATE_ADVERTISEMENT_HEARD },
                { 0x0010, DCT_JOIN_STATE_JOIN_REQUESTED, DCT_JOIN_STATE_JOIN_REQUESTED_HELP },
                { 0x0020, DCT_JOIN_STATE_JOIN_RETRYING , DCT_JOIN_STATE_JOIN_RETRYING_HELP  },
                { 0x0040, DCT_JOIN_STATE_JOIN_FAILED , DCT_JOIN_STATE_JOIN_FAILED_HELP },
                { 0x0080, DCT_JOIN_STATE_AUTHENTICATED, DCT_JOIN_STATE_AUTHENTICATED_HELP },
                { 0x0100, DCT_JOIN_STATE_NETWORK_JOINED, DCT_JOIN_STATE_NETWORK_JOINED_HELP },
                { 0x0200, DCT_JOIN_STATE_NEGOTIATING_NETWORK_PROPERTIES, DCT_JOIN_STATE_NEGOTIATING_NETWORK_PROPERTIES_HELP },
                { 0x0400, DCT_JOIN_STATE_NORMAL_OPERATION_COMMENCING, DCT_JOIN_STATE_NORMAL_OPERATION_COMMENCING_HELP }
            }
        }

				VARIABLE wireless_operation_mode_codes      //Standard dictionary
				{
				    REDEFINE TYPE ENUMERATED
				    {
				        { 0, DCT_LWA_OPERATION_MODE_IDLE },
				        { 1, DCT_LWA_OPERATION_MODE_ACTIVE_SEARCH, "Looking for neighbors." },
				        { 2, DCT_LWA_OPERATION_MODE_NEGOTIATING },
				        { 3, DCT_LWA_OPERATION_MODE_QUARANTINED },
				        { 4, DCT_LWA_OPERATION_MODE_OPERATIONAL },
				        { 5, DCT_LWA_OPERATION_MODE_SUSPENDED },
				        { 6, DCT_LWA_OPERATION_MODE_DEEP_SLEEP, "Ultra-low power" }
				    }
				}

		VARIABLE join_status
		{
			REDEFINE LABEL DCT_JOIN_STATUS_SWA;
		}

        REDEFINE VARIABLE neighbor_count
        {
            LABEL DCT_NEIGHBOR_COUNT_SWA;
            HELP "Number of available neighbors.";
            CLASS DEVICE & DYNAMIC;
            HANDLING READ;
            TYPE ENUMERATED (1)
            {
            { 0, "0" },
            { 1, "0" },
            { 2, "1" },
            { 3, "2" },
            { 4, "3" },
            { 5, "4" },
            { 6, "5" },
            { 7, "6" },
            { 8, "7" },
            { 9, "8" },
            { 10, "9" }
            }
        }

//END OF CHANGE
//******************************************************************************


		VARIABLE advertisements
		{
			REDEFINE LABEL DCT_ADVERTISEMENTS_SWA;
		}

		VARIABLE join_attempts
		{
			REDEFINE LABEL DCT_JOIN_ATTEMPTS_SWA;
		}


		REDEFINE VARIABLE advertised_neighbors_number
		{
				LABEL DCT_ADVERTISED_NEIGHBORS_NUMBER_SWA;
				CLASS LOCAL;
	    	HELP DCT_ADVERTISED_NEIGHBORS_NUMBER_SWA_HELP;
	    	HANDLING READ;
	    	TYPE UNSIGNED_INTEGER (1);
	    	DEFAULT_VALUE 0;
		}

		VARIABLE device_nickname_address
		{
			REDEFINE LABEL DCT_DEVICE_NICKNAME_ADDRESS_SWA;
		}
		// Wireless

    }
}

// Join Status -> join_status_byte_lo, join_status_byte_hi
#ifdef _PDM_
VARIABLE join_status_byte_lo {
  LABEL DCT_JOIN_STATUS_LO_SWA;
  HELP "The current location in the joining sequence of operations.";
  CLASS DEVICE & DYNAMIC;
  HANDLING READ;
  TYPE BIT_ENUMERATED (1)
  {
      { 0x01, join_process_status_codes_byte_0(0x01) },
      { 0x02, join_process_status_codes_byte_0(0x02) },
      { 0x04, join_process_status_codes_byte_0(0x04) },
      { 0x08, join_process_status_codes_byte_0(0x08) },
      { 0x10, join_process_status_codes_byte_0(0x10) },
      { 0x20, join_process_status_codes_byte_0(0x20) },
      { 0x40, join_process_status_codes_byte_0(0x40) },
      { 0x80, join_process_status_codes_byte_0(0x80) }
  }
}
VARIABLE join_status_byte_hi {
  LABEL DCT_JOIN_STATUS_HI_SWA;
  HELP "The current location in the joining sequence of operations.";
  CLASS DEVICE & DYNAMIC;
  HANDLING READ;
  TYPE BIT_ENUMERATED (1)
  {
      { 0x01, join_process_status_codes_byte_1(0x01) },
      { 0x02, join_process_status_codes_byte_1(0x02) },
      { 0x04, join_process_status_codes_byte_1(0x04) }
  }
}
VARIABLE join_process_status_codes_byte_0      //Standard dictionary
{
    CLASS LOCAL;
    TYPE ENUMERATED (1)
    {
				{ 0x01, DCT_JOIN_STATE_NETWORK_PACKETS_HEARD, DCT_JOIN_STATE_NETWORK_PACKETS_HEARD_HELP },
				{ 0x02, DCT_JOIN_STATE_ASN_ACQUIRED },
				{ 0x04, DCT_JOIN_STATE_SYNCHRONIZED_TO_SLOT_TIME },
				{ 0x08, DCT_JOIN_STATE_ADVERTISEMENT_HEARD },
				{ 0x10, DCT_JOIN_STATE_JOIN_REQUESTED, DCT_JOIN_STATE_JOIN_REQUESTED_HELP },
				{ 0x20, DCT_JOIN_STATE_JOIN_RETRYING , DCT_JOIN_STATE_JOIN_RETRYING_HELP  },
				{ 0x40, DCT_JOIN_STATE_JOIN_FAILED , DCT_JOIN_STATE_JOIN_FAILED_HELP },
				{ 0x80, DCT_JOIN_STATE_AUTHENTICATED, DCT_JOIN_STATE_AUTHENTICATED_HELP }
   }
}
VARIABLE join_process_status_codes_byte_1      //Standard dictionary
{
    CLASS LOCAL;
    TYPE ENUMERATED (1)
    {
        { 0x01, DCT_JOIN_STATE_NETWORK_JOINED, DCT_JOIN_STATE_NETWORK_JOINED_HELP },
				{ 0x02, DCT_JOIN_STATE_NEGOTIATING_NETWORK_PROPERTIES, DCT_JOIN_STATE_NEGOTIATING_NETWORK_PROPERTIES_HELP },
				{ 0x04, DCT_JOIN_STATE_NORMAL_OPERATION_COMMENCING, DCT_JOIN_STATE_NORMAL_OPERATION_COMMENCING_HELP }
   }
}
// Parameter needed in order to collect the COMMAND NUMBER out of the response from the 16 bit commands.
// Actually, only COMMAND 769 seems to need such work around.
VARIABLE dummy_16_bit_command {
	LABEL "2 Byte Parameter";
	CLASS LOCAL;
	TYPE INTEGER(2);
}
#endif

VARIABLE firmware_version
{
    HELP  DCT_FIRMWARE_VERSION_HELP;
    LABEL DCT_FIRMWARE_VERSION_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (16);
}

VARIABLE dust_firmware_version
{
    HELP  DCT_DUST_FIRMWARE_VERSION_HELP;
    LABEL DCT_DUST_FIRMWARE_VERSION_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (16);
}

COMMAND read_firmware_version
{
   NUMBER 226;
   OPERATION READ;
   TRANSACTION 0
   {
      REQUEST
      {
          MODULE_INDEX_MASTER
      }
      REPLY
      {
          response_code,
          device_status,
          MODULE_INDEX_MASTER,
          firmware_version
      }
   }
   TRANSACTION 1
   {
      REQUEST
      {
          MODULE_INDEX_SUB
      }
      REPLY
      {
          response_code,
          device_status,
          MODULE_INDEX_SUB,
          dust_firmware_version
      }
   }
   RESPONSE_CODES
   {
       0, SUCCESS,          [no_command_specific_errors] ;
       2, DATA_ENTRY_ERROR, [invalid_selection] ;
       5, MISC_ERROR,       [too_few_data_bytes_recieved];
       6, MISC_ERROR,       [xmtr_specific_command_error];
   }
}

// Generated variable.
// Source text: Serial Number;x;x;RO;;xxx;;ascii16;;227/229;;static
VARIABLE serial_number
{
    HELP  DCT_SERIAL_NUMBER_HELP;
    LABEL DCT_SERIAL_NUMBER_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (16);
}

COMMAND read_serial_number
{
   NUMBER 227;
   OPERATION READ;
   TRANSACTION
   {
      REQUEST
      {
          MODULE_INDEX_MASTER
      }
      REPLY
      {
          response_code,
          device_status,
          MODULE_INDEX_MASTER,
          serial_number
      }
   }

   RESPONSE_CODES
   {
       0, SUCCESS,          [no_command_specific_errors] ;
       2, DATA_ENTRY_ERROR, [invalid_selection] ;
       5, MISC_ERROR,       [too_few_data_bytes_recieved];
       6, MISC_ERROR,       [xmtr_specific_command_error];
   }
}

// Generated variable.
// Source text: Battery Type (or capacity, tbd);x;method;R/W;;Type 1;;ENUM;"Type 1,Type 2";;20;static
/*
VARIABLE battery_type
{
		VALIDITY FALSE;
    HELP  DCT_BATTERY_TYPE_HELP;
    LABEL DCT_BATTERY_TYPE_LABEL;
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE ENUMERATED {
        { 0, DCT_TYPE_1 },
        { 1, DCT_TYPE_2 }
    }
}
*/
// Generated variable.
// Source text: Battery Changed;x;method;W;;no;;ENUM;"no,yes";;21;static
VARIABLE battery_changed
{
    HELP  DCT_BATTERY_CHANGED_HELP;
    LABEL DCT_BATTERY_CHANGED_LABEL;
    //CLASS DEVICE;
    CLASS LOCAL;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE ENUMERATED {
        { 0, [no] },
        { 1, [yes] }
    }
}

VARIABLE last_time_battery_change
{
    CLASS DYNAMIC;
    TYPE DATE;
    HANDLING READ;
    LABEL DCT_LAST_TIME_BATTERY_CHANGE;
    HELP DCT_LAST_TIME_BATTERY_CHANGE_HELP;
}

// Generated variable.
// Source text: Minimal startup Voltage;x;method;R/W;10..22;10;V;float;;;22;static
VARIABLE startup_voltage
{
    HELP  DCT_STARTUP_VOLTAGE_HELP;
    LABEL DCT_STARTUP_VOLTAGE_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [v];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        MIN_VALUE 8;
        MAX_VALUE 23;
    }
}

// Generated variable.
// Source text: Startup time;x;method;R/W;0..1800;5;s;uint16;;;23;static
VARIABLE startup_time
{
    HELP  DCT_STARTUP_TIME_HELP;
    LABEL DCT_STARTUP_TIME_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [sec];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE UNSIGNED_INTEGER (2)
    {
        MIN_VALUE 0;
        MAX_VALUE 1800;
    }
}

// Generated variable.
// Source text: Minimal operating Voltage;x;method;R/W;10..22;10;V;float;;;24;static
VARIABLE operating_voltage
{
    HELP  DCT_OPERATING_VOLTAGE_HELP;
    LABEL DCT_OPERATING_VOLTAGE_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [v];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        MIN_VALUE 8;
        MAX_VALUE 23;
    }
}

// Generated variable.
// Source text: Setup time;x;method;R/W;0..1800;10;s;uint16;;;25;static
VARIABLE operating_time
{
    HELP  DCT_OPERATING_TIME_HELP;
    LABEL DCT_OPERATING_TIME_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [sec];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE UNSIGNED_INTEGER (2)
    {
        MIN_VALUE 0;
        MAX_VALUE 1800;
    }
}

// Generated variable.
// Source text: Startup current;x;method;RO;;;mA;float;;;26;(dynamic)
VARIABLE startup_current
{
    HELP  DCT_STARTUP_CURRENT_HELP;
    LABEL DCT_STARTUP_CURRENT_LABEL;
    CLASS DYNAMIC;
    CONSTANT_UNIT [mA];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        MIN_VALUE 3;
        MAX_VALUE 50;
    }
}

// Generated variable.
// Source text: Device Power;x;x;R/W;;Auto;;ENUM;"Auto,On,Off";;27;static
VARIABLE power_mode
{
    HELP  DCT_POWER_MODE_HELP;
    LABEL DCT_POWER_MODE_LABEL;
    CLASS DEVICE;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE ENUMERATED {
        { 0, DCT_AUTOLABEL }
        { 1, DCT_POWER_MODE_CONFIG }
        { 2, DCT_POWER_MODE_OFF }
    }
}

// Generated variable.
// Source text: Power On Time After Access;x;x + offline;R/W;0..255;10;s;uint8;;;28;static
VARIABLE power_on_time_after_external_trigger
{
    HELP  DCT_POWER_ON_TIME_AFTER_EXTERNAL_TRIGGER_HELP;
    LABEL DCT_POWER_ON_TIME_AFTER_EXTERNAL_TRIGGER_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [sec];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE UNSIGNED_INTEGER (2)
    {
        MIN_VALUE 1;
        MAX_VALUE 1800;
    }
}

VARIABLE configuration_time
{
    HELP  DCT_CONFIGURATION_TIME_HELP;
    LABEL DCT_CONFIGURATION_TIME_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [sec];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE UNSIGNED_INTEGER (2)
    {
        MIN_VALUE 1;
        MAX_VALUE 3600;
    }
    DEFAULT_VALUE 300;
}

// Generated variable.
// Source text: Lowest scan address;x;x + offline;R/W;0..63;0;;uint8;;;0;static
VARIABLE lowest_scan_address
{
    HELP  DCT_LOWEST_SCAN_ADDRESS_HELP;
    LABEL DCT_LOWEST_SCAN_ADDRESS_LABEL;
    CLASS DEVICE;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE UNSIGNED_INTEGER (1)
    {
        MIN_VALUE 0;
        MAX_VALUE //lowest_scan_address_max;
			SELECT (highest_scan_address)
        	{
        		CASE 0x00: 0x00;
        		CASE 0x01: 0x01;
        		CASE 0x02: 0x02;
        		CASE 0x03: 0x03;
        		CASE 0x04: 0x04;
        		CASE 0x05: 0x05;
        		CASE 0x06: 0x06;
        		CASE 0x07: 0x07;
        		CASE 0x08: 0x08;
        		CASE 0x09: 0x09;
        		CASE 0x0A: 0x0A;
        		CASE 0x0B: 0x0B;
        		CASE 0x0C: 0x0C;
        		CASE 0x0D: 0x0D;
        		CASE 0x0E: 0x0E;
        		CASE 0x0F: 0x0F;
        		CASE 0x10: 0x10;
        		CASE 0x11: 0x11;
        		CASE 0x12: 0x12;
        		CASE 0x13: 0x13;
        		CASE 0x14: 0x14;
        		CASE 0x15: 0x15;
        		CASE 0x16: 0x16;
        		CASE 0x17: 0x17;
        		CASE 0x18: 0x18;
        		CASE 0x19: 0x19;
        		CASE 0x1A: 0x1A;
        		CASE 0x1B: 0x1B;
        		CASE 0x1C: 0x1C;
        		CASE 0x1D: 0x1D;
        		CASE 0x1E: 0x1E;
        		CASE 0x1F: 0x1F;
        		CASE 0x20: 0x20;
        		CASE 0x21: 0x21;
        		CASE 0x22: 0x22;
        		CASE 0x23: 0x23;
        		CASE 0x24: 0x24;
        		CASE 0x25: 0x25;
        		CASE 0x26: 0x26;
        		CASE 0x27: 0x27;
        		CASE 0x28: 0x28;
        		CASE 0x29: 0x29;
        		CASE 0x2A: 0x2A;
        		CASE 0x2B: 0x2B;
        		CASE 0x2C: 0x2C;
        		CASE 0x2D: 0x2D;
        		CASE 0x2E: 0x2E;
        		CASE 0x2F: 0x2F;
        		CASE 0x30: 0x30;
        		CASE 0x31: 0x31;
        		CASE 0x32: 0x32;
        		CASE 0x33: 0x33;
        		CASE 0x34: 0x34;
        		CASE 0x35: 0x35;
        		CASE 0x36: 0x36;
        		CASE 0x37: 0x37;
        		CASE 0x38: 0x38;
        		CASE 0x39: 0x39;
        		CASE 0x3A: 0x3A;
        		CASE 0x3B: 0x3B;
        		CASE 0x3C: 0x3C;
        		CASE 0x3D: 0x3D;
        		CASE 0x3E: 0x3E;
        		DEFAULT: 0x3F;
        	}
    }
    DEFAULT_VALUE 0;
    POST_EDIT_ACTIONS {
    	send_LOWEST_SCAN_ADDRESS
    }
}

METHOD send_LOWEST_SCAN_ADDRESS {
  //#18447
  SEND_TRIGGER_TO_DEVICE11(0x00)
}

// Generated variable.
// Source text: Highest scan address;x;x + offline;R/W;0..63;1;;uint8;;;1;static
VARIABLE highest_scan_address
{
    HELP  DCT_HIGHEST_SCAN_ADDRESS_HELP;
    LABEL DCT_HIGHEST_SCAN_ADDRESS_LABEL;
    CLASS DEVICE;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE UNSIGNED_INTEGER (1)
    {
        MIN_VALUE //highest_scan_address_min;
        	SELECT (lowest_scan_address)
        	{
        		CASE 0x01: 0x01;
        		CASE 0x02: 0x02;
        		CASE 0x03: 0x03;
        		CASE 0x04: 0x04;
        		CASE 0x05: 0x05;
        		CASE 0x06: 0x06;
        		CASE 0x07: 0x07;
        		CASE 0x08: 0x08;
        		CASE 0x09: 0x09;
        		CASE 0x0A: 0x0A;
        		CASE 0x0B: 0x0B;
        		CASE 0x0C: 0x0C;
        		CASE 0x0D: 0x0D;
        		CASE 0x0E: 0x0E;
        		CASE 0x0F: 0x0F;
        		CASE 0x10: 0x10;
        		CASE 0x11: 0x11;
        		CASE 0x12: 0x12;
        		CASE 0x13: 0x13;
        		CASE 0x14: 0x14;
        		CASE 0x15: 0x15;
        		CASE 0x16: 0x16;
        		CASE 0x17: 0x17;
        		CASE 0x18: 0x18;
        		CASE 0x19: 0x19;
        		CASE 0x1A: 0x1A;
        		CASE 0x1B: 0x1B;
        		CASE 0x1C: 0x1C;
        		CASE 0x1D: 0x1D;
        		CASE 0x1E: 0x1E;
        		CASE 0x1F: 0x1F;
        		CASE 0x20: 0x20;
        		CASE 0x21: 0x21;
        		CASE 0x22: 0x22;
        		CASE 0x23: 0x23;
        		CASE 0x24: 0x24;
        		CASE 0x25: 0x25;
        		CASE 0x26: 0x26;
        		CASE 0x27: 0x27;
        		CASE 0x28: 0x28;
        		CASE 0x29: 0x29;
        		CASE 0x2A: 0x2A;
        		CASE 0x2B: 0x2B;
        		CASE 0x2C: 0x2C;
        		CASE 0x2D: 0x2D;
        		CASE 0x2E: 0x2E;
        		CASE 0x2F: 0x2F;
        		CASE 0x30: 0x30;
        		CASE 0x31: 0x31;
        		CASE 0x32: 0x32;
        		CASE 0x33: 0x33;
        		CASE 0x34: 0x34;
        		CASE 0x35: 0x35;
        		CASE 0x36: 0x36;
        		CASE 0x37: 0x37;
        		CASE 0x38: 0x38;
        		CASE 0x39: 0x39;
        		CASE 0x3A: 0x3A;
        		CASE 0x3B: 0x3B;
        		CASE 0x3C: 0x3C;
        		CASE 0x3D: 0x3D;
        		CASE 0x3E: 0x3E;
        		CASE 0x3F: 0x3F;
        		DEFAULT: 0x00;
        	}
        MAX_VALUE 63;
    }
    DEFAULT_VALUE 1;
    POST_EDIT_ACTIONS {
    	send_HIGHEST_SCAN_ADDRESS
    }
}

METHOD send_HIGHEST_SCAN_ADDRESS {
  //#18447
  SEND_TRIGGER_TO_DEVICE11(0x01)
}

// Generated variable.
// Source text: Reboot Counter;x;x;RO;;;;uint16;;;3;dynamic
VARIABLE reboot_counter
{
    HELP  DCT_REBOOT_COUNTER_HELP;
    LABEL DCT_REBOOT_COUNTER_LABEL;
    CLASS DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (2);
}

// Generated variable.
// Source text: Configure Lower False Current Warning;x;x + offline;R/W;4..20;4;mA;float;;;10;static
VARIABLE configure_lower_false_current_warning
{
    HELP  DCT_CONFIGURE_LOWER_FALSE_CURRENT_WARNING_HELP;
    LABEL DCT_CONFIGURE_LOWER_FALSE_CURRENT_WARNING_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [mA];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        MIN_VALUE 3;
        MAX_VALUE configure_upper_false_current_warning;
    }
    DEFAULT_VALUE 3.8;
}

// Generated variable.
// Source text: Configure Upper False Current Warning;x;x + offline;R/W;4..20;20;mA;float;;;11;static
VARIABLE configure_upper_false_current_warning
{
    HELP  DCT_CONFIGURE_UPPER_FALSE_CURRENT_WARNING_HELP;
    LABEL DCT_CONFIGURE_UPPER_FALSE_CURRENT_WARNING_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [mA];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        MIN_VALUE configure_lower_false_current_warning;
        MAX_VALUE 23;
    }
    DEFAULT_VALUE 20.5;
}

// Generated variable.
// Source text: Configure Lower False Current Alarm;x;x + offline;R/W;3,8..20,5;3,8;mA;float;;;12;static
VARIABLE configure_lower_false_current_alarm
{
    HELP  DCT_CONFIGURE_LOWER_FALSE_CURRENT_ALARM_HELP;
    LABEL DCT_CONFIGURE_LOWER_FALSE_CURRENT_ALARM_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [mA];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        MIN_VALUE 3;
        MAX_VALUE configure_upper_false_current_alarm;
    }
    DEFAULT_VALUE 3.6;
}

// Generated variable.
// Source text: Configure Upper False Current Alarm;x;x + offline;R/W;3,8..20,5;20,5;mA;float;;;13;static
VARIABLE configure_upper_false_current_alarm
{
    HELP  DCT_CONFIGURE_UPPER_FALSE_CURRENT_ALARM_HELP;
    LABEL DCT_CONFIGURE_UPPER_FALSE_CURRENT_ALARM_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [mA];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        MIN_VALUE configure_lower_false_current_alarm;
        MAX_VALUE 23;
    }
    DEFAULT_VALUE 21;
}

// Generated variable.
// Source text: Table Index;x;x;R/W;1..32;1;;uint8;;;101;static
VARIABLE table_index
{
    HELP  DCT_TABLE_INDEX_HELP;
    LABEL DCT_TABLE_INDEX_LABEL;
    CLASS DEVICE;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE UNSIGNED_INTEGER (1)
    {
        MIN_VALUE 1;
        MAX_VALUE 32;
    }
}

// Generated variable.
// Source text: X-Value;x;x;R/W;4..20;4;mA;float;;;102;static
VARIABLE x_value
{
    HELP  DCT_X_VALUE_HELP;
    LABEL DCT_X_VALUE_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [mA];
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        MIN_VALUE1 3;
    	MIN_VALUE2 0;
        MAX_VALUE1 23;
        MAX_VALUE2 0;
    }
}

// Generated variable.
// Source text: Y-Value;x;x;R/W;;0;PV Unit;float;;;103;static
VARIABLE y_value
{
    HELP  DCT_Y_VALUE_HELP;
    LABEL DCT_Y_VALUE_LABEL;
    CLASS DEVICE;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT;
}

VARIABLE start_scan
{
    HELP  DCT_START_SCAN_HELP;
    LABEL DCT_START_SCAN_LABEL;
    CLASS DEVICE;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE ENUMERATED {
        { 0, [none] },
        { 1, "Start" }
    }
}


METHOD method_scan_subdevice
{
	LABEL DCT_METHOD_START_SCAN;
	DEFINITION
	{
		char status[STATUS_SIZE];
		char disp_string2[60];
		char disp_string1[300];
		long var_id[4];
		int slen2;
		int slen1;
		int sel;
		unsigned int counter;

		send(76, status);
		if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
		{
			ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
			process_abort();
		}

		slen2= 60;

		get_dictionary_string(yes_no, disp_string2, slen2);
		sel= select_from_list(DCT_LABEL_CONFIRM_SCAN, var_id, disp_string2);
		if (sel == 0) /* 0 == yes == execute reset */
		{
				disp_string1 = "Please wait until scan has finished.";
				counter=0;
				slen1=300;
				iassign(start_scan,1);
				iassign(variable_array_index, 5);
				send_command(173);
				ACKNOWLEDGE(DCT_MANUAL_REFRESH_NEEDED);
/*
				send_command_trans(48,0);
				while ((device_specific_status_1&0x40)==0x40)
				{
					counter++;
					if (counter==12)
					{
						disp_string1 = "Please wait until scan has finished.\nThe scan is taking too long.\nYou may not want to wait until it has finished.";
					}
					send_command_trans(48,0);
					DELAY(5, disp_string1);
					send_command_trans(48,0);
				}
*/
		}
		save_values();
	}
}

// Generated variable.
// Source text: Ordercode;x;x;RO;;xxx;;ascii16;;228/230;;static
VARIABLE order_code
{
    HELP  DCT_ORDER_CODE_HELP;
    LABEL DCT_ORDER_CODE_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (24);
}

COMMAND read_order_code
{
   NUMBER 228;
   OPERATION READ;
   TRANSACTION
   {
      REQUEST
      {
          MODULE_INDEX_MASTER
      }
      REPLY
      {
          response_code,
          device_status,
          MODULE_INDEX_MASTER,
          order_code
      }
   }

   RESPONSE_CODES
   {
       0, SUCCESS,          [no_command_specific_errors] ;
       2, DATA_ENTRY_ERROR, [invalid_selection] ;
       5, MISC_ERROR,       [too_few_data_bytes_recieved];
       6, MISC_ERROR,       [xmtr_specific_command_error];
   }
}

VARIABLE order_ident
{
    HELP  DCT_ORDER_IDENT_HELP;
    LABEL DCT_ORDER_IDENT_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (20);
}

COMMAND read_order_ident
{
   NUMBER 233;
   OPERATION READ;
   TRANSACTION
   {
      REQUEST
      {
          MODULE_INDEX_MASTER
      }
      REPLY
      {
          response_code,
          device_status,
          MODULE_INDEX_MASTER,
          order_ident
      }
   }

   RESPONSE_CODES
   {
       0, SUCCESS,          [no_command_specific_errors] ;
       2, DATA_ENTRY_ERROR, [invalid_selection] ;
       5, MISC_ERROR,       [too_few_data_bytes_recieved];
       6, MISC_ERROR,       [xmtr_specific_command_error];
   }
}

VARIABLE enp_version
{
	LABEL DCT_ENP_VERSION_LABEL;
	CLASS DEVICE;
	HANDLING READ;
	TYPE ASCII(16);
}

COMMAND read_enp_version
{
   NUMBER 234;
   OPERATION READ;
   TRANSACTION
   {
      REQUEST
      {
          MODULE_INDEX_MASTER
      }
      REPLY
      {
          response_code,
          device_status,
          MODULE_INDEX_MASTER,
          enp_version
      }
   }

   RESPONSE_CODES
   {
       0, SUCCESS,          [no_command_specific_errors] ;
       2, DATA_ENTRY_ERROR, [invalid_selection] ;
       5, MISC_ERROR,       [too_few_data_bytes_recieved];
       6, MISC_ERROR,       [xmtr_specific_command_error];
   }
}

// Generated variable.
// Source text: Status Visualization (Actual Highest Alarm);x;x;RO;;;;uint16;;;2;dynamic
VARIABLE status_visualization_actual_highest_alarm
{
    HELP  DCT_STATUS_VISUALIZATION_ACTUAL_HIGHEST_ALARM_HELP;
    LABEL DCT_STATUS_VISUALIZATION_ACTUAL_HIGHEST_ALARM_LABEL;
    CLASS DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (2);
}

// E+H standard 150 which features NAMUR NE 107
VARIABLE standard_150_device_status
{
    HELP  DCT_STANDARD_150_DEVICE_STATUS_HELP;
    LABEL DCT_STANDARD_150_DEVICE_STATUS_LABEL;
    CLASS DYNAMIC;
    HANDLING READ;
    TYPE ENUMERATED (1)
    {

        { 0x00, DCT_STANDARD_150_GOOD }
        { 0x01, DCT_STANDARD_150_FAILURE_DETECTED }
        { 0x02, DCT_STANDARD_150_SERVICE_MODE }
        { 0x03, DCT_STANDARD_150_SERVICE_MODE }
        { 0x04, DCT_STANDARD_150_MAINTENANCE_REQUIRED }
        { 0x05, DCT_STANDARD_150_MAINTENANCE_REQUIRED }
        { 0x06, DCT_STANDARD_150_MAINTENANCE_REQUIRED }
        { 0x07, DCT_STANDARD_150_MAINTENANCE_REQUIRED }
        { 0x08, DCT_STANDARD_150_OUT_OF_SPECIFICATION }
        { 0x09, DCT_STANDARD_150_OUT_OF_SPECIFICATION }
        { 0x0A, DCT_STANDARD_150_OUT_OF_SPECIFICATION }
        { 0x0B, DCT_STANDARD_150_OUT_OF_SPECIFICATION }
        { 0x0C, DCT_STANDARD_150_OUT_OF_SPECIFICATION }
        { 0x0D, DCT_STANDARD_150_OUT_OF_SPECIFICATION }
        { 0x0E, DCT_STANDARD_150_OUT_OF_SPECIFICATION }
        { 0x0F, DCT_STANDARD_150_OUT_OF_SPECIFICATION }
    }
}

COMMAND check_device_status
{
   NUMBER 231;
   OPERATION READ;
   TRANSACTION
   {
      REQUEST
      {
          MODULE_INDEX_MASTER
      }
      REPLY
      {
          response_code,
          device_status,
          MODULE_INDEX_MASTER,
          standard_150_device_status,
          status_visualization_actual_highest_alarm
      }
   }

   RESPONSE_CODES
   {
       0, SUCCESS,          [no_command_specific_errors] ;
       2, DATA_ENTRY_ERROR, [invalid_selection] ;
       5, MISC_ERROR,       [too_few_data_bytes_recieved];
       6, MISC_ERROR,       [xmtr_specific_command_error];
   }
}

// Generated variable.
// Source text: Reset Temp Min/Max;x;x;R/W;;none;;ENUM;"none,reset min,reset max,reset min and max";;62;static
VARIABLE reset_temp_min_max
{
		HELP  DCT_RESET_TEMP_MIN_MAX_HELP;
		LABEL DCT_RESET_TEMP_MIN_MAX_LABEL;
		//CLASS DEVICE;
		CLASS LOCAL;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
		TYPE ENUMERATED {
		    { 0, [none] },
		    { 1, DCT_RESET_MIN },
		    { 2, DCT_RESET_MAX },
		    { 3, DCT_RESET_MIN_AND_MAX }
		}
/*
		POST_EDIT_ACTIONS
		{
		  method_reset_temp_min_max
		}
*/
}

// METHOD created to ask the user whether they want to perform the reset or not.
METHOD method_reset_temp_min_max
{
    LABEL DCT_METHOD_RESET_TEMP_MIN_MAX;
    DEFINITION
		{
			char status[STATUS_SIZE];
			char disp_string2[60];
			long var_id[4];
			int slen2;
			int sel;
			int resetTemp;

			IGNORE_NO_DEVICE();
			IGNORE_ALL_DEVICE_STATUS();
			IGNORE_ALL_RESPONSE_CODES();
			XMTR_IGNORE_ALL_DEVICE_STATUS();
			XMTR_IGNORE_ALL_RESPONSE_CODES();
			XMTR_IGNORE_NO_DEVICE();

			send(76, status);
			if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
			{
				ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
				process_abort();
			}

			slen2= 60;

			get_dictionary_string(yes_no, disp_string2, slen2);
			sel= select_from_list(DCT_LABEL_CONFIRM_RESET_TEMP_MIN_MAX, var_id, disp_string2);
			if (sel == 0) /* 0 == yes == execute reset */
			{
				 iassign(reset_temp_min_max, 3);
				 iassign(variable_array_index, 62);
				 send_command(173);
         //send_command_trans(173,2);//write the reset value to the device
         iassign(reset_temp_min_max, 0);
         // re-read temp min and max
         send_command(9);
			}
			save_values();
    }
}

VARIABLE join_priority
{
    HELP  DCT_JOIN_PRIORITY_HELP;
    LABEL DCT_JOIN_PRIORITY_LABEL;
    CLASS DEVICE;
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE UNSIGNED_INTEGER (1);
}

COMMAND read_join_priority
{
   NUMBER 810;
   OPERATION READ;
   TRANSACTION
   {
      REQUEST
      {
      }
      REPLY
      {
          response_code,
          device_status,
          join_priority
      }
   }

   RESPONSE_CODES
   {
       0,  SUCCESS,          [no_command_specific_errors];
   }
}

COMMAND write_join_priority
{
   NUMBER 811;
   OPERATION WRITE;
   TRANSACTION
   {
      REQUEST
      {
          join_priority
      }
      REPLY
      {
          response_code,
          device_status,
          join_priority
      }
   }

   RESPONSE_CODES
   {
       0,  SUCCESS,          [no_command_specific_errors];
       5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
       8,  MISC_WARNING,     [set_to_nearest_possible_value];
       16, MODE_ERROR,       [access_restricted];
   }
}

// Generated variable.
// Source text: MAC address;x;x;RO;;;;8 Byte;;;;static
#ifndef _PDM_
VARIABLE mac_address
{
    HELP  DCT_MAC_ADDRESS_HELP;
    LABEL DCT_MAC_ADDRESS_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (8)
    {
        DISPLAY_FORMAT "X";
    }
}
#else
VARIABLE mac_address_lower_bytes
{
    LABEL DCT_MAC_ADDRESS_LOWER_BYTES_LABEL;
    HELP DCT_MAC_ADDRESS_LOWER_BYTES_HELP;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (4)
    {
        DISPLAY_FORMAT "X";
    }
}
VARIABLE mac_address_higher_bytes
{
		LABEL DCT_MAC_ADDRESS_HIGHER_BYTES_LABEL;
		HELP DCT_MAC_ADDRESS_HIGHER_BYTES_HELP;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (4)
    {
        DISPLAY_FORMAT "X";
    }
}
#endif
//TODO get final spec for Flash Update
/*
VARIABLE max_capacity
{
    HELP  DCT_MAX_CAPACITY_HELP;
    LABEL DCT_MAX_CAPACITY_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT DCT_AH;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        MIN_VALUE 1;
        MAX_VALUE 100;
    }
    DEFAULT_VALUE 19.5;
}

VARIABLE min_capacity
{
    HELP  DCT_MIN_CAPACITY_HELP;
    LABEL DCT_MIN_CAPACITY_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT DCT_AH;
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        MIN_VALUE 1;
        MAX_VALUE 100;
    }
    DEFAULT_VALUE 8;
}

VARIABLE indication_level_at_25_degC
{
    HELP  DCT_INDICATION_LEVEL_AT_25_DEGC_HELP;
    LABEL DCT_INDICATION_LEVEL_AT_25_DEGC_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [v];
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        MIN_VALUE 2.5;
        MAX_VALUE 3.6;
    }
    DEFAULT_VALUE 3.1;
}

VARIABLE cut_off_voltage
{
    HELP  DCT_CUT_OFF_VOLTAGE_HELP;
    LABEL DCT_CUT_OFF_VOLTAGE_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [v];
    HANDLING READ & WRITE;
    TYPE FLOAT
    {
        MIN_VALUE 2.0;
        MAX_VALUE 3.6;
    }
    DEFAULT_VALUE 2.65;
}
*/
VARIABLE nominal_current
{
    HELP  DCT_NOMINAL_CURRENT_HELP;
    LABEL DCT_NOMINAL_CURRENT_LABEL;
    CLASS DEVICE;
    CONSTANT_UNIT [mA];
    HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE FLOAT
    {
        MIN_VALUE 0.1;
        MAX_VALUE 100.0;
    }
    DEFAULT_VALUE 1;
}

//******************************************************************************

VARIABLE dummy
{
	LABEL DCT_DUMMY;
	CLASS DEVICE;
	TYPE UNSIGNED_INTEGER;
}
VARIABLE dummy2
{
	LABEL DCT_DUMMY_2;
	CLASS DEVICE;
	TYPE UNSIGNED_INTEGER;
}

/*
**********************************************************************
** COMMAND #172 READ
**********************************************************************
*/
COMMAND read_index_parameter_1
{
   NUMBER 172;
   OPERATION READ;
#ifdef _PDM_
   TRANSACTION 1
   {
      REQUEST
      {
         variable_array_index(INFO,INDEX)
      }
      REPLY
      {
         response_code,
         device_status,
         variable_array_index (INFO,INDEX),
         variable_array_val_1[variable_array_index]
      }
   }

   TRANSACTION 2
   {
      REQUEST
      {
         0x00, 0x006
      }
      REPLY
      {
         response_code,
         device_status,
         variable_array_index (INFO,INDEX),
         mac_address_higher_bytes,
         mac_address_lower_bytes
      }
   }
#else
   TRANSACTION 1
   {
      REQUEST
      {
         variable_array_index(INFO,INDEX)
      }
      REPLY
      {
         response_code,
         device_status,
         variable_array_index (INFO,INDEX),
         variable_array_val_1[variable_array_index]
      }
   }
#endif

   TRANSACTION 11
   {
      REQUEST
      {
         0x00, 0x54,
		 0x00, 0x01
      }
      REPLY
      {
		response_code,
		device_status,
		0x00, 0x54,
		0x00, 0x01,
		subdevice_variables[1].IO_CARD,
		subdevice_variables[1].CHANNEL,
		subdevice_variables[1].MANUFID,
		subdevice_variables[1].EXP_DEV_TYPE,
		subdevice_variables[1].DEV_ID,
		subdevice_variables[1].UNIV_CMD_REV,
		subdevice_variables[1].LONG_TAG,
		subdevice_variables[1].POLL_ADR
		0xFF,0xFF,0xFF,0xFF,0xFF,0xFF//subdevice_variables[subdevice_index].SHORT_TAG
      }
   }
  TRANSACTION 12
   {
      REQUEST
      {
         0x00, 0x54,
		 0x00, 0x02
      }
      REPLY
      {
		response_code,
		device_status,
		0x00, 0x54,
		0x00, 0x02,
		subdevice_variables[2].IO_CARD,
		subdevice_variables[2].CHANNEL,
		subdevice_variables[2].MANUFID,
		subdevice_variables[2].EXP_DEV_TYPE,
		subdevice_variables[2].DEV_ID,
		subdevice_variables[2].UNIV_CMD_REV,
		subdevice_variables[2].LONG_TAG,
		subdevice_variables[2].POLL_ADR
		0xFF,0xFF,0xFF,0xFF,0xFF,0xFF//subdevice_variables[subdevice_index].SHORT_TAG
      }
   }
  TRANSACTION 13
   {
      REQUEST
      {
         0x00, 0x54,
		 0x00, 0x03
      }
      REPLY
      {
		response_code,
		device_status,
		0x00, 0x54,
		0x00, 0x03,
		subdevice_variables[3].IO_CARD,
		subdevice_variables[3].CHANNEL,
		subdevice_variables[3].MANUFID,
		subdevice_variables[3].EXP_DEV_TYPE,
		subdevice_variables[3].DEV_ID,
		subdevice_variables[3].UNIV_CMD_REV,
		subdevice_variables[3].LONG_TAG,
		subdevice_variables[3].POLL_ADR
		0xFF,0xFF,0xFF,0xFF,0xFF,0xFF//subdevice_variables[subdevice_index].SHORT_TAG
      }
   }
  TRANSACTION 14
   {
      REQUEST
      {
         0x00, 0x54,
		 0x00, 0x04
      }
      REPLY
      {
		response_code,
		device_status,
		0x00, 0x54,
		0x00, 0x04,
		subdevice_variables[4].IO_CARD,
		subdevice_variables[4].CHANNEL,
		subdevice_variables[4].MANUFID,
		subdevice_variables[4].EXP_DEV_TYPE,
		subdevice_variables[4].DEV_ID,
		subdevice_variables[4].UNIV_CMD_REV,
		subdevice_variables[4].LONG_TAG,
		subdevice_variables[4].POLL_ADR
		0xFF,0xFF,0xFF,0xFF,0xFF,0xFF//subdevice_variables[subdevice_index].SHORT_TAG
      }
   }

	TRANSACTION 15
	{
		REQUEST
		{
			0x00, 0x10,
			0x00
		}
		REPLY
		{
			overview_validity,
			device_status,
			0x00,
			0x10,
			0x00,
			compat_burst_mode_selection_0,
			compat_burst_mode_selection_1,
			compat_burst_mode_selection_2,
			compat_burst_mode_selection_3,
			compat_burst_mode_selection_4,
			compat_burst_mode_selection_5,
			compat_burst_mode_selection_6,
			compat_burst_mode_selection_7,
			compat_burst_mode_selection_8,
			compat_burst_mode_selection_9
		}
	}

	TRANSACTION 16
	{
		REQUEST
		{
			0x00, 0x10,
			0x01
		}
		REPLY
		{
			overview_validity,
			device_status,
			0x00,
			0x10,
			0x01,
			compat_event_notification_control_n_overview_0,
			compat_event_notification_control_n_overview_1,
			compat_event_notification_control_n_overview_2,
			compat_event_notification_control_n_overview_3,
			compat_event_notification_control_n_overview_4
		}
	}
#ifndef _PDM_
	TRANSACTION 17
	{
		REQUEST
		{
			0x00, 0x10,
			0x00
		}
		REPLY
		{
			response_code,
			device_status,
			0x00,
			0x10,
			0x00,
			compat_burst_mode_selection_ams_0,
			compat_burst_mode_selection_ams_1,
			compat_burst_mode_selection_ams_2,
			compat_burst_mode_selection_ams_3,
			compat_burst_mode_selection_ams_4,
			compat_burst_mode_selection_ams_5,
			compat_burst_mode_selection_ams_6,
			compat_burst_mode_selection_ams_7,
			compat_burst_mode_selection_ams_8,
			compat_burst_mode_selection_ams_9
		}
	}

	TRANSACTION 18
	{
		REQUEST
		{
			0x00, 0x10,
			0x01
		}
		REPLY
		{
			response_code,
			device_status,
			0x00,
			0x10,
			0x01,
			compat_event_notification_control_n_overview_ams_0,
			compat_event_notification_control_n_overview_ams_1,
			compat_event_notification_control_n_overview_ams_2,
			compat_event_notification_control_n_overview_ams_3,
			compat_event_notification_control_n_overview_ams_4
		}
	}
#endif
   RESPONSE_CODES
   {
       0, SUCCESS,          DCT_NO_COMMAND_SPECIFIC_ERRORS_SWA ;
       2, MISC_WARNING, DCT_INVALID_SELECTION_SWA ;
       5, MISC_ERROR,       [too_few_data_bytes_recieved] ;
   }

}
/*
VARIABLE mac_address_byte_0
{
    HELP  DCT_MAC_ADDRESS_HELP;
    LABEL DCT_MAC_ADDRESS_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1)
    {
        DISPLAY_FORMAT "X";
    }
}

VARIABLE mac_address_byte_1
{
    HELP  DCT_MAC_ADDRESS_HELP;
    LABEL DCT_MAC_ADDRESS_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1)
    {
        DISPLAY_FORMAT "X";
    }
}

VARIABLE mac_address_byte_2
{
    HELP  DCT_MAC_ADDRESS_HELP;
    LABEL DCT_MAC_ADDRESS_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1)
    {
        DISPLAY_FORMAT "X";
    }
}

VARIABLE mac_address_byte_3
{
    HELP  DCT_MAC_ADDRESS_HELP;
    LABEL DCT_MAC_ADDRESS_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1)
    {
        DISPLAY_FORMAT "X";
    }
}

VARIABLE mac_address_byte_4
{
    HELP  DCT_MAC_ADDRESS_HELP;
    LABEL DCT_MAC_ADDRESS_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1)
    {
        DISPLAY_FORMAT "X";
    }
}

VARIABLE mac_address_byte_5
{
    HELP  DCT_MAC_ADDRESS_HELP;
    LABEL DCT_MAC_ADDRESS_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1)
    {
        DISPLAY_FORMAT "X";
    }
}

VARIABLE mac_address_byte_6
{
    HELP  DCT_MAC_ADDRESS_HELP;
    LABEL DCT_MAC_ADDRESS_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1)
    {
        DISPLAY_FORMAT "X";
    }
}

VARIABLE mac_address_byte_7
{
    HELP  DCT_MAC_ADDRESS_HELP;
    LABEL DCT_MAC_ADDRESS_LABEL;
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (1)
    {
        DISPLAY_FORMAT "X";
    }
}

*/
/*
**********************************************************************
** VARIABLE ARRAY DESCRIPTION
**********************************************************************
*/
VARIABLE variable_array_index
{
   LABEL   "variable_array_index"; // no dictionary entry for this dummy label
   TYPE    INDEX (2) variable_array_val_1;
   CLASS   LOCAL;
}

ARRAY OF VARIABLE variable_array_val_1
{
    ELEMENTS
    {
        0,    lowest_scan_address;
        1,    highest_scan_address;
        2,    lock_dust_flash_update;
        3,    reboot_counter;

        5,    start_scan;//Bug 21069
#ifndef _PDM_
        6,    mac_address;
#endif

        10,   configure_lower_false_current_warning;
        11,   configure_upper_false_current_warning;
        12,   configure_lower_false_current_alarm;
        13,   configure_upper_false_current_alarm;

        19,   last_time_battery_change;

        /*20,   battery_type;*/
        21,   battery_changed;
        22,   startup_voltage;
        23,   startup_time;
        24,   operating_voltage;
        25,   operating_time;
        26,   startup_current;
        27,   power_mode;
        28,   power_on_time_after_external_trigger;
        29,   configuration_time;

/*
        30,   max_capacity;
        31,   min_capacity;
        32,   indication_level_at_25_degC;
        33,   cut_off_voltage;
*/
        34,   nominal_current;

        62,   reset_temp_min_max;//Bug 21069

        101,  table_index;
        102,  x_value;
        103,  y_value;

    }
}


ARRAY OF VARIABLE device_variable_units
{
    ELEMENTS
    {
     	1, PV.DEVICE_VARIABLE.DIGITAL_UNITS, DCT_PRIMARY_VARIABLE_UNIT_LABEL;
			2, SV.DEVICE_VARIABLE.DIGITAL_UNITS, DCT_SECONDARY_VARIABLE_UNIT_LABEL;
			3, TV.DEVICE_VARIABLE.DIGITAL_UNITS, DCT_TERTIARY_VARIABLE_UNIT_LABEL;
			4, QV.DEVICE_VARIABLE.DIGITAL_UNITS, DCT_QUATERNARY_VARIABLE_UNIT_LABEL;
    }
}

/*
**********************************************************************
** COMMAND #173 WRITE
**********************************************************************
*/
COMMAND write_index_parameter_1
{
   NUMBER 173;
   OPERATION WRITE;
   TRANSACTION
   {
      REQUEST
      {
         variable_array_index (INFO,INDEX),
         variable_array_val_1[variable_array_index]
      }
      REPLY
      {
         response_code,
         device_status,
         variable_array_index (INFO,INDEX),
         variable_array_val_1[variable_array_index]
      }
   }
/*
   TRANSACTION 0
   {
      REQUEST
      {
         variable_array_index (INFO,INDEX),
         variable_array_val_1[variable_array_index]
      }
      REPLY
      {
         response_code,
         device_status,
         variable_array_index (INFO,INDEX),
         variable_array_val_1[variable_array_index]
      }
   }
   TRANSACTION 1
   {
      REQUEST
      {
         0x00, 0x15, 0x01
      }
      REPLY
      {
         response_code,
         device_status,
         0x00, 0x15,
         0x01
      }
   }
   TRANSACTION 2
   {
      REQUEST
      {
         0x00, 0x3E, 0x03
      }
      REPLY
      {
         response_code,
         device_status,
         0x00, 0x3E,
         0x03
      }
   }
   TRANSACTION 3
   {
      REQUEST
      {
         0x00, 0x05, 0x01 // start_scan, 1 : "Start"
      }
      REPLY
      {
         response_code,
         device_status,
         0x00, 0x05,
         0x00
      }
   }
*/
   RESPONSE_CODES
   {
       0, SUCCESS          , DCT_NO_COMMAND_SPECIFIC_ERRORS_SWA;                /* NO_COMMAND_ERRORS */
       2, DATA_ENTRY_ERROR , DCT_INVALID_SELECTION_SWA;                         /* INVALID_SELECTION */
       3, DATA_ENTRY_ERROR , DCT_PASSED_PARAMETER_TOO_LARGE_SWA;                /* PASSED_PARAMETER_TOO_LARGE */
       4, DATA_ENTRY_ERROR , DCT_PASSED_PARAMETER_TOO_SMALL_SWA;                /* PASSED_PARAMETER_TOO_SMALL */
       5, MISC_ERROR       , [too_few_data_bytes_recieved];               /* TOO_FEW_DATA_BYTES */
       //6, DATA_ENTRY_ERROR , "Tansmitter spec command error";             /* TRANSMITTER_SPEC_CMD_ERROR */
       7, MODE_ERROR       , [in_write_protect_mode];                     /* IN_WRITE_PROTECT_MODE */
       8, MISC_WARNING     , DCT_SET_TO_NEAREST_POSSIBLE_VALUE_SWA;               /* WARNING_SET_TO_NEAREST_POSSIBLE_VALUE */
       //9, DATA_ENTRY_ERROR , "Invalid value selection" ;                  /* INCORRECT_LOOP_CURRENT_MODE_OR_VALUE */
       //10,DATA_ENTRY_ERROR , "Lower range value too small";               /* LOWER_RANGE_VALUE_TOO_LOW */
       //11,DATA_ENTRY_ERROR , "Upper range value too high";                /* UPPER_RANGE_VALUE_TOO_HIGH */
       //12,DATA_ENTRY_ERROR , "Invalid value selection";                   /* INVALID_MODE_SELECTION */
       //13,DATA_ENTRY_ERROR , "Upper and Lower range value out of range";  /* UPPER_AND_LOWER_RANGE_VALUE_OUT_OF_RNG */
       //14,DATA_ENTRY_ERROR , "Span too small";                            /* SPAN_TOO_SMALL */
       16, MODE_ERROR      , [access_restricted];                         /* ACCESS_RESTRICTED */
       /*18,DATA_ENTRY_ERROR , "Invalid units code";*/                    /* INVALID_UNITS_CODE */
       //29,DATA_ENTRY_ERROR , "Invalid span";                              /* INVALID_SPAN */
       /*32,DATA_ENTRY_ERROR , "Response busy";*/                         /* RESPONSE_BUSY*/
       /*33,DATA_ENTRY_ERROR , "Error in download record";*/              /* ERROR_IN_DOWNLOAD_RECORD */
       /*64,DATA_ENTRY_ERROR , "Command not implement";*/                 /* CMD_NOT_IMPL */
       /*96,DATA_ENTRY_ERROR , "Wrong sequence of segment no";*/          /* WRONG_SEQUENCE_OF_SEGMENT_NO */
   }
}

/*
   Starts joining the network.
   To do this it writes the following valiables:
   - join_key_1 .. join_key_4
   - network_id (will be changed by the user)
   - join_shed_time (will be changed by the user)
   - join_mode
*/
METHOD join_network
{
	LABEL DCT_JOIN_NETWORK;
	HELP  DCT_JOIN_NETWORK_HELP;
	DEFINITION
	{
		char status[STATUS_SIZE];
		char disp_string1[160];
		char disp_string2[160];
		long var_id[3];
		int sel;
		int slen1;
		int slen2;
		long temp;
		int i;

		slen1 = 160;
		slen2 = 160;

		IGNORE_ALL_DEVICE_STATUS();
		IGNORE_ALL_RESPONSE_CODES();
		XMTR_IGNORE_ALL_DEVICE_STATUS();
		XMTR_IGNORE_ALL_RESPONSE_CODES();

		send(76, status);
		if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
		{
			ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
			process_abort();
		}

		add_abort_method(method_join_network_aborted);
		//Ask if method should be started.
		disp_string1= DCT_WRITE_JOIN_VALUES;
		get_dictionary_string(yes_no, disp_string2, slen2);
		sel= select_from_list(disp_string1, var_id, disp_string2);
		if (sel == 0) /* 0 == yes == start joining */
		{
			GET_DEV_VAR_VALUE(DCT_ENTER_JOINKEY_1, join_key_1);
			GET_DEV_VAR_VALUE(DCT_ENTER_JOINKEY_2, join_key_2);
			GET_DEV_VAR_VALUE(DCT_ENTER_JOINKEY_3, join_key_3);
			GET_DEV_VAR_VALUE(DCT_ENTER_JOINKEY_4, join_key_4);

			//Check if joinkey is set properly
			temp = lvar_value(join_key_1);
			temp += lvar_value(join_key_2);
			temp += lvar_value(join_key_3);
			temp += lvar_value(join_key_4);

			if (temp != 0)
			{
				// Join Key is valid and is written to device
				send_command(768);
				lassign(join_key_1, 0);
				lassign(join_key_2, 0);
				lassign(join_key_3, 0);
				lassign(join_key_4, 0);
				save_values();
			}
			else
			{
				// Join key is not valid. Check if joining must be aborted.
				send_command(48);
				temp = lvar_value(device_specific_status_0);
				if ((temp & 0x08) != 0)
				{    // no join key set error flag
					disp_string1= DCT_JOIN_KEY_EMPTY;
					ACKNOWLEDGE(disp_string1);
					process_abort();
				}
			}
			//ask for new network id and write network id
			GET_DEV_VAR_VALUE(DCT_ENTER_NETWORK_ID, network_id);
			send(773, status);
			if( (status[STATUS_RESPONSE_CODE] != 0) && (status[STATUS_RESPONSE_CODE] != 8) )
			{
				ACKNOWLEDGE(DCT_WRITING_NETWORK_ID_FAILED);
				process_abort();
			}
			if(status[STATUS_RESPONSE_CODE] == 8)
			{
				ACKNOWLEDGE(DCT_NETWORK_ID_CHANGE_PENDING);
			}
			//ask for new join_shed_time
			GET_DEV_VAR_VALUE(DCT_ENTER_JOIN_SHED_TIME, join_shed_time);

			// Set Join mode to "Join Now" if its set to "Attempt to join..."
			temp = lvar_value(join_mode);
			if (temp > 1)
			{
				lassign(join_mode, 1);
			}
			remove_abort_method(method_join_network_aborted);
			//write join_shed_time + join_mode
			send(771, status);

			if(status[STATUS_RESPONSE_CODE] == 0)
			{
				disp_string1= DCT_NORMAL_OPERATION_COMMENCING;
			}
			else
			{
				disp_string1= DCT_JOIN_FAILED;
			}
			ACKNOWLEDGE(disp_string1);
		}
		else
		{
			ACKNOWLEDGE(DCT_JOIN_NETWORK_NO_ACTION_TAKEN);
		}
	}
}

METHOD method_join_network_aborted
{
	LABEL DCT_JOIN_NETWORK_ABORTED;
	DEFINITION
	{
		ACKNOWLEDGE(DCT_JOIN_NETWORK_NO_ACTION_TAKEN);
	}
}

/*
Requests active advertising with time span given by user.
*/
METHOD method_request_active_advertising
{
    LABEL DCT_REQUEST_ACTIVE_ADVERTISING;
    HELP  DCT_REQUEST_ACTIVE_ADVERTISING_HELP;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[260];
        char disp_string2[60];
        long var_id[4];
        int sel;
        int slen1;
        int slen2;
        slen1 = 260;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(76, status);

#ifndef _PDM_
				if (join_status < 128)
#else
    		if (JOIN_STATUS_LSS_128)
#endif
        {
            ACKNOWLEDGE(DCT_DEVICE_IS_NOT_JOINED);
		       process_abort();
		  }

        if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
        {
		    ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
		    process_abort();
        }

        disp_string1= DCT_REQUEST_ACTIVE_ADVERTISING_ACTIVATE;
        get_dictionary_string(yes_no, disp_string2, slen2);
        sel= select_from_list(disp_string1, var_id, disp_string2);
        if(sel == 0) // 0 == yes
        {

#ifndef _PDM_
			lassign(shed_time, 32000 * 1800); // => 30minutes, Bugzilla 20577
#else
			lassign(shed_time, 1800); // PDM expects the seconds here.
#endif

			disp_string1= DCT_ENTER_SHED_TIME;
            GET_DEV_VAR_VALUE(disp_string1, shed_time); // time type doesn't work with sdc 4.01 beta
            disp_string1= DCT_WAIT_ACTIVE_ADVERTISING_COMPL;
            DELAY(2, disp_string1);
            send(770, status);
            if(status[STATUS_RESPONSE_CODE] == 0)
            {
                var_id[0]= VARID(shed_time);
                var_id[1]= VARID(advertising_period);
                var_id[2]= VARID(advertised_neighbors_number);
                var_id[3]= 0;
                disp_string1= DCT_REQUEST_ACTIVE_ADVERTISING_STATUS;
                acknowledge(disp_string1, var_id);
            }
        }
        if(status[STATUS_RESPONSE_CODE] != 0)
        {
            display_response_status(770, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
        }
        process_abort() ;
    }
}


/*
Ask user to set subdevice to 4.0 mA and trims lwa to 4.0 mA.
*/
METHOD method_trim_loop_current_zero
{
    LABEL DCT_METHOD_TRIM_LOOP_CURRENT_ZERO;
    HELP  DCT_METHOD_TRIM_LOOP_CURRENT_ZERO_HELP;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[300];
        char lcm;
        int slen1;

        slen1= 300;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(76, status);
        if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
        {
		    ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
		    process_abort();
        }

        add_abort_method(return_to_normal);

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        disp_string1= DCT_SET_FIELD_DEVICE_OUTPUT_4MA;
        ACKNOWLEDGE(disp_string1);
        disp_string1= DCT_QUERY_FIELD_DEVICE_OUTPUT_IS_4MA;
        ACKNOWLEDGE(disp_string1);

        send(7, status);
        if(status[STATUS_RESPONSE_CODE] == 0)
        {
            lcm= ivar_value(loop_current_mode);
            if(lcm != 1) /* check loop current mode is disabled */
            {
                iassign(loop_current_mode, 1); /* enable loop current mode */
                send(6, status);
                lcm= ivar_value(loop_current_mode);
            }
        }
        if(status[STATUS_RESPONSE_CODE] == 0)
        {
            assign_float(PV.DAQ.ANALOG_VALUE, 4.0);
            send(45, status);
            if((status[STATUS_RESPONSE_CODE] == 9) && (lcm == 1))
            {
                disp_string1= DCT_INCORRECT_CURRENT_VALUE;
                ACKNOWLEDGE(disp_string1);
                process_abort();
            }
        }
        if(status[STATUS_RESPONSE_CODE] == 0)
        {
            disp_string1= DCT_TRIM_SUCCEEDED;
            ACKNOWLEDGE(disp_string1);
        }
        if(status[STATUS_RESPONSE_CODE] != 0)
        {
            display_response_status(45, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
        }
        process_abort() ;
    }
}

/*
Ask user to set subdevice to 20.0 mA and trims lwa to 20.0 mA.
*/
METHOD method_trim_loop_current_gain
{
    LABEL DCT_METHOD_TRIM_LOOP_CURRENT_GAIN;
    HELP  DCT_METHOD_TRIM_LOOP_CURRENT_GAIN_HELP;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[300];
        char lcm;
        int slen1;

        slen1= 300;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(76, status);
        if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
        {
		    ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
		    process_abort();
        }

        add_abort_method(return_to_normal);

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        disp_string1= DCT_SET_FIELD_DEVICE_OUTPUT_20MA;
        ACKNOWLEDGE(disp_string1);
        disp_string1= DCT_QUERY_FIELD_DEVICE_OUTPUT_IS_20MA;
        ACKNOWLEDGE(disp_string1);

        send(7, status);
        if(status[STATUS_RESPONSE_CODE] == 0)
        {
            lcm= ivar_value(loop_current_mode);
            if(lcm != 1) /* check loop current mode is disabled */
            {
                iassign(loop_current_mode, 1); /* enable loop current mode */
                send(6, status);
                lcm= ivar_value(loop_current_mode);
            }
        }
        if(status[STATUS_RESPONSE_CODE] == 0)
        {
            assign_float(PV.DAQ.ANALOG_VALUE, 20.0);
            send(46, status);
            if((status[STATUS_RESPONSE_CODE] == 9) && (lcm == 1))
            {
                disp_string1= DCT_INCORRECT_CURRENT_VALUE;
                ACKNOWLEDGE(disp_string1);
                process_abort();
            }
        }
        if(status[STATUS_RESPONSE_CODE] == 0)
        {
            disp_string1= DCT_TRIM_SUCCEEDED;
            ACKNOWLEDGE(disp_string1);
        }
        if(status[STATUS_RESPONSE_CODE] != 0)
        {
            display_response_status(45, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
        }
        process_abort() ;
    }
}


/*
  This is the post edit action method for simulation_value.
  It executes command 40.
*/
METHOD method_write_simulation_value
{
    LABEL DCT_UNUSED;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[300];
        int slen1;
        slen1= 300;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(76, status);
        if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
        {
		    ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
		    process_abort();
        }

        send(40, status);
        if(status[STATUS_RESPONSE_CODE] != 0)
        {
            display_response_status(40, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
        }
    }
}

/*
  Resets the config changed counter in the device
  */
METHOD method_reset_config_change_counter
{
    LABEL DCT_RESET_CONFIG_CHANGE_COUNTER;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[300];
        int slen1;
        slen1= 300;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(76, status);
        if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
        {
		    ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
		    process_abort();
        }

        send_command_trans(38, 1);

        disp_string1= DCT_RESET_CONFIG_CHANGE_COUNTER_DONE;
        ACKNOWLEDGE(disp_string1);
    }
}


//#18442
//If Country code set to 'Japan' make following:
//  (1) set radio power to 0
//  (2) set SI unit     to 'SI only' (1)

METHOD method_country_code_edited
{
	LABEL DCT_UNUSED;
	DEFINITION
	{
		char status[STATUS_SIZE];

		send(76, status);
		if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
		{
			//ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED); // Message should not be displayed.
			process_abort();
		}
		else
		{
			if(country_code == en_country_code_Japan)
			{
#ifndef _PDM_
				if (join_status < 128)
#else
				if (JOIN_STATUS_LSS_128)
#endif
				{
					iassign(radio_power_output, 0);
				}

				iassign(si_control, 1);
				/* do not write_country_code by send_command(513) here
				because this will be done with country code write.
				If written here SDC an AMS sometimes crash */
#ifndef _PDM_
				send_command_trans(513,2);
#else
				send_command(513);
#endif

#ifndef _PDM_
				if (join_status < 128)
#else
				if (JOIN_STATUS_LSS_128)
#endif
				{
					send_command(797  /*  write_radio_power_output*/);
				}
				save_values();
			}
		}
	}
}

// execute device reset and wait for device to recover
METHOD method_reset
{
    LABEL DCT_METHOD_RESET;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[300];
        char disp_string2[60];
        long var_id[4];
        int slen1;
        int slen2;
				int sel;

        IGNORE_NO_DEVICE();
        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_NO_DEVICE();

        send(76, status);
        if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
        {
			    ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
			    process_abort();
        }

        slen1= 300;
        slen2= 60;
        disp_string1= DCT_LABEL_CONFIRM_RESET;
				get_dictionary_string(yes_no, disp_string2, slen2);
				sel= select_from_list(disp_string1, var_id, disp_string2);
				if (sel == 0) /* 0 == yes == execute reset */
				{
				    send(42, status);
				    if(status[STATUS_RESPONSE_CODE] != 0)
				    {
				        display_response_status(42, status[STATUS_RESPONSE_CODE]);
				        get_dictionary_string(abort_last_error, disp_string1, slen1);
				        DELAY(2, disp_string1);
				    }
				    else
				    {
				        disp_string1= DCT_LABEL_EXECUTE_RESET;
				        DELAY(30, disp_string1);
				        send(0, status);
				    }
				}
    }
}

// start self test execution
METHOD method_self_test
{
    LABEL DCT_METHOD_SELF_TEST;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[300];
        int slen1;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        send(76, status);
        if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
        {
		    ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
		    process_abort();
        }

        slen1= 300;
        send(41, status);
        if(status[STATUS_RESPONSE_CODE] != 0)
        {
            display_response_status(41, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
        }
        else
        {
            disp_string1= DCT_LABEL_SELF_TEST_STARTED;
            ACKNOWLEDGE(disp_string1);
        }
    }
}


METHOD method_simulation
{
    LABEL DCT_METHOD_SIMULATION;
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[300];
        char disp_string2[60];
        long var_id[4];
        int unit_save;                      //Unit save Variable
        int slen1;
        int slen2;
        int sel, i;
        long ldv;

        slen1= 300;
        slen2= 60;

        send(76, status);
        if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
        {
		    ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
		    process_abort();
        }

        disp_string1= DCT_QUERY_SIMULATION_ON;
        get_dictionary_string(yes_no, disp_string2, slen2);
        sel= select_from_list(disp_string1, var_id, disp_string2);
        if(sel == 0) /* 0 == yes == simulation on */
        {
            disp_string1= DCT_SIMULATION_VARIABLE_SELECT;
            GET_DEV_VAR_VALUE (disp_string1, device_variable_code_simulation_enum);
            save_values();

            if (device_variable_code_simulation_enum==245)  //Exception if Loop Current is choosen
            {
                var_id[0]= VARID(deviceVariablesSimulation[device_variable_code_simulation_enum].DIGITAL_UNITS);
                disp_string1= DCT_SIMULATION_VALUE_SET;
                var_id[1]= 39;                                  //39 == [mA]
                unit_save = PV.DEVICE_VARIABLE.DIGITAL_UNITS;   //Saving of presently PV unit
                PV.DEVICE_VARIABLE.DIGITAL_UNITS=39;            //PV unit will be changed to [mA]
                get_dev_var_value(disp_string1, var_id, deviceVariablesSimulation[device_variable_code_simulation_enum].DIGITAL_VALUE);
                PV.DEVICE_VARIABLE.DIGITAL_UNITS= unit_save;    //restoring of PV unit
            }
            else
            {
                var_id[0]= VARID(deviceVariablesSimulation[device_variable_code_simulation_enum].DIGITAL_UNITS);
                disp_string1= DCT_SIMULATION_VALUE_SET;
                var_id[1]= 0;
                get_dev_var_value(disp_string1, var_id, deviceVariablesSimulation[device_variable_code_simulation_enum].DIGITAL_VALUE);
            }


            iassign(write_device_variable_code, 1); // switch simulation on (set to fixed mode)
            ldv= lvar_value(device_variable_code_simulation_enum);
            if(ldv == 245)
            {
                send_trans(79, 2, status);
            }
            else
            {
                if(ldv == 243)
                {
                    send_trans(79, 1, status);
                }
                else
                {
                    send_trans(79, 0, status);
                }
            }
            if(status[STATUS_RESPONSE_CODE] != 0)
            {
                display_response_status(79, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
            }
            else
            {
                disp_string1= DCT_SIMULATION_IS_ON;
            }
            ACKNOWLEDGE(disp_string1);
        }
        else
        {
            disp_string1= DCT_QUERY_SIMULATION_OFF;
            get_dictionary_string(yes_no, disp_string2, slen2);
            sel= select_from_list(disp_string1, var_id, disp_string2);
            if(sel == 0) /* 0 == yes == simulation off */
            {
                // Switch simulation off for all elements of deviceVariablesSimulation array.
                iassign(write_device_variable_code, 0); // switch simulation off
                for(i= 0; i <= 9; ++i)
                {
                    lassign(device_variable_code_simulation_enum, i);
                    send_trans(79, 0, status);
                }
                send_trans(79, 1, status);
                send_trans(79, 2, status);
                disp_string1= DCT_SIMULATION_IS_OFF;
                ACKNOWLEDGE(disp_string1);
            }
        }
    }
}

VARIABLE burst_command_number_enumerated
{
    LABEL DCT_LABEL_BURST_COMMAND_NUMBER_ENUMERATED_N;
    HELP [burst_command_number_help];
    CLASS HART;
    TYPE ENUMERATED (2)
    {
        {1,  [read_primary_variable], [pv_help] },
        {2,  [read_percent_range_and_current], [percent_rnge_help] },
        {3,  [read_dyn_vars_and_current], [all_process_var_help] },
        {9,  [read_device_variables_with_status], [read_device_variables_with_status_help] },
        {33, [read_device_variables],[read_device_variables_help] },
        {48, "Cmd 48: Read Additional Device Status", "Read Additional Device - This command reads additional device status information"}
    }
}

VARIABLE burst_variable_code_integer
{
    LABEL DCT_BURST_VARIABLE_CODE_LABEL;
    HELP DCT_BURST_VARIABLE_CODE_HELP;
    CLASS HART;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE UNSIGNED_INTEGER(1);
}

VARIABLE lock_dust_flash_update
{
    LABEL DCT_WIRELESS_MODULE_UPDATE_FIRMWARE_LOCK;
    CLASS HART;
		HANDLING IF ( (lock_device_status_code & 1) != 0) { READ; } ELSE { READ&WRITE; }
    TYPE ENUMERATED (1)
    {
        {0,  "Unlocked"},
        {1,   "Locked" }
    }
}

VARIABLE lock_device_code_local
{
   LABEL DCT_LOCK_UNLOCK_DEVICE;
   HELP  [lock_unlock_device_help];

   CLASS LOCAL;
   HANDLING READ&WRITE;
   TYPE ENUMERATED
   {
      {0, "Unlocked"},
      {1, "Lock Temporary"},
      {2, "Lock Permanent"},
      {3, "Lock All"},
      { 255, "Enter Lock State"}
   }

   DEFAULT_VALUE 255;

   POST_EDIT_ACTIONS
   {
      method_write_lock_device_code
   }
}

METHOD method_write_lock_device_code
{
   DEFINITION
   {
      char status[STATUS_SIZE];
      int targetLockCode;

      targetLockCode = ivar_value(lock_device_code_local);
      if(targetLockCode != 255)
      {
         assign_var(lock_device_code, lock_device_code_local);
         send(71, status);//write the lock code to the device
         send(76, status);//read the lock status back
         iassign(lock_device_code_local, 255);
      }
      save_values();
   }
}


//*********************************************************************
//*********************************************************************
//                NEIGHBOR AREA
//*********************************************************************
//*********************************************************************

VARIABLE neighbor_table_index
{
    LABEL DCT_NEIGHBOR_TABLE_INDEX;
    CLASS HART;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
    VALIDITY IF
    (
#ifndef _PDM_
			join_status >= 128
#else
    	JOIN_STATUS_GEQ_128
#endif
    ) {TRUE;} ELSE {FALSE;}
}


VARIABLE number_of_neighbor_entries_read_multi
{
    LABEL DCT_NUMBER_OF_NEIGHBOR_ENTRIES_READ_MULTI;
    CLASS HART;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
    VALIDITY IF
    (
#ifndef _PDM_
			join_status >= 128
#else
    	JOIN_STATUS_GEQ_128
#endif
    ) {TRUE;} ELSE {FALSE;}
}

VARIABLE total_number_of_neighbors
{
    LABEL DCT_TOTAL_NUMBER_OF_NEIGHBORS;
    CLASS HART&DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
    VALIDITY IF
    (
#ifndef _PDM_
			join_status >= 128
#else
    	JOIN_STATUS_GEQ_128
#endif
    ) {TRUE;} ELSE {FALSE;}
}

// This dummy variable just shows value 0 if validity of
// total_number_of_neighbors is FALSE.
VARIABLE total_number_of_neighbors_null
{
    LABEL DCT_TOTAL_NUMBER_OF_NEIGHBORS;
    CLASS LOCAL;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
    DEFAULT_VALUE 0;
		VALIDITY IF
		(
#ifndef _PDM_
				join_status < 128
#else
				JOIN_STATUS_LSS_128
#endif
		)
     {TRUE;} ELSE {FALSE;}
}
#ifndef _PDM_
#define NEIGHBORMEMBERS(num)\
VARIABLE nickname_of_neighbor_##num\
{\
    LABEL DCT_NICKNAME_OF_NEIGHBOR;\
    CLASS HART;\
    HANDLING READ;\
    TYPE UNSIGNED_INTEGER(2);\
    VALIDITY IF (join_status >= 128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE neighbor_flags_##num\
{\
    LABEL DCT_NEIGHBOR_FLAGS;\
    CLASS HART;\
    HANDLING READ;\
    TYPE BIT_ENUMERATED\
    {\
         {0x00, "NO LABEL"}\
         {0x01, "NO LABEL"}\
         {0x02, "NO LABEL"}\
         {0x04, "NO LABEL"}\
         {0x10, "NO LABEL"}\
         {0x20, "NO LABEL"}\
         {0x40, "NO LABEL"}\
\
   }\
   VALIDITY IF (join_status >= 128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE mean_rsl_##num\
{\
    LABEL DCT_MEAN_RSL;\
    CLASS HART;\
    HANDLING READ;\
    TYPE INTEGER;\
   VALIDITY IF (join_status >= 128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE number_of_packets_transmitted_to_this_neighbor_##num\
{\
    LABEL DCT_NUMBER_OF_PACKETS_TRANSMITTED_TO_THIS_NEIGHBOR;\
    CLASS HART & DYNAMIC;\
    HANDLING READ;\
    TYPE UNSIGNED_INTEGER(2);\
   VALIDITY IF (join_status >= 128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE number_of_failed_transmits_##num\
{\
    LABEL DCT_NUMBER_OF_FAILED_TRANSMITS;\
    CLASS HART;\
    HANDLING READ;\
    TYPE UNSIGNED_INTEGER(2);\
   VALIDITY IF (join_status >= 128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE number_of_packets_received_from_this_neighbor_##num\
{\
    LABEL DCT_NUMBER_OF_PACKETS_RECEIVED_FROM_THIS_NEIGHBOR;\
    CLASS HART;\
    HANDLING READ;\
    TYPE UNSIGNED_INTEGER(2);\
   VALIDITY IF (join_status >= 128) {TRUE;} ELSE {FALSE;}\
}
#else
#define NEIGHBORMEMBERS(num)\
VARIABLE nickname_of_neighbor_##num\
{\
    LABEL DCT_NICKNAME_OF_NEIGHBOR;\
    CLASS HART;\
    HANDLING READ;\
    TYPE UNSIGNED_INTEGER(2);\
    VALIDITY IF (JOIN_STATUS_GEQ_128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE neighbor_flags_##num\
{\
    LABEL DCT_NEIGHBOR_FLAGS;\
    CLASS HART;\
    HANDLING READ;\
    TYPE BIT_ENUMERATED\
    {\
         {0x00, "NO LABEL"}\
         {0x01, "NO LABEL"}\
         {0x02, "NO LABEL"}\
         {0x04, "NO LABEL"}\
         {0x10, "NO LABEL"}\
         {0x20, "NO LABEL"}\
         {0x40, "NO LABEL"}\
\
   }\
   VALIDITY IF (JOIN_STATUS_GEQ_128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE mean_rsl_##num\
{\
    LABEL DCT_MEAN_RSL;\
    CLASS HART;\
    HANDLING READ;\
    TYPE INTEGER;\
   VALIDITY IF (JOIN_STATUS_GEQ_128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE number_of_packets_transmitted_to_this_neighbor_##num\
{\
    LABEL DCT_NUMBER_OF_PACKETS_TRANSMITTED_TO_THIS_NEIGHBOR;\
    CLASS HART & DYNAMIC;\
    HANDLING READ;\
    TYPE UNSIGNED_INTEGER(2);\
   VALIDITY IF (JOIN_STATUS_GEQ_128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE number_of_failed_transmits_##num\
{\
    LABEL DCT_NUMBER_OF_FAILED_TRANSMITS;\
    CLASS HART;\
    HANDLING READ;\
    TYPE UNSIGNED_INTEGER(2);\
   VALIDITY IF (JOIN_STATUS_GEQ_128) {TRUE;} ELSE {FALSE;}\
}\
VARIABLE number_of_packets_received_from_this_neighbor_##num\
{\
    LABEL DCT_NUMBER_OF_PACKETS_RECEIVED_FROM_THIS_NEIGHBOR;\
    CLASS HART;\
    HANDLING READ;\
    TYPE UNSIGNED_INTEGER(2);\
   VALIDITY IF (JOIN_STATUS_GEQ_128) {TRUE;} ELSE {FALSE;}\
}
#endif
NEIGHBORMEMBERS(0)
NEIGHBORMEMBERS(1)
NEIGHBORMEMBERS(2)
NEIGHBORMEMBERS(3)
NEIGHBORMEMBERS(4)
NEIGHBORMEMBERS(5)
NEIGHBORMEMBERS(6)
NEIGHBORMEMBERS(7)
NEIGHBORMEMBERS(8)

ARRAY OF COLLECTION health_states
{
        ELEMENTS
        {
            0,    health_state0, "Health State 0";
            1,    health_state1, "Health State 1";
            2,    health_state2, "Health State 2";
            3,    health_state3, "Health State 3";
            4,    health_state4, "Health State 4";
            5,    health_state5, "Health State 5";
            6,    health_state6, "Health State 6";
            7,    health_state7, "Health State 7";
            8,    health_state8, "Health State 8";
        }
}

//COLLECTION MACRO

#define HEALTHSTATE(num) COLLECTION OF VARIABLE health_state##num{\
    MEMBERS\
    {\
         NICKNAME_OF_NEIGHBOR,                     nickname_of_neighbor_##num,                           DCT_NICKNAME_OF_NEIGHBOR;\
         NEIGHBOR_FLAGS,                           neighbor_flags_##num,                                 DCT_NEIGHBOR_FLAGS;\
         MEAN_RSL,                                 mean_rsl_##num,                                       DCT_MEAN_RSL;\
         PACKETS_SEND_TO_NEIGHBOR,                 number_of_packets_transmitted_to_this_neighbor_##num, DCT_NUMBER_OF_PACKETS_TRANSMITTED_TO_THIS_NEIGHBOR;\
         FAILED_TRANSMITS,                         number_of_failed_transmits_##num,                     DCT_NUMBER_OF_FAILED_TRANSMITS;\
         PACKETS_RECEIVED_FROM_NEIGHBOR,           number_of_packets_received_from_this_neighbor_##num,  DCT_NUMBER_OF_PACKETS_RECEIVED_FROM_THIS_NEIGHBOR;\
   }\
}

HEALTHSTATE(0)
HEALTHSTATE(1)
HEALTHSTATE(2)
HEALTHSTATE(3)
HEALTHSTATE(4)
HEALTHSTATE(5)
HEALTHSTATE(6)
HEALTHSTATE(7)
HEALTHSTATE(8)

//*********************************************************************
//                      COMMAND 780
//*********************************************************************



COMMAND read_device_health_state
{
    NUMBER 780;
    OPERATION READ;

    TRANSACTION 1
    {
        REQUEST {0,1}
        REPLY
        {
            response_code, device_status,
            DUMMY_16BITS_PDM

            neighbor_table_index,
            number_of_neighbor_entries_read_multi,
            total_number_of_neighbors,

            health_states[0].NICKNAME_OF_NEIGHBOR,
            health_states[0].NEIGHBOR_FLAGS,
            health_states[0].MEAN_RSL,
            health_states[0].PACKETS_SEND_TO_NEIGHBOR,
            health_states[0].FAILED_TRANSMITS,
            health_states[0].PACKETS_RECEIVED_FROM_NEIGHBOR
        }
    }

    TRANSACTION 2
    {
        REQUEST {0,2}
        REPLY
        {
            response_code, device_status,
            DUMMY_16BITS_PDM

            neighbor_table_index,
            number_of_neighbor_entries_read_multi,
            total_number_of_neighbors,

            health_states[0].NICKNAME_OF_NEIGHBOR,
            health_states[0].NEIGHBOR_FLAGS,
            health_states[0].MEAN_RSL,
            health_states[0].PACKETS_SEND_TO_NEIGHBOR,
            health_states[0].FAILED_TRANSMITS,
            health_states[0].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[1].NICKNAME_OF_NEIGHBOR,
            health_states[1].NEIGHBOR_FLAGS,
            health_states[1].MEAN_RSL,
            health_states[1].PACKETS_SEND_TO_NEIGHBOR,
            health_states[1].FAILED_TRANSMITS,
            health_states[1].PACKETS_RECEIVED_FROM_NEIGHBOR
        }
    }

    TRANSACTION 3
    {
        REQUEST {0,3}
        REPLY
        {
            response_code, device_status,
            DUMMY_16BITS_PDM

            neighbor_table_index,
            number_of_neighbor_entries_read_multi,
            total_number_of_neighbors,

            health_states[0].NICKNAME_OF_NEIGHBOR,
            health_states[0].NEIGHBOR_FLAGS,
            health_states[0].MEAN_RSL,
            health_states[0].PACKETS_SEND_TO_NEIGHBOR,
            health_states[0].FAILED_TRANSMITS,
            health_states[0].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[1].NICKNAME_OF_NEIGHBOR,
            health_states[1].NEIGHBOR_FLAGS,
            health_states[1].MEAN_RSL,
            health_states[1].PACKETS_SEND_TO_NEIGHBOR,
            health_states[1].FAILED_TRANSMITS,
            health_states[1].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[2].NICKNAME_OF_NEIGHBOR,
            health_states[2].NEIGHBOR_FLAGS,
            health_states[2].MEAN_RSL,
            health_states[2].PACKETS_SEND_TO_NEIGHBOR,
            health_states[2].FAILED_TRANSMITS,
            health_states[2].PACKETS_RECEIVED_FROM_NEIGHBOR
        }
    }

    TRANSACTION 4
    {
        REQUEST {0,4}
        REPLY
        {
            response_code, device_status,
            DUMMY_16BITS_PDM

            neighbor_table_index,
            number_of_neighbor_entries_read_multi,
            total_number_of_neighbors,

            health_states[0].NICKNAME_OF_NEIGHBOR,
            health_states[0].NEIGHBOR_FLAGS,
            health_states[0].MEAN_RSL,
            health_states[0].PACKETS_SEND_TO_NEIGHBOR,
            health_states[0].FAILED_TRANSMITS,
            health_states[0].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[1].NICKNAME_OF_NEIGHBOR,
            health_states[1].NEIGHBOR_FLAGS,
            health_states[1].MEAN_RSL,
            health_states[1].PACKETS_SEND_TO_NEIGHBOR,
            health_states[1].FAILED_TRANSMITS,
            health_states[1].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[2].NICKNAME_OF_NEIGHBOR,
            health_states[2].NEIGHBOR_FLAGS,
            health_states[2].MEAN_RSL,
            health_states[2].PACKETS_SEND_TO_NEIGHBOR,
            health_states[2].FAILED_TRANSMITS,
            health_states[2].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[3].NICKNAME_OF_NEIGHBOR,
            health_states[3].NEIGHBOR_FLAGS,
            health_states[3].MEAN_RSL,
            health_states[3].PACKETS_SEND_TO_NEIGHBOR,
            health_states[3].FAILED_TRANSMITS,
            health_states[3].PACKETS_RECEIVED_FROM_NEIGHBOR
        }
    }

    TRANSACTION 5
    {
        REQUEST {0,5}
        REPLY
        {
            response_code, device_status,
            DUMMY_16BITS_PDM

            neighbor_table_index,
            number_of_neighbor_entries_read_multi,
            total_number_of_neighbors,

            health_states[0].NICKNAME_OF_NEIGHBOR,
            health_states[0].NEIGHBOR_FLAGS,
            health_states[0].MEAN_RSL,
            health_states[0].PACKETS_SEND_TO_NEIGHBOR,
            health_states[0].FAILED_TRANSMITS,
            health_states[0].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[1].NICKNAME_OF_NEIGHBOR,
            health_states[1].NEIGHBOR_FLAGS,
            health_states[1].MEAN_RSL,
            health_states[1].PACKETS_SEND_TO_NEIGHBOR,
            health_states[1].FAILED_TRANSMITS,
            health_states[1].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[2].NICKNAME_OF_NEIGHBOR,
            health_states[2].NEIGHBOR_FLAGS,
            health_states[2].MEAN_RSL,
            health_states[2].PACKETS_SEND_TO_NEIGHBOR,
            health_states[2].FAILED_TRANSMITS,
            health_states[2].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[3].NICKNAME_OF_NEIGHBOR,
            health_states[3].NEIGHBOR_FLAGS,
            health_states[3].MEAN_RSL,
            health_states[3].PACKETS_SEND_TO_NEIGHBOR,
            health_states[3].FAILED_TRANSMITS,
            health_states[3].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[4].NICKNAME_OF_NEIGHBOR,
            health_states[4].NEIGHBOR_FLAGS,
            health_states[4].MEAN_RSL,
            health_states[4].PACKETS_SEND_TO_NEIGHBOR,
            health_states[4].FAILED_TRANSMITS,
            health_states[4].PACKETS_RECEIVED_FROM_NEIGHBOR
        }
    }

    TRANSACTION 6
    {
        REQUEST {0,6}
        REPLY
        {
            response_code, device_status,
            DUMMY_16BITS_PDM

            neighbor_table_index,
            number_of_neighbor_entries_read_multi,
            total_number_of_neighbors,

            health_states[0].NICKNAME_OF_NEIGHBOR,
            health_states[0].NEIGHBOR_FLAGS,
            health_states[0].MEAN_RSL,
            health_states[0].PACKETS_SEND_TO_NEIGHBOR,
            health_states[0].FAILED_TRANSMITS,
            health_states[0].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[1].NICKNAME_OF_NEIGHBOR,
            health_states[1].NEIGHBOR_FLAGS,
            health_states[1].MEAN_RSL,
            health_states[1].PACKETS_SEND_TO_NEIGHBOR,
            health_states[1].FAILED_TRANSMITS,
            health_states[1].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[2].NICKNAME_OF_NEIGHBOR,
            health_states[2].NEIGHBOR_FLAGS,
            health_states[2].MEAN_RSL,
            health_states[2].PACKETS_SEND_TO_NEIGHBOR,
            health_states[2].FAILED_TRANSMITS,
            health_states[2].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[3].NICKNAME_OF_NEIGHBOR,
            health_states[3].NEIGHBOR_FLAGS,
            health_states[3].MEAN_RSL,
            health_states[3].PACKETS_SEND_TO_NEIGHBOR,
            health_states[3].FAILED_TRANSMITS,
            health_states[3].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[4].NICKNAME_OF_NEIGHBOR,
            health_states[4].NEIGHBOR_FLAGS,
            health_states[4].MEAN_RSL,
            health_states[4].PACKETS_SEND_TO_NEIGHBOR,
            health_states[4].FAILED_TRANSMITS,
            health_states[4].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[5].NICKNAME_OF_NEIGHBOR,
            health_states[5].NEIGHBOR_FLAGS,
            health_states[5].MEAN_RSL,
            health_states[5].PACKETS_SEND_TO_NEIGHBOR,
            health_states[5].FAILED_TRANSMITS,
            health_states[5].PACKETS_RECEIVED_FROM_NEIGHBOR
        }
    }

    TRANSACTION 7
    {
        REQUEST {0,7}
        REPLY
        {
            response_code, device_status,
            DUMMY_16BITS_PDM

            neighbor_table_index,
            number_of_neighbor_entries_read_multi,
            total_number_of_neighbors,

            health_states[0].NICKNAME_OF_NEIGHBOR,
            health_states[0].NEIGHBOR_FLAGS,
            health_states[0].MEAN_RSL,
            health_states[0].PACKETS_SEND_TO_NEIGHBOR,
            health_states[0].FAILED_TRANSMITS,
            health_states[0].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[1].NICKNAME_OF_NEIGHBOR,
            health_states[1].NEIGHBOR_FLAGS,
            health_states[1].MEAN_RSL,
            health_states[1].PACKETS_SEND_TO_NEIGHBOR,
            health_states[1].FAILED_TRANSMITS,
            health_states[1].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[2].NICKNAME_OF_NEIGHBOR,
            health_states[2].NEIGHBOR_FLAGS,
            health_states[2].MEAN_RSL,
            health_states[2].PACKETS_SEND_TO_NEIGHBOR,
            health_states[2].FAILED_TRANSMITS,
            health_states[2].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[3].NICKNAME_OF_NEIGHBOR,
            health_states[3].NEIGHBOR_FLAGS,
            health_states[3].MEAN_RSL,
            health_states[3].PACKETS_SEND_TO_NEIGHBOR,
            health_states[3].FAILED_TRANSMITS,
            health_states[3].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[4].NICKNAME_OF_NEIGHBOR,
            health_states[4].NEIGHBOR_FLAGS,
            health_states[4].MEAN_RSL,
            health_states[4].PACKETS_SEND_TO_NEIGHBOR,
            health_states[4].FAILED_TRANSMITS,
            health_states[4].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[5].NICKNAME_OF_NEIGHBOR,
            health_states[5].NEIGHBOR_FLAGS,
            health_states[5].MEAN_RSL,
            health_states[5].PACKETS_SEND_TO_NEIGHBOR,
            health_states[5].FAILED_TRANSMITS,
            health_states[5].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[6].NICKNAME_OF_NEIGHBOR,
            health_states[6].NEIGHBOR_FLAGS,
            health_states[6].MEAN_RSL,
            health_states[6].PACKETS_SEND_TO_NEIGHBOR,
            health_states[6].FAILED_TRANSMITS,
            health_states[6].PACKETS_RECEIVED_FROM_NEIGHBOR
        }
    }

    TRANSACTION 8
    {
        REQUEST {0,8}
        REPLY
        {
            response_code, device_status,
            DUMMY_16BITS_PDM

            neighbor_table_index,
            number_of_neighbor_entries_read_multi,
            total_number_of_neighbors,

            health_states[0].NICKNAME_OF_NEIGHBOR,
            health_states[0].NEIGHBOR_FLAGS,
            health_states[0].MEAN_RSL,
            health_states[0].PACKETS_SEND_TO_NEIGHBOR,
            health_states[0].FAILED_TRANSMITS,
            health_states[0].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[1].NICKNAME_OF_NEIGHBOR,
            health_states[1].NEIGHBOR_FLAGS,
            health_states[1].MEAN_RSL,
            health_states[1].PACKETS_SEND_TO_NEIGHBOR,
            health_states[1].FAILED_TRANSMITS,
            health_states[1].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[2].NICKNAME_OF_NEIGHBOR,
            health_states[2].NEIGHBOR_FLAGS,
            health_states[2].MEAN_RSL,
            health_states[2].PACKETS_SEND_TO_NEIGHBOR,
            health_states[2].FAILED_TRANSMITS,
            health_states[2].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[3].NICKNAME_OF_NEIGHBOR,
            health_states[3].NEIGHBOR_FLAGS,
            health_states[3].MEAN_RSL,
            health_states[3].PACKETS_SEND_TO_NEIGHBOR,
            health_states[3].FAILED_TRANSMITS,
            health_states[3].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[4].NICKNAME_OF_NEIGHBOR,
            health_states[4].NEIGHBOR_FLAGS,
            health_states[4].MEAN_RSL,
            health_states[4].PACKETS_SEND_TO_NEIGHBOR,
            health_states[4].FAILED_TRANSMITS,
            health_states[4].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[5].NICKNAME_OF_NEIGHBOR,
            health_states[5].NEIGHBOR_FLAGS,
            health_states[5].MEAN_RSL,
            health_states[5].PACKETS_SEND_TO_NEIGHBOR,
            health_states[5].FAILED_TRANSMITS,
            health_states[5].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[6].NICKNAME_OF_NEIGHBOR,
            health_states[6].NEIGHBOR_FLAGS,
            health_states[6].MEAN_RSL,
            health_states[6].PACKETS_SEND_TO_NEIGHBOR,
            health_states[6].FAILED_TRANSMITS,
            health_states[6].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[7].NICKNAME_OF_NEIGHBOR,
            health_states[7].NEIGHBOR_FLAGS,
            health_states[7].MEAN_RSL,
            health_states[7].PACKETS_SEND_TO_NEIGHBOR,
            health_states[7].FAILED_TRANSMITS,
            health_states[7].PACKETS_RECEIVED_FROM_NEIGHBOR
        }
    }

    TRANSACTION 9
    {
        REQUEST {0,9}
        REPLY
        {
            response_code, device_status,
            DUMMY_16BITS_PDM

            neighbor_table_index,
            number_of_neighbor_entries_read_multi,
            total_number_of_neighbors,

            health_states[0].NICKNAME_OF_NEIGHBOR,
            health_states[0].NEIGHBOR_FLAGS,
            health_states[0].MEAN_RSL,
            health_states[0].PACKETS_SEND_TO_NEIGHBOR,
            health_states[0].FAILED_TRANSMITS,
            health_states[0].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[1].NICKNAME_OF_NEIGHBOR,
            health_states[1].NEIGHBOR_FLAGS,
            health_states[1].MEAN_RSL,
            health_states[1].PACKETS_SEND_TO_NEIGHBOR,
            health_states[1].FAILED_TRANSMITS,
            health_states[1].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[2].NICKNAME_OF_NEIGHBOR,
            health_states[2].NEIGHBOR_FLAGS,
            health_states[2].MEAN_RSL,
            health_states[2].PACKETS_SEND_TO_NEIGHBOR,
            health_states[2].FAILED_TRANSMITS,
            health_states[2].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[3].NICKNAME_OF_NEIGHBOR,
            health_states[3].NEIGHBOR_FLAGS,
            health_states[3].MEAN_RSL,
            health_states[3].PACKETS_SEND_TO_NEIGHBOR,
            health_states[3].FAILED_TRANSMITS,
            health_states[3].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[4].NICKNAME_OF_NEIGHBOR,
            health_states[4].NEIGHBOR_FLAGS,
            health_states[4].MEAN_RSL,
            health_states[4].PACKETS_SEND_TO_NEIGHBOR,
            health_states[4].FAILED_TRANSMITS,
            health_states[4].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[5].NICKNAME_OF_NEIGHBOR,
            health_states[5].NEIGHBOR_FLAGS,
            health_states[5].MEAN_RSL,
            health_states[5].PACKETS_SEND_TO_NEIGHBOR,
            health_states[5].FAILED_TRANSMITS,
            health_states[5].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[6].NICKNAME_OF_NEIGHBOR,
            health_states[6].NEIGHBOR_FLAGS,
            health_states[6].MEAN_RSL,
            health_states[6].PACKETS_SEND_TO_NEIGHBOR,
            health_states[6].FAILED_TRANSMITS,
            health_states[6].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[7].NICKNAME_OF_NEIGHBOR,
            health_states[7].NEIGHBOR_FLAGS,
            health_states[7].MEAN_RSL,
            health_states[7].PACKETS_SEND_TO_NEIGHBOR,
            health_states[7].FAILED_TRANSMITS,
            health_states[7].PACKETS_RECEIVED_FROM_NEIGHBOR,
            health_states[8].NICKNAME_OF_NEIGHBOR,
            health_states[8].NEIGHBOR_FLAGS,
            health_states[8].MEAN_RSL,
            health_states[8].PACKETS_SEND_TO_NEIGHBOR,
            health_states[8].FAILED_TRANSMITS,
            health_states[8].PACKETS_RECEIVED_FROM_NEIGHBOR
        }
    }

    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
#ifndef _PDM_
        2,  DATA_ENTRY_ERROR, [invalid_selection];
#else
				2,  SUCCESS,          [no_command_specific_errors];
#endif
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        8,  MISC_WARNING,     "Set the nearest possible Value";
    }

 }

//*********************************************************************
//*********************************************************************
//             END OF NEIGHBOR AREA
//*********************************************************************
//*********************************************************************

#ifdef _PDM_
MENU download_variables
{
	LABEL DCT_DOWNLOAD_MENU;
	ITEMS
	{
		//MENU Identification_menu
		longTag,
		descriptor,
		date,
		message,
		polling_address,
		order_ident,//read only
		country_code
		si_control,

		// Primary Unit
		dev_var_current_Unit,
		// Secondary, Tertiary and Quaternary Units
		dev_var_battery_voltage_Unit,//read only
		dev_var_battery_voltage_w_load_Unit,//read only
		dev_var_battery_voltage_wo_load_test_Unit,//read only
		dev_var_consumed_energy_Unit,//read only
		dev_var_estimated_lifetime_Unit,//read only
		dev_var_normalized_consumed_energy_Unit,//read only
		dev_var_best_neighbour_Unit,//read only
		dev_var_second_best_neighbour_Unit,//read only
		dev_var_temperature_Unit,
		dev_var_temperature_max_Unit,
		dev_var_temperature_min_Unit,

		//MENU DeviceVariableMapping_menu
		primary_variable_code,
		secondary_variable_code,
		tertiary_variable_code,
		quaternary_variable_code,

		//MENU root_menu
		//Primary value
		dev_var_current_Value,//read only
		//Secondary, tertiary and quaternary values
		dev_var_battery_voltage_Value,//read only
		dev_var_battery_voltage_w_load_Value,//read only
		dev_var_battery_voltage_wo_load_test_Value,//read only
		dev_var_consumed_energy_Value,//read only
		dev_var_estimated_lifetime_Value,//read only
		dev_var_normalized_consumed_energy_Value,//read only
		dev_var_best_neighbour_Value,//read only
		dev_var_second_best_neighbour_Value,//read only
		dev_var_temperature_Value,//read only
		dev_var_temperature_max_Value,//read only
		dev_var_temperature_min_Value,//read only

		loopCurrent,

		//MENU device_info
		enp_version,//read only
		tag,
		serial_number,//read only
		order_code,//read only
		firmware_version,//read only

		//MENU lock_unlock_device_menu
		lock_device_status_code,//read only
		lock_dust_flash_update,
		dust_firmware_version,//read only

		//MENU WirelessCommunication_menu
		network_id,
		wireless_mode,//read only
		radio_power_output,
		join_shed_time,
		join_mode,
		join_status_byte_hi,//read only
		join_status_byte_lo,//read only
		total_number_of_neighbors,//read only
		advertisements,//read only
		join_attempts,//read only

		//MENU WiredCommunication_menu
		master_mode,
		response_preambles,
		retry_count,
		lowest_scan_address,
		highest_scan_address,
		//start_scan,

		//MENU PowerSupply_menu
		power_mode,
		devices_detected,//read only
		startup_voltage,
		startup_time,
		startup_current,
		operating_voltage,
		operating_time,
		power_on_time_after_external_trigger,
		configuration_time,
		last_time_battery_change,//read only

		//MENU Device(parIndex)_menu
		//MENU DiagDevice(parIndex)_menu
		IF (devices_detected >= 0x0002) {
			subdev1_expanded_device_type,//read only
			subdev1_device_id,//read only
			subdev1_universal_command_revision,//read only
			subdev1_long_tag,//read only
			subdev1_poll_adr, //read only
			stx_sent_to_subdev1,//read only
			ack_received_from_subdev1,//read only
	 		back_received_from_subdev1//read only
	 		IF (devices_detected >= 0x0003) {
				subdev2_expanded_device_type,//read only
				subdev2_device_id,//read only
				subdev2_universal_command_revision,//read only
				subdev2_long_tag,//read only
				subdev2_poll_adr, //read only
				stx_sent_to_subdev2,//read only
				ack_received_from_subdev2,//read only
		 		back_received_from_subdev2//read only
		 		IF (devices_detected >= 0x0004) {
					subdev3_expanded_device_type,//read only
					subdev3_device_id,//read only
					subdev3_universal_command_revision,//read only
					subdev3_long_tag,//read only
					subdev3_poll_adr, //read only
					stx_sent_to_subdev3,//read only
					ack_received_from_subdev3,//read only
			 		back_received_from_subdev3//read only
			 		IF (devices_detected >= 0x0005) {
						subdev4_expanded_device_type,//read only
						subdev4_device_id,//read only
						subdev4_universal_command_revision,//read only
						subdev4_long_tag,//read only
						subdev4_poll_adr, //read only
						stx_sent_to_subdev4,//read only
						ack_received_from_subdev4,//read only
				 		back_received_from_subdev4//read only
			 		}
		 		}
	 		}
		}
		//MENU mA_4_20_menu
		upperRange_value,
		lowerRange_value,
		dev_var_current_damping_value,
		configure_lower_false_current_alarm,
		configure_upper_false_current_alarm,
		configure_upper_false_current_warning,
		configure_lower_false_current_warning,

		//MENU Linearization_menu
		table_index,
		x_value,
		y_value,

		//MENU Burst(parIndex)_menu
		compat_burst_mode_0,
		compat_burst_subdev_index_0,
		compat_update_per_0,
		compat_max_update_per_0,
		compat_burst_trig_mode_0,
		compat_burst_trig_class_0,
		compat_burst_trig_units_0,
		compat_burst_trig_level_0,
		compat_burst_cmd_0,
		compat_burst_mode_1,
		compat_burst_subdev_index_1,
		compat_update_per_1,
		compat_max_update_per_1,
		compat_burst_trig_mode_1,
		compat_burst_trig_class_1,
		compat_burst_trig_units_1,
		compat_burst_trig_level_1,
		compat_burst_cmd_1,
		compat_burst_mode_2,
		compat_burst_subdev_index_2,
		compat_update_per_2,
		compat_max_update_per_2,
		compat_burst_trig_mode_2,
		compat_burst_trig_class_2,
		compat_burst_trig_units_2,
		compat_burst_trig_level_2,
		compat_burst_cmd_2,
		compat_burst_mode_3,
		compat_burst_subdev_index_3,
		compat_update_per_3,
		compat_max_update_per_3,
		compat_burst_trig_mode_3,
		compat_burst_trig_class_3,
		compat_burst_trig_units_3,
		compat_burst_trig_level_3,
		compat_burst_cmd_3,
		compat_burst_mode_4,
		compat_burst_subdev_index_4,
		compat_update_per_4,
		compat_max_update_per_4,
		compat_burst_trig_mode_4,
		compat_burst_trig_class_4,
		compat_burst_trig_units_4,
		compat_burst_trig_level_4,
		compat_burst_cmd_4,
		compat_burst_mode_5,
		compat_burst_subdev_index_5,
		compat_update_per_5,
		compat_max_update_per_5,
		compat_burst_trig_mode_5,
		compat_burst_trig_class_5,
		compat_burst_trig_units_5,
		compat_burst_trig_level_5,
		compat_burst_cmd_5,
		compat_burst_mode_6,
		compat_burst_subdev_index_6,
		compat_update_per_6,
		compat_max_update_per_6,
		compat_burst_trig_mode_6,
		compat_burst_trig_class_6,
		compat_burst_trig_units_6,
		compat_burst_trig_level_6,
		compat_burst_cmd_6,
		compat_burst_mode_7,
		compat_burst_subdev_index_7,
		compat_update_per_7,
		compat_max_update_per_7,
		compat_burst_trig_mode_7,
		compat_burst_trig_class_7,
		compat_burst_trig_units_7,
		compat_burst_trig_level_7,
		compat_burst_cmd_7,
		compat_burst_mode_8,
		compat_burst_subdev_index_8,
		compat_update_per_8,
		compat_max_update_per_8,
		compat_burst_trig_mode_8,
		compat_burst_trig_class_8,
		compat_burst_trig_units_8,
		compat_burst_trig_level_8,
		compat_burst_cmd_8,
		compat_burst_mode_9,
		compat_burst_subdev_index_9,
		compat_update_per_9,
		compat_max_update_per_9,
		compat_burst_trig_mode_9,
		compat_burst_trig_class_9,
		compat_burst_trig_units_9,
		compat_burst_trig_level_9,
		compat_burst_cmd_9,
		compat_burst_vars_0_0,
		compat_burst_vars_0_1,
		compat_burst_vars_0_2,
		compat_burst_vars_0_3,
		compat_burst_vars_0_4,
		compat_burst_vars_0_5,
		compat_burst_vars_0_6,
		compat_burst_vars_0_7,
		compat_burst_vars_1_0,
		compat_burst_vars_1_1,
		compat_burst_vars_1_2,
		compat_burst_vars_1_3,
		compat_burst_vars_1_4,
		compat_burst_vars_1_5,
		compat_burst_vars_1_6,
		compat_burst_vars_1_7,
		compat_burst_vars_2_0,
		compat_burst_vars_2_1,
		compat_burst_vars_2_2,
		compat_burst_vars_2_3,
		compat_burst_vars_2_4,
		compat_burst_vars_2_5,
		compat_burst_vars_2_6,
		compat_burst_vars_2_7,
		compat_burst_vars_3_0,
		compat_burst_vars_3_1,
		compat_burst_vars_3_2,
		compat_burst_vars_3_3,
		compat_burst_vars_3_4,
		compat_burst_vars_3_5,
		compat_burst_vars_3_6,
		compat_burst_vars_3_7,
		compat_burst_vars_4_0,
		compat_burst_vars_4_1,
		compat_burst_vars_4_2,
		compat_burst_vars_4_3,
		compat_burst_vars_4_4,
		compat_burst_vars_4_5,
		compat_burst_vars_4_6,
		compat_burst_vars_4_7,
		compat_burst_vars_5_0,
		compat_burst_vars_5_1,
		compat_burst_vars_5_2,
		compat_burst_vars_5_3,
		compat_burst_vars_5_4,
		compat_burst_vars_5_5,
		compat_burst_vars_5_6,
		compat_burst_vars_5_7,
		compat_burst_vars_6_0,
		compat_burst_vars_6_1,
		compat_burst_vars_6_2,
		compat_burst_vars_6_3,
		compat_burst_vars_6_4,
		compat_burst_vars_6_5,
		compat_burst_vars_6_6,
		compat_burst_vars_6_7,
		compat_burst_vars_7_0,
		compat_burst_vars_7_1,
		compat_burst_vars_7_2,
		compat_burst_vars_7_3,
		compat_burst_vars_7_4,
		compat_burst_vars_7_5,
		compat_burst_vars_7_6,
		compat_burst_vars_7_7,
		compat_burst_vars_8_0,
		compat_burst_vars_8_1,
		compat_burst_vars_8_2,
		compat_burst_vars_8_3,
		compat_burst_vars_8_4,
		compat_burst_vars_8_5,
		compat_burst_vars_8_6,
		compat_burst_vars_8_7,
		compat_burst_vars_9_0,
		compat_burst_vars_9_1,
		compat_burst_vars_9_2,
		compat_burst_vars_9_3,
		compat_burst_vars_9_4,
		compat_burst_vars_9_5,
		compat_burst_vars_9_6,
		compat_burst_vars_9_7,

		//MENU EventNotification(parIndex)_Event_menu
		compat_event_status_0,//read only
		compat_event_notification_control_n_0,
		compat_time_of_first_unack_event_0,//read only
		compat_event_notification_retry_time_event_0,
		compat_max_update_time_event_0,
		compat_event_debounce_interval_event_0,
		compat_event_subdevice_index_0,
		compat_device_status_mask_0,
		compat_extended_fld_device_status_mask_0,
		compat_analog_channel_fixed1_mask_0,
		compat_analog_channel_saturated1_mask_0,
		compat_standardized_status_0_mask_0,
		compat_standardized_status_2_mask_0,
		compat_standardized_status_3_mask_0,
		compat_device_specific_status_0_mask_0,
		compat_device_specific_status_1_mask_0,
		compat_device_specific_status_2_mask_0,
		compat_device_specific_status_3_mask_0,
		compat_device_specific_status_4_mask_0,
		compat_device_specific_status_5_mask_0,
		compat_device_specific_status_14_mask_0,
		compat_device_specific_status_15_mask_0,
		compat_device_specific_status_16_mask_0,
		compat_device_specific_status_17_mask_0,
		compat_device_specific_status_18_mask_0,
		compat_device_specific_status_19_mask_0,
		compat_device_specific_status_20_mask_0,
		compat_device_specific_status_21_mask_0,
		compat_device_specific_status_22_mask_0,
		compat_device_specific_status_23_mask_0,
		compat_device_specific_status_24_mask_0,

		compat_event_status_1,//read only
		compat_event_notification_control_n_1,
		compat_time_of_first_unack_event_1,//read only
		compat_event_notification_retry_time_event_1,
		compat_max_update_time_event_1,
		compat_event_debounce_interval_event_1,
		compat_event_subdevice_index_1,
		compat_device_status_mask_1,
		compat_extended_fld_device_status_mask_1,
		compat_analog_channel_fixed1_mask_1,
		compat_analog_channel_saturated1_mask_1,
		compat_standardized_status_0_mask_1,
		compat_standardized_status_2_mask_1,
		compat_standardized_status_3_mask_1,
		compat_device_specific_status_0_mask_1,
		compat_device_specific_status_1_mask_1,
		compat_device_specific_status_2_mask_1,
		compat_device_specific_status_3_mask_1,
		compat_device_specific_status_4_mask_1,
		compat_device_specific_status_5_mask_1,
		compat_device_specific_status_14_mask_1,
		compat_device_specific_status_15_mask_1,
		compat_device_specific_status_16_mask_1,
		compat_device_specific_status_17_mask_1,
		compat_device_specific_status_18_mask_1,
		compat_device_specific_status_19_mask_1,
		compat_device_specific_status_20_mask_1,
		compat_device_specific_status_21_mask_1,
		compat_device_specific_status_22_mask_1,
		compat_device_specific_status_23_mask_1,
		compat_device_specific_status_24_mask_1,

		compat_event_status_2,//read only
		compat_event_notification_control_n_2,
		compat_time_of_first_unack_event_2,//read only
		compat_event_notification_retry_time_event_2,
		compat_max_update_time_event_2,
		compat_event_debounce_interval_event_2,
		compat_event_subdevice_index_2,
		compat_device_status_mask_2,
		compat_extended_fld_device_status_mask_2,
		compat_analog_channel_fixed1_mask_2,
		compat_analog_channel_saturated1_mask_2,
		compat_standardized_status_0_mask_2,
		compat_standardized_status_2_mask_2,
		compat_standardized_status_3_mask_2,
		compat_device_specific_status_0_mask_2,
		compat_device_specific_status_1_mask_2,
		compat_device_specific_status_2_mask_2,
		compat_device_specific_status_3_mask_2,
		compat_device_specific_status_4_mask_2,
		compat_device_specific_status_5_mask_2,
		compat_device_specific_status_14_mask_2,
		compat_device_specific_status_15_mask_2,
		compat_device_specific_status_16_mask_2,
		compat_device_specific_status_17_mask_2,
		compat_device_specific_status_18_mask_2,
		compat_device_specific_status_19_mask_2,
		compat_device_specific_status_20_mask_2,
		compat_device_specific_status_21_mask_2,
		compat_device_specific_status_22_mask_2,
		compat_device_specific_status_23_mask_2,
		compat_device_specific_status_24_mask_2,

		compat_event_status_3,//read only
		compat_event_notification_control_n_3,
		compat_time_of_first_unack_event_3,//read only
		compat_event_notification_retry_time_event_3,
		compat_max_update_time_event_3,
		compat_event_debounce_interval_event_3,
		compat_event_subdevice_index_3,
		compat_device_status_mask_3,
		compat_extended_fld_device_status_mask_3,
		compat_analog_channel_fixed1_mask_3,
		compat_analog_channel_saturated1_mask_3,
		compat_standardized_status_0_mask_3,
		compat_standardized_status_2_mask_3,
		compat_standardized_status_3_mask_3,
		compat_device_specific_status_0_mask_3,
		compat_device_specific_status_1_mask_3,
		compat_device_specific_status_2_mask_3,
		compat_device_specific_status_3_mask_3,
		compat_device_specific_status_4_mask_3,
		compat_device_specific_status_5_mask_3,
		compat_device_specific_status_14_mask_3,
		compat_device_specific_status_15_mask_3,
		compat_device_specific_status_16_mask_3,
		compat_device_specific_status_17_mask_3,
		compat_device_specific_status_18_mask_3,
		compat_device_specific_status_19_mask_3,
		compat_device_specific_status_20_mask_3,
		compat_device_specific_status_21_mask_3,
		compat_device_specific_status_22_mask_3,
		compat_device_specific_status_23_mask_3,
		compat_device_specific_status_24_mask_3,

		compat_event_status_4,//read only
		compat_event_notification_control_n_4,
		compat_time_of_first_unack_event_4,//read only
		compat_event_notification_retry_time_event_4,
		compat_max_update_time_event_4,
		compat_event_debounce_interval_event_4,
		compat_event_subdevice_index_4,
		compat_device_status_mask_4,
		compat_extended_fld_device_status_mask_4,
		compat_analog_channel_fixed1_mask_4,
		compat_analog_channel_saturated1_mask_4,
		compat_standardized_status_0_mask_4,
		compat_standardized_status_2_mask_4,
		compat_standardized_status_3_mask_4,
		compat_device_specific_status_0_mask_4,
		compat_device_specific_status_1_mask_4,
		compat_device_specific_status_2_mask_4,
		compat_device_specific_status_3_mask_4,
		compat_device_specific_status_4_mask_4,
		compat_device_specific_status_5_mask_4,
		compat_device_specific_status_14_mask_4,
		compat_device_specific_status_15_mask_4,
		compat_device_specific_status_16_mask_4,
		compat_device_specific_status_17_mask_4,
		compat_device_specific_status_18_mask_4,
		compat_device_specific_status_19_mask_4,
		compat_device_specific_status_20_mask_4,
		compat_device_specific_status_21_mask_4,
		compat_device_specific_status_22_mask_4,
		compat_device_specific_status_23_mask_4,
		compat_device_specific_status_24_mask_4,

		//MENU DiagIdentification_menu
		current_date,//read only
		current_time,//read only
		transmitter_revision,//read only
		software_revision,//read only
		hardware_revision,//read only
		universal_revision,//read only

		//MENU DiagWirelessCommunication_menu
		mac_address_lower_bytes,//read only
		mac_address_higher_bytes,//read only
		device_nickname_address,//read only

		//MENU HART_menu
		config_change_counter,//read only
		device_status,//read only
		reboot_counter,//read only
		device_specific_status_0,//read only
		device_specific_status_1,//read only
		device_specific_status_2,//read only
		device_specific_status_3,//read only
		device_specific_status_4,//read only
		extended_fld_device_status,//read only
		standardized_status_0,//read only
		analog_channel_saturated1,//read only
		standardized_status_2,//read only
		standardized_status_3,//read only

		//MENU DeviceHealthState0
    IF (JOIN_STATUS_GEQ_128) {
			nickname_of_neighbor_0,//read only
			mean_rsl_0,//read only
			number_of_packets_transmitted_to_this_neighbor_0,//read only
			number_of_failed_transmits_0,//read only
			number_of_packets_received_from_this_neighbor_0,//read only
			nickname_of_neighbor_1,//read only
			mean_rsl_1,//read only
			number_of_packets_transmitted_to_this_neighbor_1,//read only
			number_of_failed_transmits_1,//read only
			number_of_packets_received_from_this_neighbor_1,//read only
			nickname_of_neighbor_2,//read only
			mean_rsl_2,//read only
			number_of_packets_transmitted_to_this_neighbor_2,//read only
			number_of_failed_transmits_2,//read only
			number_of_packets_received_from_this_neighbor_2,//read only
			nickname_of_neighbor_3,//read only
			mean_rsl_3,//read only
			number_of_packets_transmitted_to_this_neighbor_3,//read only
			number_of_failed_transmits_3,//read only
			number_of_packets_received_from_this_neighbor_3,//read only
			nickname_of_neighbor_4,//read only
			mean_rsl_4,//read only
			number_of_packets_transmitted_to_this_neighbor_4,//read only
			number_of_failed_transmits_4,//read only
			number_of_packets_received_from_this_neighbor_4,//read only
			nickname_of_neighbor_5,//read only
			mean_rsl_5,//read only
			number_of_packets_transmitted_to_this_neighbor_5,//read only
			number_of_failed_transmits_5,//read only
			number_of_packets_received_from_this_neighbor_5,//read only
			nickname_of_neighbor_6,//read only
			mean_rsl_6,//read only
			number_of_packets_transmitted_to_this_neighbor_6,//read only
			number_of_failed_transmits_6,//read only
			number_of_packets_received_from_this_neighbor_6,//read only
			nickname_of_neighbor_7,//read only
			mean_rsl_7,//read only
			number_of_packets_transmitted_to_this_neighbor_7,//read only
			number_of_failed_transmits_7,//read only
			number_of_packets_received_from_this_neighbor_7,//read only
			nickname_of_neighbor_8,//read only
			mean_rsl_8,//read only
			number_of_packets_transmitted_to_this_neighbor_8,//read only
			number_of_failed_transmits_8,//read only
			number_of_packets_received_from_this_neighbor_8//read only
		}
		//MENU Status_GROUP
		standard_150_device_status,//read only

		//MENU mA_4_20_PAGE
		transfer_function
	}
}

MENU upload_variables
{
	LABEL DCT_UPLOAD_MENU;
	ITEMS
	{
		//MENU Identification_menu
		descriptor,
		date,
		message,
		polling_address,
		country_code,
		si_control,

		//MENU DeviceVariableMapping_menu
		//primary_variable_code,
		secondary_variable_code,
		tertiary_variable_code,
		quaternary_variable_code,
		// Primary Unit
		//dev_var_current_Unit,
		// Secondary, Tertiary and Quaternary Units
		dev_var_temperature_Unit,
		dev_var_temperature_max_Unit,
		dev_var_temperature_min_Unit,

		//MENU root_menu
		loopCurrent,

		//MENU device_info
		tag,

		//MENU lock_unlock_device_menu
		lock_dust_flash_update,

		//MENU WirelessCommunication_menu
		//network_id,
		IF ( (country_code != en_country_code_Japan) && !(JOIN_STATUS_EQU_x079F_2)) {
			radio_power_output
		}
		join_shed_time,
		join_mode,

		//MENU WiredCommunication_menu
		master_mode,
		response_preambles,
		retry_count,
		lowest_scan_address,
		highest_scan_address,
		//start_scan,

		//MENU PowerSupply_menu
		power_mode,
		startup_voltage,
		startup_time,
		startup_current,
		operating_voltage,
		operating_time,
		power_on_time_after_external_trigger,
		configuration_time,
		//battery_changed,

		//MENU mA_4_20_menu
		upperRange_value,
		lowerRange_value,
		dev_var_current_damping_value,
		configure_lower_false_current_alarm,
		configure_upper_false_current_alarm,
		configure_upper_false_current_warning,
		configure_lower_false_current_warning,

		//MENU Linearization_menu
		table_index,
		x_value,
		y_value,

		//MENU Burst(parIndex)_menu
		compat_burst_mode_0,
		IF (compat_burst_subdev_index_0 != 0x7FFF) {compat_burst_subdev_index_0}
		compat_update_per_0,
		compat_max_update_per_0,
		compat_burst_trig_mode_0,
		compat_burst_trig_class_0,
		compat_burst_trig_units_0,
		compat_burst_trig_level_0,
		compat_burst_cmd_0,
		compat_burst_mode_1,
		IF (compat_burst_subdev_index_1 != 0x7FFF) {compat_burst_subdev_index_1}
		compat_update_per_1,
		compat_max_update_per_1,
		compat_burst_trig_mode_1,
		compat_burst_trig_class_1,
		compat_burst_trig_units_1,
		compat_burst_trig_level_1,
		compat_burst_cmd_1,
		compat_burst_mode_2,
		IF (compat_burst_subdev_index_2 != 0x7FFF) {compat_burst_subdev_index_2}
		compat_update_per_2,
		compat_max_update_per_2,
		compat_burst_trig_mode_2,
		compat_burst_trig_class_2,
		compat_burst_trig_units_2,
		compat_burst_trig_level_2,
		compat_burst_cmd_2,
		compat_burst_mode_3,
		IF (compat_burst_subdev_index_3 != 0x7FFF) {compat_burst_subdev_index_3}
		compat_update_per_3,
		compat_max_update_per_3,
		compat_burst_trig_mode_3,
		compat_burst_trig_class_3,
		compat_burst_trig_units_3,
		compat_burst_trig_level_3,
		compat_burst_cmd_3,
		compat_burst_mode_4,
		IF (compat_burst_subdev_index_4 != 0x7FFF) {compat_burst_subdev_index_4}
		compat_update_per_4,
		compat_max_update_per_4,
		compat_burst_trig_mode_4,
		compat_burst_trig_class_4,
		compat_burst_trig_units_4,
		compat_burst_trig_level_4,
		compat_burst_cmd_4,
		compat_burst_mode_5,
		IF (compat_burst_subdev_index_5 != 0x7FFF) {compat_burst_subdev_index_5}
		compat_update_per_5,
		compat_max_update_per_5,
		compat_burst_trig_mode_5,
		compat_burst_trig_class_5,
		compat_burst_trig_units_5,
		compat_burst_trig_level_5,
		compat_burst_cmd_5,
		compat_burst_mode_6,
		IF (compat_burst_subdev_index_6 != 0x7FFF) {compat_burst_subdev_index_6}
		compat_update_per_6,
		compat_max_update_per_6,
		compat_burst_trig_mode_6,
		compat_burst_trig_class_6,
		compat_burst_trig_units_6,
		compat_burst_trig_level_6,
		compat_burst_cmd_6,
		compat_burst_mode_7,
		IF (compat_burst_subdev_index_7 != 0x7FFF) {compat_burst_subdev_index_7}
		compat_update_per_7,
		compat_max_update_per_7,
		compat_burst_trig_mode_7,
		compat_burst_trig_class_7,
		compat_burst_trig_units_7,
		compat_burst_trig_level_7,
		compat_burst_cmd_7,
		compat_burst_mode_8,
		IF (compat_burst_subdev_index_8 != 0x7FFF) {compat_burst_subdev_index_8}
		compat_update_per_8,
		compat_max_update_per_8,
		compat_burst_trig_mode_8,
		compat_burst_trig_class_8,
		compat_burst_trig_units_8,
		compat_burst_trig_level_8,
		compat_burst_cmd_8,
		compat_burst_mode_9,
		IF (compat_burst_subdev_index_9 != 0x7FFF) {compat_burst_subdev_index_9}
		compat_update_per_9,
		compat_max_update_per_9,
		compat_burst_trig_mode_9,
		compat_burst_trig_class_9,
		compat_burst_trig_units_9,
		compat_burst_trig_level_9,
		compat_burst_cmd_9,
		compat_burst_vars_0_0,
		compat_burst_vars_0_1,
		compat_burst_vars_0_2,
		compat_burst_vars_0_3,
		compat_burst_vars_0_4,
		compat_burst_vars_0_5,
		compat_burst_vars_0_6,
		compat_burst_vars_0_7,
		compat_burst_vars_1_0,
		compat_burst_vars_1_1,
		compat_burst_vars_1_2,
		compat_burst_vars_1_3,
		compat_burst_vars_1_4,
		compat_burst_vars_1_5,
		compat_burst_vars_1_6,
		compat_burst_vars_1_7,
		compat_burst_vars_2_0,
		compat_burst_vars_2_1,
		compat_burst_vars_2_2,
		compat_burst_vars_2_3,
		compat_burst_vars_2_4,
		compat_burst_vars_2_5,
		compat_burst_vars_2_6,
		compat_burst_vars_2_7,
		compat_burst_vars_3_0,
		compat_burst_vars_3_1,
		compat_burst_vars_3_2,
		compat_burst_vars_3_3,
		compat_burst_vars_3_4,
		compat_burst_vars_3_5,
		compat_burst_vars_3_6,
		compat_burst_vars_3_7,
		compat_burst_vars_4_0,
		compat_burst_vars_4_1,
		compat_burst_vars_4_2,
		compat_burst_vars_4_3,
		compat_burst_vars_4_4,
		compat_burst_vars_4_5,
		compat_burst_vars_4_6,
		compat_burst_vars_4_7,
		compat_burst_vars_5_0,
		compat_burst_vars_5_1,
		compat_burst_vars_5_2,
		compat_burst_vars_5_3,
		compat_burst_vars_5_4,
		compat_burst_vars_5_5,
		compat_burst_vars_5_6,
		compat_burst_vars_5_7,
		compat_burst_vars_6_0,
		compat_burst_vars_6_1,
		compat_burst_vars_6_2,
		compat_burst_vars_6_3,
		compat_burst_vars_6_4,
		compat_burst_vars_6_5,
		compat_burst_vars_6_6,
		compat_burst_vars_6_7,
		compat_burst_vars_7_0,
		compat_burst_vars_7_1,
		compat_burst_vars_7_2,
		compat_burst_vars_7_3,
		compat_burst_vars_7_4,
		compat_burst_vars_7_5,
		compat_burst_vars_7_6,
		compat_burst_vars_7_7,
		compat_burst_vars_8_0,
		compat_burst_vars_8_1,
		compat_burst_vars_8_2,
		compat_burst_vars_8_3,
		compat_burst_vars_8_4,
		compat_burst_vars_8_5,
		compat_burst_vars_8_6,
		compat_burst_vars_8_7,
		compat_burst_vars_9_0,
		compat_burst_vars_9_1,
		compat_burst_vars_9_2,
		compat_burst_vars_9_3,
		compat_burst_vars_9_4,
		compat_burst_vars_9_5,
		compat_burst_vars_9_6,
		compat_burst_vars_9_7,

		//MENU EventNotification(parIndex)_Event_menu
		compat_event_notification_control_n_0,
		compat_event_notification_retry_time_event_0,
		compat_max_update_time_event_0,
		compat_event_debounce_interval_event_0,
		IF (compat_event_subdevice_index_0 != 0x7FFF) {compat_event_subdevice_index_0}
		compat_device_status_mask_0,
		compat_extended_fld_device_status_mask_0,
		compat_analog_channel_fixed1_mask_0,
		compat_analog_channel_saturated1_mask_0,
		compat_standardized_status_0_mask_0,
		compat_standardized_status_2_mask_0,
		compat_standardized_status_3_mask_0,
		compat_device_specific_status_0_mask_0,
		compat_device_specific_status_1_mask_0,
		compat_device_specific_status_2_mask_0,
		compat_device_specific_status_3_mask_0,
		compat_device_specific_status_4_mask_0,
		compat_device_specific_status_5_mask_0,
		compat_device_specific_status_14_mask_0,
		compat_device_specific_status_15_mask_0,
		compat_device_specific_status_16_mask_0,
		compat_device_specific_status_17_mask_0,
		compat_device_specific_status_18_mask_0,
		compat_device_specific_status_19_mask_0,
		compat_device_specific_status_20_mask_0,
		compat_device_specific_status_21_mask_0,
		compat_device_specific_status_22_mask_0,
		compat_device_specific_status_23_mask_0,
		compat_device_specific_status_24_mask_0,

		compat_event_notification_control_n_1,
		compat_event_notification_retry_time_event_1,
		compat_max_update_time_event_1,
		compat_event_debounce_interval_event_1,
		IF (compat_event_subdevice_index_1 != 0x7FFF) {compat_event_subdevice_index_1}
		compat_device_status_mask_1,
		compat_extended_fld_device_status_mask_1,
		compat_analog_channel_fixed1_mask_1,
		compat_analog_channel_saturated1_mask_1,
		compat_standardized_status_0_mask_1,
		compat_standardized_status_2_mask_1,
		compat_standardized_status_3_mask_1,
		compat_device_specific_status_0_mask_1,
		compat_device_specific_status_1_mask_1,
		compat_device_specific_status_2_mask_1,
		compat_device_specific_status_3_mask_1,
		compat_device_specific_status_4_mask_1,
		compat_device_specific_status_5_mask_1,
		compat_device_specific_status_14_mask_1,
		compat_device_specific_status_15_mask_1,
		compat_device_specific_status_16_mask_1,
		compat_device_specific_status_17_mask_1,
		compat_device_specific_status_18_mask_1,
		compat_device_specific_status_19_mask_1,
		compat_device_specific_status_20_mask_1,
		compat_device_specific_status_21_mask_1,
		compat_device_specific_status_22_mask_1,
		compat_device_specific_status_23_mask_1,
		compat_device_specific_status_24_mask_1,

		compat_event_notification_control_n_2,
		compat_event_notification_retry_time_event_2,
		compat_max_update_time_event_2,
		compat_event_debounce_interval_event_2,
		IF (compat_event_subdevice_index_2 != 0x7FFF) {compat_event_subdevice_index_2}
		compat_device_status_mask_2,
		compat_extended_fld_device_status_mask_2,
		compat_analog_channel_fixed1_mask_2,
		compat_analog_channel_saturated1_mask_2,
		compat_standardized_status_0_mask_2,
		compat_standardized_status_2_mask_2,
		compat_standardized_status_3_mask_2,
		compat_device_specific_status_0_mask_2,
		compat_device_specific_status_1_mask_2,
		compat_device_specific_status_2_mask_2,
		compat_device_specific_status_3_mask_2,
		compat_device_specific_status_4_mask_2,
		compat_device_specific_status_5_mask_2,
		compat_device_specific_status_14_mask_2,
		compat_device_specific_status_15_mask_2,
		compat_device_specific_status_16_mask_2,
		compat_device_specific_status_17_mask_2,
		compat_device_specific_status_18_mask_2,
		compat_device_specific_status_19_mask_2,
		compat_device_specific_status_20_mask_2,
		compat_device_specific_status_21_mask_2,
		compat_device_specific_status_22_mask_2,
		compat_device_specific_status_23_mask_2,
		compat_device_specific_status_24_mask_2,

		compat_event_notification_control_n_3,
		compat_event_notification_retry_time_event_3,
		compat_max_update_time_event_3,
		compat_event_debounce_interval_event_3,
		IF (compat_event_subdevice_index_3 != 0x7FFF) {compat_event_subdevice_index_3}
		compat_device_status_mask_3,
		compat_extended_fld_device_status_mask_3,
		compat_analog_channel_fixed1_mask_3,
		compat_analog_channel_saturated1_mask_3,
		compat_standardized_status_0_mask_3,
		compat_standardized_status_2_mask_3,
		compat_standardized_status_3_mask_3,
		compat_device_specific_status_0_mask_3,
		compat_device_specific_status_1_mask_3,
		compat_device_specific_status_2_mask_3,
		compat_device_specific_status_3_mask_3,
		compat_device_specific_status_4_mask_3,
		compat_device_specific_status_5_mask_3,
		compat_device_specific_status_14_mask_3,
		compat_device_specific_status_15_mask_3,
		compat_device_specific_status_16_mask_3,
		compat_device_specific_status_17_mask_3,
		compat_device_specific_status_18_mask_3,
		compat_device_specific_status_19_mask_3,
		compat_device_specific_status_20_mask_3,
		compat_device_specific_status_21_mask_3,
		compat_device_specific_status_22_mask_3,
		compat_device_specific_status_23_mask_3,
		compat_device_specific_status_24_mask_3,

		compat_event_notification_control_n_4,
		compat_event_notification_retry_time_event_4,
		compat_max_update_time_event_4,
		compat_event_debounce_interval_event_4,
		IF (compat_event_subdevice_index_4 != 0x7FFF) {compat_event_subdevice_index_4}
		compat_device_status_mask_4,
		compat_extended_fld_device_status_mask_4,
		compat_analog_channel_fixed1_mask_4,
		compat_analog_channel_saturated1_mask_4,
		compat_standardized_status_0_mask_4,
		compat_standardized_status_2_mask_4,
		compat_standardized_status_3_mask_4,
		compat_device_specific_status_0_mask_4,
		compat_device_specific_status_1_mask_4,
		compat_device_specific_status_2_mask_4,
		compat_device_specific_status_3_mask_4,
		compat_device_specific_status_4_mask_4,
		compat_device_specific_status_5_mask_4,
		compat_device_specific_status_14_mask_4,
		compat_device_specific_status_15_mask_4,
		compat_device_specific_status_16_mask_4,
		compat_device_specific_status_17_mask_4,
		compat_device_specific_status_18_mask_4,
		compat_device_specific_status_19_mask_4,
		compat_device_specific_status_20_mask_4,
		compat_device_specific_status_21_mask_4,
		compat_device_specific_status_22_mask_4,
		compat_device_specific_status_23_mask_4,
		compat_device_specific_status_24_mask_4,

		//MENU mA_4_20_PAGE
		transfer_function
	}
	PRE_WRITE_ACTIONS {
		method_read_join_status
	}
	POST_WRITE_ACTIONS {
		method_burst_mode_and_event_notification_sync_upload
	}
}
#endif


#ifdef _PDM_
COMMAND read_join_status
				{
				    NUMBER 31;
				    OPERATION READ;
				    TRANSACTION 1
				    {
				        REQUEST
				        {
				        	0x03, 0x01
				        }
				        REPLY
				        {
				            0x03, 0x01,
				            response_code, device_status,
				            //dummy_16_bit_command,
				            reserved_bits<0xF0>, wireless_mode<0x0F>,
				            join_status_byte_hi,
				            join_status_byte_lo,
				            neighbor_count,
				            advertisements,
				            join_attempts,
				            join_retry_timer,
				            network_search_timer
				        }
				    }
				    TRANSACTION 2
				    {
				        REQUEST
				        {
				        	0x03, 0x01
				        }
				        REPLY
				        {
				            0x03, 0x01,
				            response_code, device_status,
				            //dummy_16_bit_command,
				            0x00,
				            join_status_byte_hi_2,
				            join_status_byte_lo_2,
				            0x00,
				            0x00,
				            0x00,
				            0x00, 0x00, 0x00, 0x00,
				            0x00, 0x00, 0x00, 0x00
				        }
				    }
				    RESPONSE_CODES
				    {
				        0,  SUCCESS,    [no_command_specific_errors];
				        6,  MISC_ERROR, [xmtr_specific_command_error];
				        32, MODE_ERROR, [busy];
				    }
				}
#endif

#ifdef _PDM_
join_status_byte_hi_2 LIKE VARIABLE join_status_byte_hi {}
join_status_byte_lo_2 LIKE VARIABLE join_status_byte_lo {}
#endif

METHOD method_read_join_status
{
#ifndef _PDM_
    VALIDITY FALSE;
#endif
    LABEL "Read Join status upfront the download";
    DEFINITION
		{
			char status[STATUS_SIZE];

			send_command_trans(31,2);

			save_values();
    }
}

METHOD method_battery_changed
{
	LABEL DCT_METHOD_LABEL_BATTERY_CHANGED;
	DEFINITION
	{
			char status[STATUS_SIZE];
			int sel;
			long var_id[4];
			char disp_string2[60];
			int slen2;

			slen2=60;

			send(76, status);
			if((lvar_value(lock_device_status_code) & 1) != 0) // check device is locked
			{
				ACKNOWLEDGE(DCT_DEVICE_IS_LOCKED);
				process_abort();
			}

			get_dictionary_string(yes_no, disp_string2, slen2);
			sel=select_from_list(DCT_METHOD_QUERY_CONFIRM_BATTERY_CHANGED, var_id, disp_string2);
			if (sel == 0) /* 0 == yes == execute reset */
			{
				 iassign(battery_changed,1);
				 iassign(variable_array_index, 21);
         send_command(173);//write the battery_changed trigger to the device
         iassign(battery_changed, 0);
         iassign(variable_array_index, 19);  //19: last_time_battery_changed
         send_command(172); //read back the last_time_battery_changed parameter.
			}
			save_values();
	}
}

METHOD method_burst_mode_sync_offline
{
	LABEL DCT_METHOD_BURST_MODE_SYNC_OFFLINE;
#ifndef _PDM_
    VALIDITY FALSE;
#endif
    DEFINITION
	{

			vassign(compat_burst_mode_selection_0,compat_burst_mode_local_0);
			vassign(compat_burst_mode_selection_1,compat_burst_mode_local_1);
			vassign(compat_burst_mode_selection_2,compat_burst_mode_local_2);
			vassign(compat_burst_mode_selection_3,compat_burst_mode_local_3);
			vassign(compat_burst_mode_selection_4,compat_burst_mode_local_4);
			vassign(compat_burst_mode_selection_5,compat_burst_mode_local_5);
			vassign(compat_burst_mode_selection_6,compat_burst_mode_local_6);
			vassign(compat_burst_mode_selection_7,compat_burst_mode_local_7);
			vassign(compat_burst_mode_selection_8,compat_burst_mode_local_8);
			vassign(compat_burst_mode_selection_9,compat_burst_mode_local_9);
			
			vassign(compat_burst_messages[0].BURST_MODE_SELECT,compat_burst_mode_local_0);
			vassign(compat_burst_messages[1].BURST_MODE_SELECT,compat_burst_mode_local_1);
			vassign(compat_burst_messages[2].BURST_MODE_SELECT,compat_burst_mode_local_2);
			vassign(compat_burst_messages[3].BURST_MODE_SELECT,compat_burst_mode_local_3);
			vassign(compat_burst_messages[4].BURST_MODE_SELECT,compat_burst_mode_local_4);
			vassign(compat_burst_messages[5].BURST_MODE_SELECT,compat_burst_mode_local_5);
			vassign(compat_burst_messages[6].BURST_MODE_SELECT,compat_burst_mode_local_6);
			vassign(compat_burst_messages[7].BURST_MODE_SELECT,compat_burst_mode_local_7);
			vassign(compat_burst_messages[8].BURST_MODE_SELECT,compat_burst_mode_local_8);
			vassign(compat_burst_messages[9].BURST_MODE_SELECT,compat_burst_mode_local_9);
			
			save_values();
	}
}

METHOD method_burst_mode_force_refresh
{
    LABEL DCT_METHOD_BURST_MODE_FORCE_REFRESH;
    DEFINITION
	{
			IGNORE_ALL_RESPONSE_CODES();
			XMTR_IGNORE_ALL_RESPONSE_CODES();
			send_command_trans(172,15);
#ifndef _PDM_
            send_command_trans(172,17);
#endif
			save_values();
	}
}


METHOD method_burst_mode_and_event_notification_sync_upload
{
	LABEL DCT_METHOD_BURST_MODE_AND_EVENT_NOTIFICATION_SYNC_UPLOAD;
#ifndef _PDM_
    VALIDITY FALSE;
#endif
    DEFINITION
	{
			// Burst Mode Control
			vassign(compat_burst_mode_local_0,compat_burst_mode_selection_0);
			vassign(compat_burst_mode_local_1,compat_burst_mode_selection_1);
			vassign(compat_burst_mode_local_2,compat_burst_mode_selection_2);
			vassign(compat_burst_mode_local_3,compat_burst_mode_selection_3);
			vassign(compat_burst_mode_local_4,compat_burst_mode_selection_4);
			vassign(compat_burst_mode_local_5,compat_burst_mode_selection_5);
			vassign(compat_burst_mode_local_6,compat_burst_mode_selection_6);
			vassign(compat_burst_mode_local_7,compat_burst_mode_selection_7);
			vassign(compat_burst_mode_local_8,compat_burst_mode_selection_8);
			vassign(compat_burst_mode_local_9,compat_burst_mode_selection_9);
			// Event Notification Control
			vassign(compat_event_notification_control_n_local_0,compat_event_notification_control_n_overview_0);
			vassign(compat_event_notification_control_n_local_1,compat_event_notification_control_n_overview_1);
			vassign(compat_event_notification_control_n_local_2,compat_event_notification_control_n_overview_2);
			vassign(compat_event_notification_control_n_local_3,compat_event_notification_control_n_overview_3);
			vassign(compat_event_notification_control_n_local_4,compat_event_notification_control_n_overview_4);
			
			save_values();
	}
}

METHOD method_event_notification_control_sync_offline
{
	LABEL DCT_METHOD_EVENT_NOTIFICATION_CONTROL_SYNC_OFFLINE ;
#ifndef _PDM_
    VALIDITY FALSE;
#endif
	DEFINITION
	{
			vassign(compat_event_notification_control_n_overview_0,compat_event_notification_control_n_local_0);
			vassign(compat_event_notification_control_n_overview_1,compat_event_notification_control_n_local_1);
			vassign(compat_event_notification_control_n_overview_2,compat_event_notification_control_n_local_2);
			vassign(compat_event_notification_control_n_overview_3,compat_event_notification_control_n_local_3);
			vassign(compat_event_notification_control_n_overview_4,compat_event_notification_control_n_local_4);

			vassign(compat_events[0].EVENT_NOTIFICATION_CONTROL,compat_event_notification_control_n_local_0);
			vassign(compat_events[1].EVENT_NOTIFICATION_CONTROL,compat_event_notification_control_n_local_1);
			vassign(compat_events[2].EVENT_NOTIFICATION_CONTROL,compat_event_notification_control_n_local_2);
			vassign(compat_events[3].EVENT_NOTIFICATION_CONTROL,compat_event_notification_control_n_local_3);
			vassign(compat_events[4].EVENT_NOTIFICATION_CONTROL,compat_event_notification_control_n_local_4);
			
			save_values();
	}
}

METHOD method_event_notification_force_refresh
{
	LABEL DCT_METHOD_EVENT_NOTIFICATION_FORCE_REFRESH;
    DEFINITION
	{
			IGNORE_ALL_RESPONSE_CODES();
			XMTR_IGNORE_ALL_RESPONSE_CODES();
			send_command_trans(172,16);
#ifndef _PDM_
            send_command_trans(172,18);
#endif
			save_values();
	}
}

#define BURSTMODECONTROL_REFRESH(num) REFRESH burst_mode_control_rfsh_##num\
{\
	compat_burst_mode_##num:\
		compat_burst_mode_selection_##num \
}

BURSTMODECONTROL_REFRESH(0)
BURSTMODECONTROL_REFRESH(1)
BURSTMODECONTROL_REFRESH(2)
BURSTMODECONTROL_REFRESH(3)
BURSTMODECONTROL_REFRESH(4)
BURSTMODECONTROL_REFRESH(5)
BURSTMODECONTROL_REFRESH(6)
BURSTMODECONTROL_REFRESH(7)
BURSTMODECONTROL_REFRESH(8)
BURSTMODECONTROL_REFRESH(9)

#define EVENTNOTIFICATIONCONTROL_REFRESH(num) REFRESH event_notification_control_rfsh_##num\
{\
	compat_event_notification_control_n_##num :\
		compat_event_notification_control_n_overview_##num \
}

EVENTNOTIFICATIONCONTROL_REFRESH(0)
EVENTNOTIFICATIONCONTROL_REFRESH(1)
EVENTNOTIFICATIONCONTROL_REFRESH(2)
EVENTNOTIFICATIONCONTROL_REFRESH(3)
EVENTNOTIFICATIONCONTROL_REFRESH(4)

VARIABLE overview_validity
{
    LABEL [command_specific_response_code];
    HELP [command_specific_response_code_help];
    CLASS HART & DIAGNOSTIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
    DEFAULT_VALUE 0;
}
